#include "joynet/joynet.h"

#define VGOLF_STATE_INTRO           0
#define VGOLF_STATE_MENU            1
#define VGOLF_STATE_SCORE_CARD      2

#define VGOLF_STATE_PLAY            3

	#define VGOLF_PLAY_STATE_SCORES 0
	#define VGOLF_PLAY_STATE_GAME   1

#define VGOLF_STATE_PLAYER_SETUP    4
#define VGOLF_STATE_PLAYER_ACCOUNT  5
#define VGOLF_STATE_PLAYER_AVATAR   6
#define VGOLF_STATE_PLAYER_NEW      7
#define VGOLF_STATE_COURSE          8
#define VGOLF_STATE_CREDITS        10
#define VGOLF_STATE_PAUSE          11
#define VGOLF_STATE_PROFILE        13
#define VGOLF_STATE_PROFILE_DELETE 14
#define VGOLF_STATE_LEADERBOARDS   15
#define VGOLF_STATE_DLC_AVATAR     16
#define VGOLF_STATE_DLC            17
#define VGOLF_STATE_SUMMARY        20
#define VGOLF_STATE_STARTING_GAME  21
#define VGOLF_STATE_PRIVACY        22
#define VGOLF_STATE_LOAD           86

#define VGOLF_STATE_FIN       99
#define VGOLF_STATE_NONE          256

#define VGOLF_COLOR_MENU_HEADER al_map_rgb(0, 255, 0)
#define VGOLF_COLOR_MENU_OPTION al_map_rgb(255, 255, 255)
#define VGOLF_COLOR_MENU_OPTION_DARK al_map_rgb(192, 192, 192)
#define VGOLF_COLOR_MENU_MAIN al_map_rgb(0, 255, 0)
#define VGOLF_COLOR_MENU_MAIN_DARK al_map_rgb(0, 192, 0)

#define MAX_RANGE 10000

#define VGOLF_MAX_COURSES               32

#define CREDITS_SPACE 360

#define MAX_IMAGES 256
#define MAX_SOUNDS 256

#define VGOLF_MAX_FONTS 16
#define VGOLF_FONT_96    0
#define VGOLF_FONT_64    1
#define VGOLF_FONT_48    2
#define VGOLF_FONT_32    3

#ifdef T3F_ANDROID
	#define VGOLF_FONT_CHAT_OVERLAY VGOLF_FONT_64
#else
	#define VGOLF_FONT_CHAT_OVERLAY VGOLF_FONT_32
#endif

#define CONFIG_MAX               32
#define CONFIG_GFX_MODE           0
#define CONFIG_SFX_VOLUME         1
#define CONFIG_MUSIC_VOLUME       2
#define CONFIG_COMMENTARY         3
#define CONFIG_COMMENTARY_VOLUME  4
#define CONFIG_UPLOAD             5
#define CONFIG_INPUT_BUFFER       6
#define CONFIG_DLC                7
#define CONFIG_LOW_BANDWIDTH      8
#define CONFIG_HD_TEXTURES        9
#define CONFIG_DEAD_SPOT         10

/* sound defines */
#define SOUND_SWING0          0
#define SOUND_SWING1          1
#define SOUND_SWING2          2
#define SOUND_BALLCOLLISION0  3
#define SOUND_BALLCOLLISION1  4
#define SOUND_BALLCOLLISION2  5
#define SOUND_HOLESINK        6
#define SOUND_HOLEFINISH00    7
#define SOUND_HOLEFINISH01    8
#define SOUND_HOLEFINISH02    9
#define SOUND_HOLEFINISH03   10
#define SOUND_HOLEFINISH10   11
#define SOUND_HOLEFINISH11   12
#define SOUND_HOLEFINISH12   13
#define SOUND_HOLEFINISH13   14
#define SOUND_HOLEFINISH20   15
#define SOUND_HOLEFINISH21   16
#define SOUND_HOLEFINISH22   17
#define SOUND_HOLEFINISH23   18
#define SOUND_AHH0           19
#define SOUND_AHH1           20
#define SOUND_BALLSPLASH     21
#define SOUND_BALLROTATE     22
#define SOUND_MENUSELECT     23
#define SOUND_MENUESCAPE     24
#define SOUND_MENUSLIDER     25
#define SOUND_MENUITEMSEL    26
#define SOUND_COIN           27
#define SOUND_BALLZAPPED     28
#define SOUND_BALLTELEPORT   29
#define SOUND_TITLE_SWING    30

#define MAX_ANIMDEF_FRAMES 6

#define MAX_BALL_ANIM_MODES 4

#define MAX_PLAYER_COURSES_COMPLETED 16
#define MAX_PLAYER_SAVED_STATS 16

#define MAX_COURSE_TOPSCORES 3

#define MAX_TRAJECTORY_POINTS 24

#define GAMEMODE_STROKE_PLAY 0

#define GAMESTATE_START_HOLE 0
#define GAMESTATE_READY_TOPLAY 1
#define GAMESTATE_HOLE_FINISHED 2
#define GAMESTATE_SCOREBOARD_SEGIN 3
#define GAMESTATE_SCOREBOARD_SEGMID 4
#define GAMESTATE_SCOREBOARD_SEGOUT 5
#define GAMESTATE_QUIT_PROMPT 6
#define GAMESTATE_COURSE_COMPLETED_1P 7
#define GAMESTATE_COURSE_COMPLETED_XP 8
#define GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN 9
#define GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN 10
#define GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID 11
#define GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID 12
#define GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGOUT 13
#define GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGOUT 14
#define GAMESTATE_COIN_COLLECT_FINISHED 15
#define GAMESTATE_COIN_COLLECT_FAILED_STROKES 16
#define GAMESTATE_COIN_COLLECT_FAILED_TIME 17
#define GAMESTATE_PLAYER_FINISHED_HOLE 18
#define GAMESTATE_BALL_LOST      19
#define GAMESTATE_OFFER_GIMME    20

#define TITLE_SONG_FILENAME "vgolf.xm"
#define CONFIG_FILE_FILENAME "vgolf.cfg"
#define MESSAGE_FILE_FILENAME "vgolf.msg"

//#define MAX_CLUB_POWER 10
#define MAX_CLUB_POWER 100

#define BALL_MODE_IDLE 0
#define BALL_MODE_HIT 1
#define BALL_MODE_SUNK 2
#define BALL_MODE_WATER_SINKING 3
#define BALL_MODE_WATER_SUNK 4
#define BALL_MODE_PIPE_ENTER 5
#define BALL_MODE_PIPE_EXIT 6
#define BALL_MODE_ON_INDENT 7
#define BALL_MODE_IN_POLYMORPH 8
#define BALL_MODE_DROP 9
#define BALL_MODE_IN_MOTION_AREA 10
#define BALL_MODE_COIN_FINISHED 12
#define BALL_MODE_LASER_ZAPPING 13
#define BALL_MODE_LASER_ZAPPED 14
#define BALL_MODE_ON_CONVEYOR 15
#define BALL_MODE_DOOR_SMASHED 16

#define BALL_STD_RADIUS 8.0

#define BALL_COLLISION_FACTOR 0.5

#define BALL_MAX_HOLEHIT_SPEED 4.95

#define CLUB_STD_LENGTH 40

#define MAX_CONTROLLER_TYPES 5
#define CONTROLLER_KEYBOARD_A 0
#define CONTROLLER_KEYBOARD_B 1
#define CONTROLLER_KEYBOARD_C 2
#define CONTROLLER_KEYBOARD_D 3
#define CONTROLLER_KEYBOARD_X 5
#define CONTROLLER_MOUSE 4
#define CONTROLLER_JOYSTICK_1 6
#define CONTROLLER_JOYSTICK_2 7
#define CONTROLLER_JOYSTICK_3 8
#define CONTROLLER_JOYSTICK_4 9

#define GAME_VMESSAGE_QUEUE_SIZE 4

#define FX_SHADOWS 1
#define FX_RESET_HUDS 2

#define MAX_PACKFILES 16

#define MAX_GAME_CHARS 24

#define MAX_SFX_SWING_SAMPLES 3
#define MAX_SFX_COLLISION_SAMPLES 3
#define MAX_SFX_HOLE_FINISH_SAMPLES 4
#define MAX_SFX_HOLE_AHHH_SAMPLES 2

#define MAX_CONVEYOR_SPEED 8

#define COIN_COMPLETE_BONUS 1000

#ifndef random
#define random() rand()
#endif

#ifndef srandom
#define srandom(int) srand(int)
#endif

#define VGOLF_GAME_MESSAGE_TYPING        JOYNET_GAME_MESSAGE_USER
#define VGOLF_GAME_MESSAGE_TYPING_DONE   JOYNET_GAME_MESSAGE_USER + 1
#define VGOLF_GAME_MESSAGE_WANT_START    JOYNET_GAME_MESSAGE_USER + 2
#define VGOLF_GAME_MESSAGE_CANCEL_START  JOYNET_GAME_MESSAGE_USER + 3
#define VGOLF_GAME_MESSAGE_STARTING      JOYNET_GAME_MESSAGE_USER + 4

#define VGOLF_CONTENT_AVATARS      0
#define VGOLF_CONTENT_COURSES      1
#define VGOLF_CONTENT_COMMENTARIES 2

#define BALL_COLOR al_map_rgb(255, 255, 255)
#define BALL_SHADOW_COLOR al_map_rgba_f(0.0, 0.0, 0.0, 0.5)
#define BALL_MIRAGE_COLOR al_map_rgba_f(0.5, 0.5, 0.5, 0.5)
#define DEFAULT_CLUB_COLOR al_map_rgb(255, 2, 0)
#define CLUB_FILL_COLOR al_map_rgb(255, 255, 255)
#define CLUB_LINE_COLOR al_map_rgb(170, 170, 170)

#define VGOLF_MAX_PLAYERS 8
#define VGOLF_PLAYER_SETUP_START   0
#define VGOLF_PLAYER_SETUP_COURSE  1
#define VGOLF_PLAYER_SETUP_SUMMARY 2
