#include "globals.h"

static double vgolf_camera_bound_left;
static double vgolf_camera_bound_right;
static double vgolf_camera_bound_top;
static double vgolf_camera_bound_bottom;
static double vgolf_camera_bound_ratio;

void vgolf_reset_camera(void)
{
	main_game_vars.camera.target_x = 0.0;
	main_game_vars.camera.target_y = 0.0;
	main_game_vars.camera.target_z = 0.0;
	main_game_vars.camera.vx = 0.0;
	main_game_vars.camera.vy = 0.0;
	main_game_vars.camera.vz = 0.0;
	main_game_vars.camera.x = 0.0;
	main_game_vars.camera.y = 0.0;
	main_game_vars.camera.z = 0.0;
}

void vgolf_get_camera_boundaries(void)
{
	vgolf_camera_bound_ratio = 480.0 / 640.0;
	
	/* uncomment when we finish implementing FILL_SCREEN 
	vgolf_camera_bound_ratio = (float)al_get_display_height(t3f_display) / (float)al_get_display_width(t3f_display); */
	vgolf_camera_bound_left = -main_game_vars.camera.z / 2.0;
	vgolf_camera_bound_right = main_game_vars.camera.z / 2.0;
	vgolf_camera_bound_top = (-main_game_vars.camera.z / 2.0) * vgolf_camera_bound_ratio;
	vgolf_camera_bound_bottom = (main_game_vars.camera.z / 2.0) * vgolf_camera_bound_ratio;
}

void vgolf_aim_camera(float x, float y)
{
	main_game_vars.camera.target_x = x - 320.0;
	main_game_vars.camera.target_y = y - 240.0;
}

/* force camera to stay within boundaries */
void vgolf_restrict_camera(void)
{
	if(main_game_vars.camera.x < vgolf_camera_bound_left)
	{
		main_game_vars.camera.x = vgolf_camera_bound_left;
	}
	else if(main_game_vars.camera.x > vgolf_camera_bound_right)
	{
		main_game_vars.camera.x = vgolf_camera_bound_right;
	}
	if(main_game_vars.camera.y < vgolf_camera_bound_top)
	{
		main_game_vars.camera.y = vgolf_camera_bound_top;
	}
	else if(main_game_vars.camera.y > vgolf_camera_bound_bottom)
	{
		main_game_vars.camera.y = vgolf_camera_bound_bottom;
	}
}

void vgolf_restrict_camera_target(void)
{
	if(main_game_vars.camera.target_x < vgolf_camera_bound_left)
	{
		main_game_vars.camera.target_x = vgolf_camera_bound_left;
	}
	else if(main_game_vars.camera.target_x > vgolf_camera_bound_right)
	{
		main_game_vars.camera.target_x = vgolf_camera_bound_right;
	}
	if(main_game_vars.camera.target_y < vgolf_camera_bound_top)
	{
		main_game_vars.camera.target_y = vgolf_camera_bound_top;
	}
	else if(main_game_vars.camera.target_y > vgolf_camera_bound_bottom)
	{
		main_game_vars.camera.target_y = vgolf_camera_bound_bottom;
	}
}

/* returns true if we have reached our target */
bool vgolf_camera_coordinate_logic(float * cur, float * target)
{
	float d = *target - *cur;
	
	if(d > 1.0 && d < 1.0)
	{
		*cur = *target;
		return true;
	}
	else if(*cur < *target)
	{
		*cur += d / 4.0;
		if(*cur > *target)
		{
			*cur = *target;
			return true;
		}
	}
	else if(*cur > *target)
	{
		*cur += d / 4.0;
		if(*cur < *target)
		{
			*cur = *target;
			return true;
		}
	}
	return false;
}

void vgolf_camera_toggle_zoom(void)
{
	if(main_game_vars.camera.target_z <= 0.0)
	{
		main_game_vars.camera.target_z = 240.0;
	}
	else
	{
		main_game_vars.camera.target_z = 0.0;
	}
}

void vgolf_camera_logic(void)
{
	BALL * ball = &game_player[main_game_vars.cur_player].ball;
	if(t3f_key[ALLEGRO_KEY_Z])
	{
		vgolf_camera_toggle_zoom();
		t3f_key[ALLEGRO_KEY_Z] = 0;
	}
	
	/* adjust z-coordinate before calculating boundaries since they depend on an
	 * up-to-date z-coordinate to function properly */
	if(vgolf_camera_coordinate_logic(&main_game_vars.camera.z, &main_game_vars.camera.target_z))
	{
		vgolf_get_camera_boundaries();
		vgolf_restrict_camera_target();
	}
	else
	{
		vgolf_get_camera_boundaries();
	}
	if(ball->mode == BALL_MODE_HIT || ball->mode == BALL_MODE_IN_MOTION_AREA || ball->mode == BALL_MODE_ON_CONVEYOR || ball->mode == BALL_MODE_PIPE_ENTER || ball->mode == BALL_MODE_SUNK)
	{
		vgolf_aim_camera(ball->s.x, ball->s.y);
	}
	vgolf_camera_coordinate_logic(&main_game_vars.camera.x, &main_game_vars.camera.target_x);
	vgolf_camera_coordinate_logic(&main_game_vars.camera.y, &main_game_vars.camera.target_y);
	vgolf_restrict_camera();
}
