#include "t3f/t3f.h"
#include "t3f/view.h"
#include "t3f/draw.h"
#include "globals.h"
#include "fx.h"
#include "vfont.h"
#include "structs.h"
#include "core/course.h"
#include "core/utility.h"
#include "menu_proc.h"
#include "replay.h"
#include "game_camera.h"

#include "core/vert.h"
#include "gameplay_hud.h"
#include "gameplay_scoreboard.h"
#include "gameplay_player.h"
#include "gameplay_message.h"
#include "gameplay_ball.h"
#include "gameplay_results.h"

//get controller input from exiting, pausing, etc.)
void get_main_keys(void)
{
	if(t3f_key[ALLEGRO_KEY_TILDE] && debug_mode)
	{  //toggle debug mode
		main_game_vars.debug_on = !main_game_vars.debug_on;
		t3f_key[ALLEGRO_KEY_TILDE] = 0;
	}
}

//hole logic..
void hole_logic(HOLE * hole)
{
	int v;

	for(v = 0; v < hole->num_verts; v++)
	{
		vert_logic(&hole->vert[v], hole, &game_course->image_bank);
	}
}

//get main in-game input (for exlayer whose ball is furthest from the hole
/*int furthest_player_from_hole(PLAYER pl[], int num_players, HOLE * hole)
{
	int p;
	int hole_vert = 0;
	int nearest = -1;

	for(p = 0; p < VGOLF_MAX_PLAYERS; p++)
	{
		if(vgolf_player_list[p])
		{
			if(pl[p].ball.mode != BALL_MODE_SUNK)
			{
				nearest = p;
				break;
			}
		}
	}
	if(nearest == -1) return nearest;

	for(p = 0; p < hole->num_verts; p++)
	{
		if(hole->vert[p].flag == VERT_FLAG_HOLE)
		{
			hole_vert = p;
			break;
		}
	}
	for(p = 1; p < 4; p++)
	{
		if(vgolf_player_list[p])
		{
			if(v_distance(pl[p].ball.s, v_offset(hole->s, hole->vert[hole_vert].s)) > v_distance(pl[nearest].ball.s, v_offset(hole->s, hole->vert[hole_vert].s)) && pl[p].ball.mode != BALL_MODE_SUNK)
			{
				nearest = p;
			}
		}
	}
	return nearest;
} */

/* returns which player is next
   if all players done, return -1 */
int vgolf_get_next_player(void)
{
	int p = main_game_vars.cur_player;

	while(1)
	{
		p++;
		if(p >= VGOLF_MAX_PLAYERS)
		{
			p = 0;
		}
		if(p == main_game_vars.cur_player)
		{
			return -1;
		}
		if(vgolf_player_list[p] && game_player[p].ball.mode != BALL_MODE_SUNK)
		{
			return p;
		}
	}
	return -1;
}

/* return which player got the best score */
int vgolf_get_best_player(void)
{
	int best = -1;
	int best_score = 10000;
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			if(game_player[i].cur_strokes < best_score)
			{
				best = i;
				best_score = game_player[i].cur_strokes;
			}
		}
	}

	return best;
}

void build_turn_text(int p)
{
	int r = vgolf_rand() % 4;

	switch(r)
	{
		case 0:
		{
			sprintf(temp_text, "You're up, %s!", vgolf_client_game->player[p]->name);
			break;
		}
		case 1:
		{
			sprintf(temp_text, "%s, it's your turn.", vgolf_client_game->player[p]->name);
			break;
		}
		case 2:
		{
			sprintf(temp_text, "Take your shot, %s!", vgolf_client_game->player[p]->name);
			break;
		}
		case 3:
		{
			sprintf(temp_text, "It's your turn, %s.", vgolf_client_game->player[p]->name);
			break;
		}
	}
}

//finalize the match
void do_match_finish(HOLE * hole, int cur_hole, PLAYER pl[], int num_players)
{
	int i;
	V_INT p_no, p_total; //p_no contains the player numbers from 0 to num_players
                             //p_total contains the stroke total for each player in p_no
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			p_no.c[i] = i;
			p_total.c[i] = pl[i].cur_strokes;
		}
	}

	//sort from smallest to largest score
	v_int_heap_sort_ref(&p_total, &p_no, num_players);
	//p_no should now be sorted from the player with the best score to that with the worst

/*	//if there are only 2 players, the sort is simple
	if(num_players == 2)
	{
		if(p_total.c[0] < p_total.c[1])
		{
			//the player with least score is tagged the winner
			pl[p_no.c[0]].scorecard.match_won[cur_hole] = 1;
			//other players are tagged losers
			pl[p_no.c[1]].scorecard.match_won[cur_hole] = 0;
			//winning player's win total increments
			pl[p_no.c[0]].scorecard.matches_won++;
		}
		else
		{
			//players are tied
			pl[p_no.c[0]].scorecard.match_won[cur_hole] = pl[p_no.c[1]].scorecard.match_won[cur_hole] = 2;
		}
	}
	else if(num_players == 3)
	{
		//players are tied
		if(p_total.c[0] == p_total.c[1])
		{
			pl[p_no.c[0]].scorecard.match_won[cur_hole] = pl[p_no.c[1]].scorecard.match_won[cur_hole] = pl[p_no.c[2]].scorecard.match_won[cur_hole] = 2;
		}
		else
		{
			//the player with least score is tagged the winner
			pl[p_no.c[0]].scorecard.match_won[cur_hole] = 1;
			//other players are tagged losers
			pl[p_no.c[1]].scorecard.match_won[cur_hole] = pl[p_no.c[2]].scorecard.match_won[cur_hole] = 0;
			//winning player's win total increments
			pl[p_no.c[0]].scorecard.matches_won++;
		}
	}
	else
	{
		//players are tied
		if(p_total.c[0] == p_total.c[1])
		{
			//all players are tagged as tied
			pl[0].scorecard.match_won[cur_hole] = pl[1].scorecard.match_won[cur_hole] = pl[2].scorecard.match_won[cur_hole] = pl[3].scorecard.match_won[cur_hole] = 2;
		}
		else
		{
			//the player with least score is tagged the winner
			pl[p_no.c[0]].scorecard.match_won[cur_hole] = 1;
			//other players are tagged losers
			pl[p_no.c[1]].scorecard.match_won[cur_hole] = pl[p_no.c[2]].scorecard.match_won[cur_hole] = pl[p_no.c[3]].scorecard.match_won[cur_hole] = 0;
			//winning player's win total increments
			pl[p_no.c[0]].scorecard.matches_won++;
		}
	} */
}

int vgolf_remaining_players(void)
{
	int i, c = 0;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			c++;
		}
	}
	return c;
}

bool vgolf_remaining_players_done(void)
{
	int i, c = 0;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && game_player[i].ball.mode != BALL_MODE_SUNK)
		{
			c++;
		}
	}
	if(c > 0)
	{
		return false;
	}
	return true;
}

//the main logic stuff
void vgolf_game_logic_tick(void)
{
	int i, p;
	bool removed_player = false;

	vgolf_hidden_layer = -1;

	/* handle pending player removals */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_remove_player[i] && vgolf_remove_player_frame[i] == main_game_vars.tick)
		{
			if(vgolf_player_list[i])
			{
				vgolf_player_list[i] = 0;
				player_page_pointer[i] = player_start_page[i];
				if(i == main_game_vars.cur_player)
				{
					p = vgolf_get_next_player();
					if(main_game_vars.cur_player >= 0)
					{
						vgolf_select_player(p);
					}
					else
					{
						joynet_end_game(vgolf_client_game);
					}
				}
				vgolf_remove_player[i] = 0;
				removed_player = true;
			}
		}
	}

	if(removed_player)
	{
		/* exit the game if no players left */
		if(vgolf_remaining_players() == 0)
		{
			vgolf_state = VGOLF_STATE_MENU;
			if(vgolf_hosting)
			{
				current_newmenu = nmenu[VGOLF_MENU_MAIN_HOST];
			}
			else if(vgolf_client)
			{
				current_newmenu = nmenu[VGOLF_MENU_MAIN_JOIN];
			}
			else
			{
				current_newmenu = nmenu[VGOLF_MENU_MAIN_HOST];
			}
			t3f_play_music("data/music/menu.xm");
			return;
		}

		/* finish the hole if player removed and all other players have finished the hole */
		if(vgolf_remaining_players_done())
		{
			main_game_vars.game_state = GAMESTATE_HOLE_FINISHED;
			main_game_vars.hole_finished = 1;
		}
	}

	if(vgolf_passing_turn)
	{

		/* should finish passing turn when the game tick counter reaches the passed turn frame */
//		if(main_game_vars.tick == vgolf_passed_turn_frame)
		if(vgolf_passed_turn_frame >= 0 && main_game_vars.tick >= vgolf_passed_turn_frame)
		{
//			printf("main_game_vars.tick = %d, vgolf_passed_turn_frame = %d\n", main_game_vars.tick, vgolf_passed_turn_frame);
			vgolf_passing_turn = false;
			vgolf_passed_turn_frame = -1;
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				game_player[i].local_club_data.swing = 0;
				vgolf_local_club_blocked = true;
			}
		}
//		else
//		{
//			printf("passing turn %d\n", vgolf_passed_turn_frame);
//		}
	}
	hide_player_huds(game_player);

	/* handle in-game click blocking */
	if(vgolf_game_block_click)
	{
		if(!vgolf_game_mouse_b)
		{
			vgolf_game_block_click = false;
		}
		else
		{
			vgolf_game_mouse_b = 0;
		}
	}

	switch(main_game_vars.game_state)
	{
		case GAMESTATE_SCOREBOARD_SEGIN:   //scoreboard is coming onscreen
			hole_logic(&game_course->current_hole);
			course_scoreboard_segin_logic(game_course);
			break;
		case GAMESTATE_SCOREBOARD_SEGMID:  //scoreboard is onscreen
			course_scoreboard_main_logic(game_course, game_player, vgolf_player_count, main_game_vars.cur_hole);
			break;
		case GAMESTATE_SCOREBOARD_SEGOUT:  //scoreboard is disappearing
			hole_logic(&game_course->current_hole);
			course_scoreboard_segout_logic(game_course);
			break;
		case GAMESTATE_START_HOLE:         //hole needs to be set up before playing
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(vgolf_player_list[i])
				{
					player_start_hole(&game_player[i], &game_course->current_hole);
				}
			}
			main_game_vars.game_state = main_game_vars.game_state_old = GAMESTATE_READY_TOPLAY;

			sprintf(temp_text, "Tee off, %s!", vgolf_client_game->player[main_game_vars.cur_player]->name);
			add_vfont_queue_message(game_vmessage, GAME_VMESSAGE_QUEUE_SIZE, vgolf_font[VGOLF_FONT_48], &main_game_vars.cur_vmessage_slot, temp_text, t3f_virtual_display_width, t3f_virtual_display_height / 2, 0, VMSG_TEXT_ENTER_FROMRIGHT, -10, 1, 0, 0, 99999);

			break;
		case GAMESTATE_READY_TOPLAY:       //we're ready to play golf
			hole_logic(&game_course->current_hole);
			player_logic_all(game_player, vgolf_player_count, &game_course->current_hole);
			get_main_keys();
			if(main_game_vars.cur_vmessage_slot > 0)
			{
				game_vmessage_logic(game_vmessage, &main_game_vars.cur_vmessage_slot, vgolf_font[VGOLF_FONT_48]);
			}
			break;
		case GAMESTATE_PLAYER_FINISHED_HOLE:
		{
			hole_logic(&game_course->current_hole);
			if(main_game_vars.cur_vmessage_slot > 0)
			{
				ball_mirage_logic(&game_player[main_game_vars.cur_player].ball); // process mirage so it will disappear properly
				game_vmessage_logic(game_vmessage, &main_game_vars.cur_vmessage_slot, vgolf_font[VGOLF_FONT_48]);
			}
			else
			{
				int p = vgolf_get_next_player();

				if(p != -1) // all players finished when -1
				{
					int v, vert_ref = 0;

					//find the vert for the hole
					for(v = 0; v < game_course->current_hole.num_verts; v++)
					{
						if(game_course->current_hole.vert[v].flag == VERT_FLAG_HOLE)
						{
							vert_ref = v;
							break;
						}
					}
					game_course->current_hole.vert[vert_ref].ref_no[0] = 0;  //empty the hole out for the next player..
					game_player[main_game_vars.cur_player].pic_mode = 0; // reset face

					/* find next player */
					if(main_game_vars.cur_player != p)
					{
						vgolf_select_player(p);
						build_turn_text(p);
						add_vfont_queue_message(game_vmessage, GAME_VMESSAGE_QUEUE_SIZE, vgolf_font[VGOLF_FONT_48], &main_game_vars.cur_vmessage_slot, temp_text, t3f_virtual_display_width / 2 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], temp_text), t3f_virtual_display_height / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]), 0, VMSG_TEXT_ENTER_FROMRIGHT, -10, 4, 0, 0, -1);
					}
					else
					{
						game_player[main_game_vars.cur_player].local_club_data.swing = 0;
						vgolf_local_club_blocked = true;
					}
					main_game_vars.game_state = GAMESTATE_READY_TOPLAY;
				}
				else
				{
					//begin seg to scoreboard
					//set current player to winner of previous hole..
					game_player[main_game_vars.cur_player].pic_mode = 0; // reset face
					main_game_vars.game_state = GAMESTATE_HOLE_FINISHED;
					main_game_vars.hole_finished = 1;
					p = vgolf_get_best_player();
					vgolf_select_player(p);
				}
			}
			break;
		}
		case GAMESTATE_HOLE_FINISHED:      //hole is finished..
			//wait for messages to finish
			if(main_game_vars.cur_vmessage_slot == 0)
			{
				//start the scoreboard seg-in
				main_game_vars.game_state = GAMESTATE_SCOREBOARD_SEGIN;
				next_hole = main_game_vars.cur_hole + 1;
			}
			else
			{
				game_vmessage_logic(game_vmessage, &main_game_vars.cur_vmessage_slot, vgolf_font[VGOLF_FONT_48]);
			}

			break;
		case GAMESTATE_COURSE_COMPLETED_1P:  //course is completed in 1-player mode
			single_player_course_completed_logic(&main_game_vars, &game_player[0], game_course);
			break;
		case GAMESTATE_COURSE_COMPLETED_XP:  //course is completed in multiplayer mode
			multi_player_course_completed_logic(&main_game_vars, game_player, vgolf_player_count, game_course);
			break;
		case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN:
			//victory screen is coming onscreen
			victory_screen_segin_logic(&main_game_vars, game_course);
			break;
		case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN:
			//victory screen is coming onscreen
			victory_screen_segin_logic(&main_game_vars, game_course);
			break;
		case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID:
			//victory screen is in place
			victory_screen_single_player_logic_main(&main_game_vars, &game_player[0], game_course);
			break;
		case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
			//victory screen is in place
			victory_screen_multi_player_logic_main(&main_game_vars, game_player, vgolf_player_count, game_course);
			break;
		case GAMESTATE_BALL_LOST:
			hyperlink_click_logic(ball_drop_page);
			hyperlink_page_logic(ball_drop_page);
			break;
		case GAMESTATE_OFFER_GIMME:
			hyperlink_click_logic(take_gimme_page);
			hyperlink_page_logic(take_gimme_page);
			break;
		default:
			//get main keyboard input (for pausing, exiting, etc.)
			get_main_keys();

			break;
	}
	main_game_vars.tick++;

	/* keep time for all players */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			if(vgolf_client_game->player[i]->local)
			{
				vgolf_profiles.item[game_player[i].profile].total.time++;
			}
		}
	}
}

bool vgolf_game_replay_logic_tick(void)
{
	int i;

	if(vgolf_read_replay_frame())
	{
		vgolf_game_mouse_x = vgolf_client_game->mouse[0]->x;
		vgolf_game_mouse_y = vgolf_client_game->mouse[0]->y;
		vgolf_game_mouse_z = vgolf_client_game->mouse[0]->z;
		vgolf_game_mouse_b = vgolf_client_game->mouse[0]->b;
		vgolf_mouse_x = vgolf_game_mouse_x;
		vgolf_mouse_y = vgolf_game_mouse_y;
		vgolf_mouse_z = vgolf_game_mouse_z;
		vgolf_mouse_b = vgolf_game_mouse_b;

		/* read player removal signals (stored in mouse button bits) */
		vgolf_remove_player[0] = vgolf_game_mouse_b & 8;
		vgolf_remove_player[1] = vgolf_game_mouse_b & 16;
		vgolf_remove_player[2] = vgolf_game_mouse_b & 32;
		vgolf_remove_player[3] = vgolf_game_mouse_b & 64;
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_remove_player[i])
			{
				vgolf_remove_player_frame[i] = main_game_vars.tick;
			}
		}
		vgolf_game_logic_tick();
		return true;
	}
	end_game_helper();
	return false;
}

#ifndef T3F_ANDROID
	static bool vgolf_game_touch_controls = false;
#else
	static bool vgolf_game_touch_controls = true;
#endif
static int vgolf_game_touch_id = -1;
static float vgolf_game_touch_start_x, vgolf_game_touch_start_y;
static float vgolf_game_touch_camera_x, vgolf_game_touch_camera_y;
static float vgolf_game_touch_ox, vgolf_game_touch_oy;
static float vgolf_game_touch_tap_x, vgolf_game_touch_tap_y;
static int vgolf_game_touch_tap_tick;
static bool vgolf_game_touch_double_tap = false;
static bool vgolf_game_touch_released = false;

/*static void vgolf_game_touch_pan_logic(PLAYER * pl)
{
	float camera_speed;

	if(vgolf_game_touch_double_tap)
	{
		vgolf_camera_toggle_zoom();
		if(main_game_vars.camera.target_z > 0.0)
		{
			vgolf_aim_camera(vgolf_game_touch_start_x, vgolf_game_touch_start_y);
		}
	}
	else
	{
		if(vgolf_game_touch_id >= 0)
		{
			camera_speed = (t3f_project_x(1.0, main_game_vars.camera.z) - t3f_project_x(0.0, main_game_vars.camera.z));
			main_game_vars.camera.x = vgolf_game_touch_camera_x - camera_speed * vgolf_game_touch_ox;
			main_game_vars.camera.target_x = main_game_vars.camera.x;
			main_game_vars.camera.y = vgolf_game_touch_camera_y - camera_speed * vgolf_game_touch_oy;
			main_game_vars.camera.target_y = main_game_vars.camera.y;
		}
	}
} */

static void vgolf_game_touch_shot_logic(PLAYER * pl)
{
	float d = 0.0;
	if(vgolf_game_touch_released)
	{
		if(vgolf_touch_mouse_z > 0)
		{
			vgolf_touch_mouse_b = 1;
		}
	}
	else if(vgolf_game_touch_id >= 0)
	{
		vgolf_touch_mouse_z = 0;
		vgolf_touch_mouse_x = t3f_project_x(pl->ball.s.x - main_game_vars.camera.x, -main_game_vars.camera.z) + vgolf_game_touch_ox * 2.0;
		vgolf_touch_mouse_y = t3f_project_y(pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z) + vgolf_game_touch_oy * 2.0;
		d = t3f_distance(vgolf_game_touch_start_x, vgolf_game_touch_start_y, t3f_touch[vgolf_game_touch_id].x, t3f_touch[vgolf_game_touch_id].y);
		if(d >= vgolf_config[CONFIG_DEAD_SPOT])
		{
			vgolf_touch_mouse_z = (d - vgolf_config[CONFIG_DEAD_SPOT]) / 2.0;
			pl->local_club_data.hidden = false;
		}
	}
}

/* emulate mouse with touch controls */
static void vgolf_game_touch_logic(PLAYER * pl)
{
	int i;

	vgolf_touch_mouse_b = 0;
	vgolf_game_touch_double_tap = false;
	vgolf_game_touch_released = false;

	/* find first touch and use it */
	if(vgolf_game_touch_id < 0)
	{
		vgolf_touch_mouse_z = 0;
		for(i = 0; i < T3F_MAX_TOUCHES; i++)
		{
			if(t3f_touch[i].active)
			{
				vgolf_game_touch_id = i;
				vgolf_game_touch_start_x = t3f_touch[i].x;
				vgolf_game_touch_start_y = t3f_touch[i].y;
				vgolf_game_touch_camera_x = main_game_vars.camera.x;
				vgolf_game_touch_camera_y = main_game_vars.camera.y;
				vgolf_game_touch_ox = 0;
				vgolf_game_touch_oy = 0;
				if(main_game_vars.tick - vgolf_game_touch_tap_tick <= 12 && fabs(t3f_distance(t3f_touch[i].x, t3f_touch[i].y, vgolf_game_touch_tap_x, vgolf_game_touch_tap_y)) < 10.0)
				{
					vgolf_game_touch_double_tap = true;
					t3f_touch[i].active = false; // deactivate touch to prevent touch input after double tap
				}
				main_game_vars.player_huds_state = 1;
				break;
			}
		}
	}
	else
	{
		if(!t3f_touch[vgolf_game_touch_id].active)
		{
//				pl->club_data.swing = 1;
			vgolf_game_touch_tap_x = t3f_touch[vgolf_game_touch_id].x;
			vgolf_game_touch_tap_y = t3f_touch[vgolf_game_touch_id].y;
			vgolf_game_touch_tap_tick = main_game_vars.tick;
			vgolf_game_touch_released = true;

			vgolf_game_touch_id = -1;
			main_game_vars.player_huds_state = 3;
		}
		else
		{
			vgolf_game_touch_ox = t3f_touch[vgolf_game_touch_id].x - vgolf_game_touch_start_x;
			vgolf_game_touch_oy = t3f_touch[vgolf_game_touch_id].y - vgolf_game_touch_start_y;
		}
	}

/*	switch(vgolf_input_mode)
	{
		case 0: // pan/zoom
		{
			vgolf_game_touch_pan_logic(pl, camera_speed);
			break;
		}
		case 1: // shoot
		{
			vgolf_game_touch_shot_logic(pl);
			break;
		}
	} */
	vgolf_game_touch_shot_logic(pl);
}

void vgolf_game_logic(void)
{
	int i, f, h;
	int update = 0;

	/* handle commentary sample start */
	if(cvlength > 0.0)
	{
		cvlength -= 1.0 / 60.0;
		if(cvlength <= 0.0)
		{
			t3f_set_music_volume((float)vgolf_config[CONFIG_MUSIC_VOLUME] / 100.0);
		}
	}
	if(cvtime > 0 && csample)
	{
		cvtime--;
		if(cvtime <= 0)
		{
			al_play_sample(csample, (float)vgolf_config[CONFIG_COMMENTARY_VOLUME] / 100.0, 0.0, 1.0, ALLEGRO_PLAYMODE_ONCE, NULL);
			csample = NULL;
		}
	}

	/* bypass joynet for replay playback */
	if(vgolf_replay_mode == VGOLF_REPLAY_MODE_PLAY)
	{
		vgolf_passing_turn = false; // ignore turn passing in replay mode since input was modified in regards to turn passing before being recorded into the replay
		f = 1;
		if(t3f_key[ALLEGRO_KEY_RIGHT])
		{
			f = 8;
		}
		if(t3f_key[ALLEGRO_KEY_PGDN])
		{
			h = main_game_vars.cur_hole;
			vgolf_replay_seek = true;
			al_stop_timer(t3f_timer);
			while(h == main_game_vars.cur_hole)
			{
				if(!vgolf_game_replay_logic_tick())
				{
					break;
				}
			}
			al_start_timer(t3f_timer);
			vgolf_replay_seek = false;
			f = 0;
			t3f_key[ALLEGRO_KEY_PGDN] = 0;
		}
		if(t3f_key[ALLEGRO_KEY_S])
		{
			vgolf_replay_slowmo_tick++;
			if(vgolf_replay_slowmo_tick % 4 == 0)
			{
				f = 1;
			}
			else
			{
				f = 0;
			}
		}
		if(t3f_key[ALLEGRO_KEY_SPACE])
		{
			vgolf_replay_paused = !vgolf_replay_paused;
			t3f_key[ALLEGRO_KEY_SPACE] = 0;
		}
		if(vgolf_replay_paused)
		{
			f = 0;
		}
		if(t3f_key[ALLEGRO_KEY_ENTER] && vgolf_replay_paused)
		{
			f = 1;
			t3f_key[ALLEGRO_KEY_ENTER] = 0;
		}
		for(i = 0; i < f; i++)
		{
			if(!vgolf_game_replay_logic_tick())
			{
				break;
			}
		}
	}
	else
	{

		/* handle fake menu logic (for latency hiding) */
		switch(main_game_vars.game_state)
		{
			case GAMESTATE_SCOREBOARD_SEGMID:  //scoreboard is onscreen
				if(game_course->scoreboard.scoreboard_seg_count >= game_course->scoreboard.num_scoreboard_segs + 1)
				{
					hyperlink_click_logic(fake_scoreboard_page);
					hyperlink_page_logic(fake_scoreboard_page);
				}
				break;
			case GAMESTATE_BALL_LOST:
				hyperlink_click_logic(fake_ball_drop_page);
				hyperlink_page_logic(fake_ball_drop_page);
				break;
			case GAMESTATE_OFFER_GIMME:
				hyperlink_click_logic(fake_take_gimme_page);
				hyperlink_page_logic(fake_take_gimme_page);
				break;
			case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID:
			case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
				hyperlink_click_logic(fake_game_complete_page);
				hyperlink_page_logic(fake_game_complete_page);
				break;
		}


		if(vgolf_game_touch_controls)
		{
			game_player[main_game_vars.cur_player].local_club_data.hidden = true;
			if(!locally_block_click && main_game_vars.game_state == GAMESTATE_READY_TOPLAY && game_player[main_game_vars.cur_player].ball.mode == BALL_MODE_IDLE && main_game_vars.game_state != GAMESTATE_BALL_LOST)
			{
				/* store 'untouched' mouse data so we can interact with menus */
				vgolf_game_touch_logic(&game_player[main_game_vars.cur_player]);
			}

			/* overwrite mouse data with that generated by touch handler */
			switch(main_game_vars.game_state)
			{
				case GAMESTATE_SCOREBOARD_SEGMID:
				case GAMESTATE_BALL_LOST:
				case GAMESTATE_OFFER_GIMME:
				case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID:
				case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
				{
					break;
				}
				default:
				{
					vgolf_local_mouse_x = vgolf_touch_mouse_x;
					vgolf_local_mouse_y = vgolf_touch_mouse_y;
					vgolf_local_mouse_z = vgolf_touch_mouse_z;
					vgolf_local_mouse_b = vgolf_touch_mouse_b;
					vgolf_mouse_x = vgolf_touch_mouse_x;
					vgolf_mouse_y = vgolf_touch_mouse_y;
					vgolf_mouse_z = vgolf_touch_mouse_z;
					vgolf_mouse_b = vgolf_touch_mouse_b;
					break;
				}
			}
		}
		else
		{
			vgolf_local_mouse_x = t3f_mouse_x;
			vgolf_local_mouse_y = t3f_mouse_y;
			vgolf_local_mouse_z = t3f_mouse_z * 5;
			vgolf_local_mouse_b = t3f_mouse_button[0] ? 1 : 0;
		}

		/* temporary fix until we figure out how to keep the club from disappearing at the wrong time */
		if(t3f_key[ALLEGRO_KEY_C])
		{
			game_player[main_game_vars.cur_player].local_club_data.swing = 0;
			t3f_key[ALLEGRO_KEY_C] = 0;
		}

		/* handle local club logic (for latency hiding) */
		if(main_game_vars.game_state == GAMESTATE_READY_TOPLAY)
		{
			if(vgolf_client_game->player[main_game_vars.cur_player]->local)
			{
				vgolf_get_player_club_angle(&game_player[main_game_vars.cur_player], &game_player[main_game_vars.cur_player].local_club_data, vgolf_local_mouse_x, vgolf_local_mouse_y);
				if(!game_player[main_game_vars.cur_player].local_club_data.swing)
				{
					if(game_player[main_game_vars.cur_player].ball.mode == BALL_MODE_IDLE && main_game_vars.game_state != GAMESTATE_BALL_LOST)
					{
						game_player[main_game_vars.cur_player].local_club_data.power = vgolf_local_mouse_z;
						if(game_player[main_game_vars.cur_player].local_club_data.power < 0)
						{
							game_player[main_game_vars.cur_player].local_club_data.power = 0;
						}
						else if(game_player[main_game_vars.cur_player].local_club_data.power > MAX_CLUB_POWER)
						{
							game_player[main_game_vars.cur_player].local_club_data.power = MAX_CLUB_POWER;
						}

						if(main_game_vars.game_state == GAMESTATE_READY_TOPLAY && vgolf_local_mouse_b && !locally_block_click && !vgolf_local_club_blocked)
						{
							if(game_player[main_game_vars.cur_player].local_club_data.power > 0)
							{
								game_player[main_game_vars.cur_player].local_club_data.swing = 2;     //swing mode active..
							}
							else
							{
								game_player[main_game_vars.cur_player].local_club_data.swing = 1;     //swing mode active..
							}
						}
					}
				}
				else
				{
					if(!vgolf_local_mouse_b)
					{
						game_player[main_game_vars.cur_player].local_club_data.swing = 2; // set back to 0 after turn has passed
					}
				}
			}
			if(vgolf_local_club_blocked && !vgolf_local_mouse_b)
			{
				vgolf_local_club_blocked = false;
			}
		}

		/* store old mouse data (use to optimize bandwidth usage by skipping updates where the mouse input is the same as last time) */
		vgolf_old_mouse_x = vgolf_client_game->mouse[0]->x;
		vgolf_old_mouse_y = vgolf_client_game->mouse[0]->y;
		vgolf_old_mouse_z = vgolf_client_game->mouse[0]->z;
		vgolf_old_mouse_b = vgolf_client_game->mouse[0]->b;
		if(!locally_block_click)
		{
			vgolf_client_game->mouse[0]->b = vgolf_mouse_b;
		}
		else
		{
			vgolf_client_game->mouse[0]->b = 0;
		}
		vgolf_client_game->mouse[0]->x = vgolf_mouse_x;
		vgolf_client_game->mouse[0]->y = vgolf_mouse_y;
		vgolf_client_game->mouse[0]->z = vgolf_mouse_z;

		/* update input only if mouse data changed (for low-bandwidth mode, we'll only update if the mouse button has changed) */
		if(vgolf_low_bandwidth_mode)
		{
			if(vgolf_old_mouse_b != vgolf_client_game->mouse[0]->b)
			{
				update = 1;
			}
		}
		else
		{
			if(vgolf_old_mouse_x != vgolf_client_game->mouse[0]->x || vgolf_old_mouse_y != vgolf_client_game->mouse[0]->y || vgolf_old_mouse_z != vgolf_client_game->mouse[0]->z || vgolf_old_mouse_b != vgolf_client_game->mouse[0]->b)
			{
				update = 1;
			}
		}
		joynet_update_game(vgolf_client_game, update);

		/* use up one frame at a time unless we exceed the input buffer limit */
		f = joynet_get_input_buffer_frames(vgolf_client_game->input_buffer);
		if(f > 0 && f <= vgolf_config[CONFIG_INPUT_BUFFER])
		{
			f = 1;
		}
		/* lag simulation code */
/*		if(f < 30)
		{
			f = 0;
		}
		else
		{
			f -= 30;
		} */
		for(i = 0; i < f; i++)
		{
			/* clear mouse button until turn has finished passing to the next player,
			 * should prevent previous player's clicks from coming through as the next player's */
			if(vgolf_passing_turn)
			{
				vgolf_client_game->player_mouse[0]->b = 0;
			}

			joynet_game_logic(vgolf_client_game);

			/* store pending player removals in mouse button bits */
			if(vgolf_remove_player[0] && main_game_vars.tick == vgolf_remove_player_frame[0])
			{
				vgolf_client_game->player_mouse[0]->b |= 8;
			}
			if(vgolf_remove_player[1] && main_game_vars.tick == vgolf_remove_player_frame[1])
			{
				vgolf_client_game->player_mouse[0]->b |= 16;
			}
			if(vgolf_remove_player[2] && main_game_vars.tick == vgolf_remove_player_frame[2])
			{
				vgolf_client_game->player_mouse[0]->b |= 32;
			}
			if(vgolf_remove_player[3] && main_game_vars.tick == vgolf_remove_player_frame[3])
			{
				vgolf_client_game->player_mouse[0]->b |= 64;
			}

			/* write input to replay file if recording */
			if(vgolf_replay_mode == VGOLF_REPLAY_MODE_RECORD)
			{
				vgolf_write_replay_frame();
			}

//			game_player[main_game_vars.cur_player].local_club_data.hidden = false;

			/* copy current frame of input into local game input variablese */
			vgolf_game_mouse_x = vgolf_client_game->player_mouse[0]->x;
			vgolf_game_mouse_y = vgolf_client_game->player_mouse[0]->y;
			vgolf_game_mouse_z = vgolf_client_game->player_mouse[0]->z;
			vgolf_game_mouse_b = vgolf_client_game->player_mouse[0]->b;

			/* copy current frame of input into variables used by the menu system so menus can receive input through the network */
			vgolf_mouse_x = vgolf_game_mouse_x;
			vgolf_mouse_y = vgolf_game_mouse_y;
			vgolf_mouse_z = vgolf_game_mouse_z;
			vgolf_mouse_b = vgolf_game_mouse_b;

			vgolf_game_logic_tick();
		}
		if((t3f_key[ALLEGRO_KEY_LCTRL] || t3f_key[ALLEGRO_KEY_RCTRL]) && t3f_key[ALLEGRO_KEY_Q])
		{
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(vgolf_client_game->player[i]->playing && vgolf_client_game->player[i]->local)
				{
					joynet_disconnect_from_game(vgolf_client_game, 0, i);
				}
			}
			t3f_key[ALLEGRO_KEY_Q] = 0;
		}
	}
}

//draw background layer..
void draw_hole_bkg(ALLEGRO_BITMAP * bp, HOLE * hole)
{
	ALLEGRO_COLOR color;

	if(hole->layer_pic[0] != NULL)
	{
		if(vgolf_hidden_layer == 0)
		{
			color = al_map_rgba_f(0.5, 0.5, 0.5, 0.5);
		}
		else
		{
			color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
		}
		t3f_draw_scaled_bitmap(hole->layer_pic[0], color, 0 - main_game_vars.camera.x, 0 - main_game_vars.camera.y, -main_game_vars.camera.z, t3f_virtual_display_width, t3f_virtual_display_height, 0);
	}
}

//draw foreground layer..
void draw_hole_fg(ALLEGRO_BITMAP * bp, HOLE * hole)
{
	ALLEGRO_COLOR color;

	if(hole->layer_pic[1] != NULL)
	{
		if(vgolf_hidden_layer == 1)
		{
			color = al_map_rgba_f(0.5, 0.5, 0.5, 0.5);
		}
		else
		{
			color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
		}
		t3f_draw_scaled_bitmap(hole->layer_pic[1], color, 0 - main_game_vars.camera.x, 0 - main_game_vars.camera.y, -main_game_vars.camera.z, t3f_virtual_display_width, t3f_virtual_display_height, 0);
	}
}

//draws a laser between the endpoints
void laser_display(ALLEGRO_BITMAP * bp, int x1, int y1, int x2, int y2, ALLEGRO_COLOR color)
{
	//use the two-step line algorithm so we can add some noise outside the line
	draw_line_twostep(bp, x1, y1, x2, y2, color, 3);
}

//hole background display stuff
void hole_bkg_display(ALLEGRO_BITMAP * bp, HOLE * hole, COURSE_IMAGE_BANK * i_bank)
{
	//draw the hole's bitmap
	draw_hole_bkg(bp, hole);

	//draw the verts
	al_hold_bitmap_drawing(true);
	hole_vert_display_bitmap(bp, hole, 0, i_bank, &game_course->palette, 0, 0, 1.0);
	al_hold_bitmap_drawing(false);
}

//hole foreground display stuff
void hole_fg_display(ALLEGRO_BITMAP * bp, HOLE * hole, COURSE_IMAGE_BANK * i_bank)
{
	//draw the foreground if it's there
	draw_hole_fg(bp, hole);

	//draw the verts
	al_hold_bitmap_drawing(true);
	hole_vert_display_bitmap(bp, hole, 1, i_bank, &game_course->palette, 0, 0, 1.0);
	al_hold_bitmap_drawing(false);
}

//the main display stuff
void vgolf_game_render(void)
{
	float strength = 0.0;

	if(main_game_vars.cur_hole >= 0)
	{  //make sure the current hole isn't negative before doing anything

		/* don't draw in-game graphics unless we are in a state the demands it */
		switch(main_game_vars.game_state)
		{
			case GAMESTATE_SCOREBOARD_SEGMID:                  //scoreboard is set in place
			case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN:  //victory screen is segging-in
			case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN:
			case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID: //victory screen is set in place
			case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
			{
				break;
			}
			default:
			{
				//draw the hole image
				hole_bkg_display(NULL, &game_course->current_hole, &game_course->image_bank);

				//draw all player balls
				player_ball_display_all(NULL, game_player, vgolf_player_count, 0);

				hole_vert_display_primitive(NULL, &game_course->current_hole, 0, &game_course->image_bank, &game_course->palette, 0, 0, 1.0);

				//draw the foreground layer
				hole_fg_display(NULL, &game_course->current_hole, &game_course->image_bank);

				//draw all player balls
				player_ball_display_all(NULL, game_player, vgolf_player_count, 1);

				//draw the player clubs
				if(vgolf_client_game->player[main_game_vars.cur_player]->local && main_game_vars.game_state != GAMESTATE_BALL_LOST)
				{
					player_club_display_all(game_player, vgolf_player_count, main_game_vars.cur_player);
				}

				if(vgolf_client_game->player[main_game_vars.cur_player]->local && vgolf_game_touch_id >= 0)
				{
					al_hold_bitmap_drawing(false);
					al_draw_filled_circle(vgolf_game_touch_start_x, vgolf_game_touch_start_y, vgolf_config[CONFIG_DEAD_SPOT], al_map_rgba_f(1.0, 0.0, 0.0, 0.5));
					al_draw_filled_circle(vgolf_game_touch_start_x, vgolf_game_touch_start_y, (float)vgolf_config[CONFIG_DEAD_SPOT] + 200.0, al_map_rgba_f(0.25, 0.25, 0.25, 0.25));

					if(vgolf_mouse_z > 0)
					{
						strength = (float)game_player[main_game_vars.cur_player].local_club_data.power / (float)MAX_CLUB_POWER;
						al_draw_circle(vgolf_game_touch_start_x, vgolf_game_touch_start_y, (float)vgolf_config[CONFIG_DEAD_SPOT] + strength * 200.0, al_map_rgba_f(0.0, 1.0, 0.0, 1.0), 1.0);
					}
					al_hold_bitmap_drawing(true);
				}

				//draw the player heads up displays
				player_hud_display_all(game_player, vgolf_player_count, &game_course->current_hole);

				//draw the hole heads up display
				hole_hud_display(&game_course->current_hole, main_game_vars.game_mode);
				break;
			}
		}
	}

	switch(main_game_vars.game_state)
	{
		case GAMESTATE_SCOREBOARD_SEGIN:                   //scoreboard is segging-in
		case GAMESTATE_SCOREBOARD_SEGMID:                  //scoreboard is set in place
		case GAMESTATE_SCOREBOARD_SEGOUT:                  //scoreboard is leaving
			course_scoreboard_display(NULL, NULL, game_course, game_player, vgolf_player_count, main_game_vars.cur_hole);
			break;
		case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN:  //victory screen is segging-in
		case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN:
			victory_screen_display(NULL, game_course, &main_game_vars, game_player, vgolf_player_count);
			course_scoreboard_display(NULL, NULL, game_course, game_player, vgolf_player_count, main_game_vars.cur_hole);
			break;
		case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID: //victory screen is set in place
		case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
			victory_screen_display(NULL, game_course, &main_game_vars, game_player, vgolf_player_count);
			break;
		case GAMESTATE_BALL_LOST:
			al_draw_filled_rectangle(0, 0, t3f_virtual_display_width - 1, t3f_virtual_display_height - 1, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
			if(vgolf_client_game->player[main_game_vars.cur_player]->local)
			{
				if(!fake_page_hidden)
				{
					hyperlink_page_render(fake_ball_drop_page, NULL);
				}
			}
			else
			{
				hyperlink_page_render(ball_drop_page, NULL);
			}
			break;
		case GAMESTATE_OFFER_GIMME:
			al_draw_filled_rectangle(0, 0, t3f_virtual_display_width - 1, t3f_virtual_display_height - 1, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
			if(vgolf_client_game->player[main_game_vars.cur_player]->local)
			{
				if(!fake_page_hidden)
				{
					hyperlink_page_render(fake_take_gimme_page, NULL);
				}
			}
			else
			{
				hyperlink_page_render(take_gimme_page, NULL);
			}
			break;
		default:
			//draw the message found in slot 0 (this is the current message)
			game_vmessage_display(NULL, game_vmessage[0], vgolf_font[VGOLF_FONT_48]);
			break;
	}

//	sprintf(temp_text, "(%f, %f) - %d", game_player[main_game_vars.cur_player].ball.s.x, game_player[main_game_vars.cur_player].ball.s.y, main_game_vars.tick);
//	t3f_draw_text(vgolf_font[VGOLF_FONT_48]2, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 640 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48]2, temp_text), 480 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]2), 0, 0, 0, 0, temp_text);
}

void vgolf_end_game(void)
{
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 2, end_game_helper);
}

void vgolf_pause_game(void)
{
	current_newmenu = nmenu[VGOLF_MENU_IN_GAME];
	last_commentary = vgolf_config[CONFIG_COMMENTARY];
	vgolf_state = VGOLF_STATE_PAUSE;
}

void vgolf_resume_game(void)
{
	if(last_commentary != vgolf_config[CONFIG_COMMENTARY])
	{
		destroy_vgolf_commentary(main_game_vars.commentary);
		main_game_vars.commentary = load_vgolf_commentary(vgolf_commentaries.filename[vgolf_config[CONFIG_COMMENTARY]], vgolf_commentaries.fi[vgolf_config[CONFIG_COMMENTARY]]);
		if(!main_game_vars.commentary)
		{
			printf("Unable to load commentary!\n");
			exit(0);
		}
	}
	vgolf_state = VGOLF_STATE_PLAY;
}
