#include "structs.h"
#include "core/course.h"

#include "globals.h"
#include "menu_proc.h"
#include "core/utility.h"

#include "gameplay_player.h"
#include "gameplay_sound.h"
#include "gameplay_message.h"
#include "gameplay_animation.h"

//drop the ball to the edge of the offending vert (water, ob, etc)
//TODO: make ball drop near the closest edge..
void do_ball_drop(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole)
{
	float ang = -atan2((hole->s.x + vert->s.x) - ball->s.x, (hole->s.y + vert->s.y) - ball->s.y) - ALLEGRO_PI / 2.0;
	ball->s.x = hole->s.x + vert->s.x + (float)(vert->radius + ball->radius) * cos(ang);
	ball->s.y = hole->s.y + vert->s.y + (float)(vert->radius + ball->radius) * sin(ang);
	ball->visible = 1;                //make ball visible again
	ball->mode = BALL_MODE_IDLE;      //reset ball mode for next hit
	do_player_turn_finish(pl, hole);  //finish the player's turn
}

int vgolf_ball_lost_drop(int i, void * p)
{
	play_select_sample();
	do_ball_drop(&current_player->ball, current_player, &current_hole->vert[current_player->ball.water_vert], current_hole);
	deactivate_vfont_message(&game_vmessage[0]);
	current_player->pic_mode = 0;             //make player's face content again
	main_game_vars.game_state = GAMESTATE_READY_TOPLAY;
	locally_block_click = 1;
	vgolf_local_club_blocked = true;
	vgolf_game_block_click = true;
	return 1;
}

int vgolf_ball_lost_rehit(int i, void * p)
{
	play_select_sample();
	do_ball_rehit(&current_player->ball, current_hole);
	deactivate_vfont_message(&game_vmessage[0]);
	current_player->pic_mode = 0;             //make player's face content again
	main_game_vars.game_state = GAMESTATE_READY_TOPLAY;
	locally_block_click = 1;
	vgolf_local_club_blocked = true;
	vgolf_game_block_click = true;
	return 1;
}

//laser destroys ball
void do_ball_laser_zap(BALL * ball, PLAYER * pl, HOLE * hole)
{
	ball->speed = 0;                      //set ball speed to 0
	ball->v.x = ball->v.y = 0;
	ball->mode = BALL_MODE_LASER_ZAPPED;  //set ball to laser zap mode
	ball->visible = 0;                    //set ball invisible
	pl->cur_strokes++;                    //add another stroke to the player stroke total
	if(vgolf_client_game->player[pl->id]->local)
	{
		vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_STROKES]++;
	}

	//play the zap sound effect
	play_ball_zap_sample();

	//add some random message to the queue
	add_ball_zap_message();

	pl->pic_mode = 2;                     //set player pic to "ouch" mode
	if(pl->cur_strokes >= game_course->current_hole.par_score + 4 && !pl->gimme_offered)
	{
		main_game_vars.game_state = GAMESTATE_OFFER_GIMME;
		fake_page_hidden = false;
		pl->gimme_offered = 1;
	}
}

//check for collisions between the ball and the hole's walls
void check_ball_hole_collision(BALL * ball, PLAYER * pl, HOLE * hole)
{
	int i, j;
	float d, r_off, r, m;
	VERTEX v, v2, dv;

	for(i = 0; i < hole->bodies[ball->layer]; i++)
	{
		/* if it's a conveyor poly skip it */
		if(hole->body[ball->layer][i].poly.color == POLY_TYPE_CONVEYOR_WALL || hole->body[ball->layer][i].poly.color == POLY_TYPE_LAYER_TRANSFER || hole->body[ball->layer][i].poly.color == POLY_TYPE_TRANSLUCENCY)
		{
		}

		/* see if we're going to be in the path of a laser */
		else if(poly_is_empty(hole->body[ball->layer][i].poly))
		{
			if(ball->speed <= 0 && line_circle_collide(ball->s.x, ball->s.y, ball->radius, hole->body[ball->layer][i].poly.p[0].x + hole->s.x, hole->body[ball->layer][i].poly.p[0].y + hole->s.y, hole->body[ball->layer][i].poly.p[1].x + hole->s.x, hole->body[ball->layer][i].poly.p[1].y + hole->s.y))
			{
				if(ball->mode != BALL_MODE_IN_MOTION_AREA)
				{
					ball->mode = BALL_MODE_IN_MOTION_AREA;
					ball->mode2 = 1;
					ball->idle_timer = 0; //reset idle timer
					ball->idle_timeout = 60 * 5; // count 5 seconds
					return;
				}
			}
		}
		else
		{
			for(r_off = 0; r_off < (ball->speed <= 0 ? 0.5 : ball->speed); r_off += 1.0)
			{
				if(poly_is_laser(hole->body[ball->layer][i].poly))
				{
					VERTEX ball_off = new_vertex(r_off*cos(ball->f_angle), r_off*sin(ball->f_angle));
					VERTEX ball_off_pos = vertex_add(ball->s, ball_off);
					if(line_circle_collide(ball_off_pos.x, ball_off_pos.y, ball->radius, hole->body[ball->layer][i].poly.p[0].x + hole->s.x, hole->body[ball->layer][i].poly.p[0].y + hole->s.y, hole->body[ball->layer][i].poly.p[1].x + hole->s.x, hole->body[ball->layer][i].poly.p[1].y + hole->s.y) && ball->mode != BALL_MODE_LASER_ZAPPED)
					{
						do_ball_laser_zap(ball, pl, hole);
						return;
					}
				}
				else
				{
					VERTEX ball_off = new_vertex(r_off*cos(ball->f_angle), r_off*sin(ball->f_angle));
					VERTEX ball_off_pos = vertex_add(ball->s, ball_off);

					for(j = 0; j < hole->body[ball->layer][i].poly.num_vertices-1; j++)
					{
						BODY seg_body, ball_offset;

						//goto the next two points if the ball isn't within a rectangle defined by the current two points
						if(!circle_lineseg_rect_collision(ball_off_pos, ball->radius, hole->body[ball->layer][i].poly.p[j], hole->body[ball->layer][i].poly.p[j+1], hole->s))
						{
							continue;
						}

						//create a new segment body based on these two endpoints..
						new_seg_body(&seg_body, &hole->body[ball->layer][i].poly.p[j], &hole->body[ball->layer][i].poly.p[j+1], &hole->s);

						//offset the ball's body by the offset so we can check to see if it will hit a wall segment before its next velocity calculation
						body_offset(&ball_offset, &ball->body, &ball_off);

						//check for a collision between the ball body and this segment body..
						if(check_body_collision(&ball_offset, &seg_body))
						{
							//play a random sound when the ball hits a solid wall (don't do this for the trajectory stuff)
							play_ball_wall_hit_sample();

//							if(ball->speed <= 0 && ball->mode == BALL_MODE_IN_MOTION_AREA)

							/* hit by a rotating poly */
							if(hole->body[ball->layer][i].poly.color == POLY_TYPE_MORPH_ROTATE)
							{
								dv = vertex_add(hole->vert[hole->body[ball->layer][i].poly.misc].s, hole->s);
								d = v_distance(ball->s, dv);
								if(ball->speed <= 0)
								{
//									ball->speed += 1.0;   //nudge the ball a little bit if it's within a motion area and stopped when the poly hits it
									ball->idle_timer = 0; //reset idle timer
								}
                                r = (hole->vert[hole->body[ball->layer][i].poly.misc].ref_no[2] * ALLEGRO_PI) / 180.0;
                                new_vertex2(&v, ball->s.x - dv.x, ball->s.y - dv.y);
                                copy_vertex(&v2, &v);
								rotate_vertex(&v2, r);
								m = ball->speed > 5.0 ? ball->speed : 5.0;
								ball->speed += v_distance(v, v2) * 0.5;
								if(ball->speed > m)
								{
									ball->speed = m;
								}
							}

							ball->s_med = ball_offset.pos;
							ball->mirage_visible = 1;
							ball->body = ball_offset;
							ball->v = ball->body.vel;
							ball->f_angle = atan2(ball->v.y, ball->v.x);
							ball->v.x = ball->speed*cos(ball->f_angle);
							ball->v.y = ball->speed*sin(ball->f_angle);

							return;
						}
					}
				}
			}
		}
	}
}

//slow the ball down
//TODO: might need some work..
void decell_ball(BALL * ball, PLAYER * pl, HOLE * hole)
{
}

//hit the ball from the previous hit position
void do_ball_rehit(BALL * ball, HOLE * hole)
{
	ball->s = ball->s_old;        //place the ball at the last position it was hit from
	ball->layer = ball->old_layer;
	ball->visible = 1;            //make ball visible
	ball->mode = BALL_MODE_IDLE;  //reset ball mode
	ball->idle_timer = 0;         //reset idle timer if needed
	game_player[main_game_vars.cur_player].local_club_data.swing = 0;
	vgolf_local_club_blocked = true;
//	do_player_turn_finish(&game_player[main_game_vars.cur_player], hole);
}

//drop the ball on the nearest laser respawn pad (after zap)
void do_ball_laser_drop(BALL * ball, PLAYER * pl, HOLE * hole)
{
	int v;
	int dist = 0;
	int cur_slot = 0;

	for(v = 0; v < hole->num_verts; v++)
	{
		//check for a laser respawn position vert
		if(hole->vert[v].flag == VERT_FLAG_LASER_RESPAWN)
		{
			//find the distance between the ball and this vert..
			dist = v_distance(ball->s, v_offset(hole->s, hole->vert[v].s));
			//if this is the least distance of the currently found respawn positions, set the current slot to this vert
			if(dist < v_distance(ball->s, v_offset(hole->s, hole->vert[cur_slot].s)))
			{
				cur_slot = v;
			}
		}
	}
	//place the ball at the closest laser respawn position
	ball->s = v_offset(hole->s, hole->vert[cur_slot].s);
	ball->layer = hole->vert[cur_slot].layer;
	//make ball visible again
	ball->visible = 1;
	//reset ball mode
	ball->mode = BALL_MODE_IDLE;
	//make player content again
	pl->pic_mode = 0;
	do_player_turn_finish(pl, hole);  //finish the player's turn
}

/* [[new stuff */
//drop the ball on the nearest smash respawn pad (after smash)
void do_ball_smash_drop(BALL * ball, PLAYER * pl, HOLE * hole)
{
	int v;
	int dist = 0;
	int cur_slot = 0;

	for(v = 0; v < hole->num_verts; v++)
	{
		//check for a smash respawn position vert
		if(hole->vert[v].flag == VERT_FLAG_SMASH_RESPAWN)
		{
			//find the distance between the ball and this vert..
			dist = v_distance(ball->s, v_offset(hole->s, hole->vert[v].s));
			//if this is the least distance of the currently found respawn positions, set the current slot to this vert
			if(dist < v_distance(ball->s, v_offset(hole->s, hole->vert[cur_slot].s)))
			{
				cur_slot = v;
			}
		}
	}
	//place the ball at the closest laser respawn position
	ball->s = v_offset(hole->s, hole->vert[cur_slot].s);
	ball->layer = hole->vert[cur_slot].layer;
	//make ball visible again
	ball->visible = 1;
	//reset idle timer
	ball->idle_timer = 0;
	//reset ball mode
	ball->mode = BALL_MODE_IDLE;
	//make player content again
	pl->pic_mode = 0;
	do_player_turn_finish(pl, hole);  //finish the player's turn
}

//laser destroys ball
void do_ball_smash(BALL * ball, PLAYER * pl, HOLE * hole)
{
	ball->speed = 0;                      //set ball speed to 0
	ball->v.x = ball->v.y = 0;
	ball->mode = BALL_MODE_DOOR_SMASHED;  //set ball to door smash mode
	ball->visible = 0;                    //set ball invisible
	pl->cur_strokes++;                    //add another stroke to the player stroke total
	if(vgolf_client_game->player[pl->id]->local)
	{
		vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_STROKES]++;
	}

	//play the zap sound effect
	play_ball_smash_sample();

	//add some random message to the queue
	add_ball_smash_message();

	pl->pic_mode = 2;                     //set player pic to "ouch" mode
	if(pl->cur_strokes >= game_course->current_hole.par_score + 4 && !pl->gimme_offered)
	{
		main_game_vars.game_state = GAMESTATE_OFFER_GIMME;
		fake_page_hidden = false;
		pl->gimme_offered = 1;
	}
}
/* new stuff]] */

//ball sinks into water hazard
void do_ball_water_sink(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole)
{
	ball->speed = 0;                       //set speed to zero
	ball->mode = BALL_MODE_WATER_SINKING;  //ball is sinking (now we can animate it)
	ball->visible = 0;                     //ball is invisible temporarily
	pl->cur_strokes++;                     //add another stroke to the player's current total
	if(vgolf_client_game->player[pl->id]->local)
	{
		vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_STROKES]++;
	}

	//play the "splash" sample
	play_ball_splash_sample();

	//add the commentary message, and the prompt message
	add_ball_water_sunk_messages();

	pl->pic_mode = 2;    //player is unhappy
}

//ball enters the teleport pad, send ball to linked teleport pad
void do_ball_teleport(BALL * ball, VERT * vert, HOLE * hole)
{
	//play the teleport sound
	play_ball_teleport_sample();

	vert->ref_no[4] = hole->vert[vert->ref_no[0]].ref_no[4] = 0;
	ball->angle = vert->ref_no[1];
	ball->f_angle = PI*ball->angle/128.0f;
	ball->s = v_offset(hole->s, hole->vert[vert->ref_no[0]].s);

	//set ball so it sits outside the pad, to avoid re-teleporting
	ball->s.x += (hole->vert[vert->ref_no[0]].radius+ball->radius)*cos(ball->f_angle);
	ball->s.y += (hole->vert[vert->ref_no[0]].radius+ball->radius)*sin(ball->f_angle);
	ball->layer = hole->vert[vert->ref_no[0]].layer;

	//make ball visible again
	ball->visible = 1;
	//set ball activity level to hit mode so it can keep going after re-appearing
	ball->mode = BALL_MODE_HIT;
}

//ball enters the pipe.. start counter, when counter ends, ball pops out other end
void do_ball_pipe_enter(BALL * ball, VERT * vert, HOLE * hole)
{

	//if the ball isn't currently in pipe enter mode, set it
	if(ball->mode != BALL_MODE_PIPE_ENTER)
	{
		vgolf_play_sample(main_game_vars.sound_bank.sfx_ball_enter, 1.0, 0.0, 1.0);
		vert->ref_no[5] = (int)(ball->speed*100);   //record the current speed
		ball->speed = 0;                            //stop the ball temporarily
		ball->mode = BALL_MODE_PIPE_ENTER;
		ball->visible = 0;
		ball->mirage_visible = 1;
		ball->s_med = v_offset(hole->s, vert->s);
	}

	if(++vert->ref_no[2] >= vert->ref_no[3])
	{
		vert->ref_no[2] = 0;
		//ball->mode = BALL_MODE_PIPE_EXIT;
		ball->angle = vert->ref_no[4];
		ball->f_angle = PI*ball->angle/128.0;   //convert the 0-256 angle to 0-2PI angle
		ball->s = v_offset(hole->s, hole->vert[vert->ref_no[1]].s);
		ball->layer = hole->vert[vert->ref_no[1]].layer;
		ball->speed = vert->ref_no[5]/100.0;    //get the old speed back
		ball->mode = BALL_MODE_HIT;             //go back to normal hit mode
		ball->visible = 1;
		vgolf_play_sample(main_game_vars.sound_bank.sfx_ball_exit, 1.0, 0.0, 1.0);
	}
}

//ball sinks in the cup..
void do_ball_hole_sink(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole)
{
	int c;

	ball->speed = 0;
	ball->mode = BALL_MODE_SUNK;
	ball->visible = 0;
	ball->s = v_offset(hole->s, vert->s);
	vert->ref_no[0] = 1;

	//play the cup sink sample
	play_ball_holesink_sample();

	//player got a hole-in-one
	if(pl->cur_strokes == 1)
	{
		//play a hole-in-one sample
		play_hole_in_one_sample();

		//add the hole-in-one message to the queue
		add_hole_in_one_message();
		if(vgolf_client_game->player[pl->id]->local)
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_HOLES_IN_ONE]++;
		}
	}
	//player got par
	if(!(hole->par_score-pl->cur_strokes))
	{
		//plays sample for a good finish
		play_good_finish_sample();

		//add the par finish messages to the message queue
		add_par_finish_messages();

		pl->pic_mode = 1;              //player is happy
		if(vgolf_client_game->player[pl->id]->local)
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_PARS]++;
		}
	}
	//player got birdie
	else if(hole->par_score-pl->cur_strokes == 1)
	{
		//plays sample for a good finish
		play_good_finish_sample();

		//add the birdie finish messages to the message queue
		add_birdie_finish_messages();

//		pl->scorecard.num_birdies++;  //increment player's birdie count
		pl->pic_mode = 1;             //player is happy
		if(vgolf_client_game->player[pl->id]->local)
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_BIRDIES]++;
		}
	}
	//player got eagle
	else if(hole->par_score-pl->cur_strokes == 2)
	{
		//plays sample for a good finish
		play_good_finish_sample();

		//add the eagle finish messages to the message queue
		add_eagle_finish_messages();

//		pl->scorecard.num_eagles++;  //increment player's eagle count
		pl->pic_mode = 1;            //player is happy
		if(vgolf_client_game->player[pl->id]->local)
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_EAGLES]++;
		}
	}
	//player got albatross
	else if(hole->par_score-pl->cur_strokes >= 3)
	{
		//play an "albatross" sample
		play_hole_in_one_sample();

		//add the albatross finish messages to the message queue
		add_albatross_finish_messages();

//		pl->scorecard.num_albo++;  //increment player's albatross count
		pl->pic_mode = 1;          //player is happy
		if(vgolf_client_game->player[pl->id]->local)
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_ALBATROSSES]++;
		}
	}
	//player got bogey
	else if(hole->par_score-pl->cur_strokes == -1)
	{
		//plays a "bad finish" sample
		play_poor_finish_sample();

		//add the bogey finish messages to the message queue
		add_bogey_finish_messages();

		pl->pic_mode = 3;          //player is upset..
	}
	//player got worse than bogey
	else if(hole->par_score-pl->cur_strokes < -1)
	{
		//plays a "bad finish" sample
		play_poor_finish_sample();

		//add the worse-than-bogey finish messages to the message queue
		add_under_bogey_finish_messages(hole->par_score-pl->cur_strokes);

		pl->pic_mode = 3;         //player makes an upset face
	}

	if(vgolf_client_game->player[pl->id]->local)
	{
		vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_HOLES]++;
	}

	//add the latest player tallies to his scorecard
	pl->scorecard.score[main_game_vars.cur_hole] = pl->cur_strokes;
	pl->scorecard.par_score[main_game_vars.cur_hole] = hole->par_score;

	/* update profile course info */
	c = vgolf_find_course(vgolf_selected_course);
	if(c >= 0)
	{
		if(vgolf_client_game->player[pl->id]->local && (vgolf_profiles.item[pl->profile].course_info[c].score[main_game_vars.cur_hole] == 0 || vgolf_profiles.item[pl->profile].course_info[c].score[main_game_vars.cur_hole] > pl->cur_strokes))
		{
			vgolf_profiles.item[pl->profile].course_info[c].score[main_game_vars.cur_hole] = pl->cur_strokes;
		}
	}
}

//does the vert's reaction to the ball being nearby
void do_ball_vert_reaction(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole, int v_no)
{
	switch(vert->flag)
	{
		case VERT_FLAG_HOLE:      //the cup..
		{
			//make sure the ball isn't going too fast..
			if(ball->speed < BALL_MAX_HOLEHIT_SPEED)
			{
				//make sure the ball is within a certain distance of the cup's center
				if(v_distance(ball->s, v_offset(hole->s, vert->s)) < ball->radius)
				{
					//drop the ball in the cup
					do_ball_hole_sink(ball, pl, vert, hole);
					pl->so_close = 0;
				}
				else
				{
					pl->so_close = 1;
				}
			}
			//ball is really close to the hole, but not in the cup..
			ball->hole_vert = v_no;
			break;
		}
		case VERT_FLAG_HAZARD_WATER:   //this vert is a circular water hole
		{
			if(v_distance(ball->s, v_offset(hole->s, vert->s)) < vert->radius)
			{
				do_ball_water_sink(ball, pl, vert, hole);
			}
			ball->cur_vert = v_no;
			ball->water_vert = v_no;
			break;
		}
		case VERT_FLAG_PIPE_ENTER:     //this vert just teleports the ball to another vert position after a delay
		{
			if(check_circle_collision(ball->s, v_offset(hole->s, vert->s), ball->radius, vert->radius-2))
			{
				do_ball_pipe_enter(ball, vert, hole);
			}
			break;
		}
		case VERT_FLAG_PIPE_EXIT:
		{
			break;
		}
		case VERT_FLAG_INDENT_HILL:   //this vert is a circular hill that moves the ball away from its center
		{
			//find angle between ball and the hill's center point
			float ang = atan2(ball->s.y - (hole->s.y + vert->s.y), ball->s.x - (hole->s.x + vert->s.x));
			float dist = sqrt((ball->s.x - (hole->s.x + vert->s.x)) * (ball->s.x - (hole->s.x + vert->s.x)) + (ball->s.y - (hole->s.y + vert->s.y)) * (ball->s.y - (hole->s.y + vert->s.y)));
			if(dist < 0.5)
			{
				dist = 0.5;
			}
			float rad = vert->radius;
			float hill_f = dist / rad / 10.0;  //hill force

			//add the force to the ball's force vector
			ball->f.x = hill_f * cos(ang);
			ball->f.y = hill_f * sin(ang);
			break;
		}
		case VERT_FLAG_INDENT_RUT:    //this vert is a circular rut that moves the ball toward its center
		{
			//find angle between ball and the hill's center point
			float ang = atan2(ball->s.y - (hole->s.y + vert->s.y), ball->s.x - (hole->s.x + vert->s.x));
			float dist = sqrt((ball->s.x - (hole->s.x + vert->s.x)) * (ball->s.x - (hole->s.x + vert->s.x)) + (ball->s.y - (hole->s.y + vert->s.y)) * (ball->s.y - (hole->s.y + vert->s.y)));
			if(dist < 5)
			{
				dist = 5;
			}
			float rad = vert->radius;
			float hill_f = dist / rad / 10.0;  //hill force

			//add the force to the ball's force vector
			ball->f.x = -hill_f*cos(ang);
			ball->f.y = -hill_f*sin(ang);
			break;
		}
		case VERT_FLAG_HAZARD_SAND:   //this vert is a circle of sand that slows the ball down
		{
			if(ball->speed > 0) ball->decel_type = 1;
			break;
		}
		case VERT_FLAG_HAZARD_SLIME:  //this vert is a circle of slime that slows the ball down even more
		{
			if(ball->speed > 0) ball->decel_type = 2;
			break;
		}
		case VERT_FLAG_POLY_MOTION_AREA:   //this vert defines a motion area that surrounds a moving polygon
		{
			ball->cur_vert = v_no;

			//horizontally moving poly
			if(vert->ref_no[0] == 1)
			{
				VGOLF_RECT temp_rect;    //create a temp rect and check to see if ball is touching it
				temp_rect.x1 = vert->s.x-vert->radius;
				temp_rect.y1 = vert->s.y-vert->ref_no[1];
				temp_rect.x2 = vert->s.x+vert->radius;
				temp_rect.y2 = vert->s.y+vert->ref_no[2];

				//if the rects touch, keep the ball "hit" so it stays here until the poly contacts it again (moving it out of the space)
				if(rect_collision(ball->hit_rect, temp_rect, ball->s, hole->s))
				{
					ball->mode = BALL_MODE_IN_MOTION_AREA;
					ball->mode2 = 0;
					ball->idle_timeout = 75;
				}
				else ball->mode = BALL_MODE_HIT;
			}

			//vertically moving poly
			else if(vert->ref_no[0] == 2)
			{
				VGOLF_RECT temp_rect;    //create a temp rect and check to see if ball is touching it
				temp_rect.x1 = vert->s.x-vert->ref_no[1];
				temp_rect.y1 = vert->s.y-vert->radius;
				temp_rect.x2 = vert->s.x+vert->ref_no[2];
				temp_rect.y2 = vert->s.y+vert->radius;

				//if the rects touch, keep the ball "hit" so it stays here until the poly contacts it again (moving it out of the space)
				if(rect_collision(ball->hit_rect, temp_rect, ball->s, hole->s))
				{
					ball->mode = BALL_MODE_IN_MOTION_AREA;
					ball->mode2 = 0;
					ball->idle_timeout = 75;
				}
				else
				{
					ball->mode = BALL_MODE_HIT;
				}
			}

			//horizontally moving door poly
			else if(vert->ref_no[0] == 3)
			{
				//only check when doors are closing..
				if(hole->vert[vert->ref_no[3]].ref_no[0] <= 1)
				{
					VGOLF_RECT temp_rect;    //create a temp rect and check to see if ball is touching it
					temp_rect.x1 = vert->s.x-vert->radius+hole->vert[vert->ref_no[3]].ref_no[1]*hole->vert[vert->ref_no[3]].ref_no[0];
					temp_rect.y1 = vert->s.y-vert->ref_no[2];
					temp_rect.x2 = vert->s.x+vert->radius-hole->vert[vert->ref_no[3]].ref_no[1]*hole->vert[vert->ref_no[3]].ref_no[0];
					temp_rect.y2 = vert->s.y+vert->ref_no[2];

					//if the rects touch, keep the ball "hit" so it stays here until the poly contacts it again (moving it out of the space)
					if(rect_collision(ball->hit_rect, temp_rect, ball->s, hole->s))
					{
						VGOLF_RECT rect_l, rect_r;    //create a temp rect for the left and right doors
						rect_l.x1 = vert->s.x-vert->radius; rect_l.y1 = vert->s.y-vert->ref_no[2];
						rect_l.x2 = vert->s.x-vert->radius+hole->vert[vert->ref_no[3]].ref_no[1]*hole->vert[vert->ref_no[3]].ref_no[0]+hole->vert[vert->ref_no[3]].ref_no[3]+ball->radius; rect_l.y2 = vert->s.y+vert->ref_no[2];
						rect_r.x1 = vert->s.x+vert->radius-hole->vert[vert->ref_no[3]].ref_no[1]*hole->vert[vert->ref_no[3]].ref_no[0]-hole->vert[vert->ref_no[3]].ref_no[3]-ball->radius; rect_r.y1 = vert->s.y-vert->ref_no[2];
						rect_r.x2 = vert->s.x+vert->radius; rect_r.y2 = vert->s.y+vert->ref_no[2];

						//if ball touching both walls, smash..
						if(hole->vert[vert->ref_no[3]].ref_no[0] == 1)
						{
							if(rect_collision(rect_l, ball->hit_rect, hole->s, ball->s) && rect_collision(rect_r, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.x = hole->s.x+vert->s.x;
								do_ball_smash(ball, pl, hole);
							}
							//if ball is touching left wall, cling to left wall while it closes
							else if(rect_collision(rect_l, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.x = hole->s.x+rect_l.x2+ball->radius;
							}
							//if ball is touching rt wall, cling to rt wall while it closes
							else if(rect_collision(rect_r, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.x = hole->s.x+rect_r.x1-ball->radius;
							}
							else
							{
								ball->mode = BALL_MODE_IN_MOTION_AREA;
								ball->mode2 = 0;
								ball->idle_timeout = 75;
							}
						}
						else
						{
							ball->mode = BALL_MODE_IN_MOTION_AREA;
							ball->mode2 = 0;
							ball->idle_timeout = 75;
						}
					}
					else ball->mode = BALL_MODE_HIT;
				}
				else ball->mode = BALL_MODE_HIT;
			}

			//vertically moving door poly
			else if(vert->ref_no[0] == 4)
			{
				//only check when doors are closing..
				if(hole->vert[vert->ref_no[3]].ref_no[0] <= 1)
				{
					VGOLF_RECT temp_rect;    //create a temp rect and check to see if ball is touching it
					temp_rect.x1 = vert->s.x - vert->ref_no[2];
					temp_rect.y1 = vert->s.y - vert->radius + hole->vert[vert->ref_no[3]].ref_no[1] * hole->vert[vert->ref_no[3]].ref_no[0];
					temp_rect.x2 = vert->s.x + vert->ref_no[2];
					temp_rect.y2 = vert->s.y + vert->radius - hole->vert[vert->ref_no[3]].ref_no[1] * hole->vert[vert->ref_no[3]].ref_no[0];

					//if the rects touch, keep the ball "hit" so it stays here until the poly contacts it again (moving it out of the space)
					if(rect_collision(ball->hit_rect, temp_rect, ball->s, hole->s))
					{
						VGOLF_RECT rect_l, rect_r;    //create a temp rect for the left and right doors
						rect_l.x1 = vert->s.x - vert->ref_no[2]; rect_l.y1 = vert->s.y - vert->radius;
						rect_l.x2 = vert->s.x + vert->ref_no[2]; rect_l.y2 = vert->s.y - vert->radius + hole->vert[vert->ref_no[3]].ref_no[1] * hole->vert[vert->ref_no[3]].ref_no[0] + hole->vert[vert->ref_no[3]].ref_no[3] + ball->radius;
						rect_r.x1 = vert->s.x - vert->ref_no[2]; rect_r.y1 = vert->s.y + vert->radius - hole->vert[vert->ref_no[3]].ref_no[1] * hole->vert[vert->ref_no[3]].ref_no[0] - hole->vert[vert->ref_no[3]].ref_no[3] - ball->radius;
						rect_r.x2 = vert->s.x + vert->ref_no[2]; rect_r.y2 = vert->s.y + vert->radius;

						if(hole->vert[vert->ref_no[3]].ref_no[0] == 1)
						{
							//if ball touching both walls, smash..
							if(rect_collision(rect_l, ball->hit_rect, hole->s, ball->s) && rect_collision(rect_r, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.y = hole->s.y+vert->s.y;
								do_ball_smash(ball, pl, hole);
							}
							//if ball is touching left wall, cling to left wall while it closes
							else if(rect_collision(rect_l, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.y = hole->s.y+rect_l.y2+ball->radius;
							}
							//if ball is touching rt wall, cling to rt wall while it closes
							else if(rect_collision(rect_r, ball->hit_rect, hole->s, ball->s))
							{
								ball->s.y = hole->s.y+rect_r.y1-ball->radius;
							}
							else
							{
								ball->mode = BALL_MODE_IN_MOTION_AREA;
								ball->mode2 = 0;
								ball->idle_timeout = 75;
							}
						}
						else
						{
							ball->mode = BALL_MODE_IN_MOTION_AREA;
							ball->mode2 = 0;
							ball->idle_timeout = 75;
						}
					}
					else ball->mode = BALL_MODE_HIT;
				}
				else ball->mode = BALL_MODE_HIT;
			}
			break;
		}
		case VERT_FLAG_POLY_MORPH:
		{
			ball->cur_vert = v_no;
			if(vert->ref_no[0] == 0)
			{
				ball->mode = BALL_MODE_IN_MOTION_AREA;
				ball->mode2 = 0;
				ball->idle_timeout = 75;
			}
			break;
		}
		case VERT_FLAG_TELEPORT:
		{
			if(vert->ref_no[4] == 0)
			{  //if active flag isn't set, do teleport
				if(v_distance(ball->s, v_offset(hole->s, vert->s)) < ball->radius)
				{  //make sure the ball is close enough to the center of the teleport pad
					ball->visible = 0;       //set ball to visible for a second
					do_ball_teleport(ball, vert, hole);  //teleport the ball to the specified teleport pad
				}
			}
			break;
		}
		case VERT_FLAG_CONVEYOR:
		{
			ball->cur_vert = v_no;
			if((ball->conveyor_vert == -1 || ball->conveyor_vert == v_no) && point_in_poly(ball->s, hole->body[ball->layer][vert->ref_no[5]].poly, hole->s))
			{
				//if(rect_collision(ball->hit_rect, temp_rect, ball->s, hole->s)){
				if(vert->ref_no[0] == 0)
				{
					ball->cf.x = 0;
					ball->cf.y = -vert->ref_no[2]/2.0;
					//if(ball->angle < 64) ball->angle++;
					//else if(ball->angle > 64) ball->angle--;
				}
				else if(vert->ref_no[0] == 1)
				{
					ball->cf.x = 0;
					ball->cf.y = vert->ref_no[2]/2.0;
					//if(ball->angle < 192) ball->angle++;
					//else if(ball->angle > 192) ball->angle--;
				}
				else if(vert->ref_no[0] == 2)
				{
					ball->cf.y = 0;
					ball->cf.x = -vert->ref_no[2]/2.0;
					//if(ball->angle > 0) ball->angle--;
				}
				else if(vert->ref_no[0] == 3)
				{
					ball->cf.y = 0;
					ball->cf.x = vert->ref_no[2]/2.0;
					//if(ball->angle < 128) ball->angle++;
					//else if(ball->angle > 128) ball->angle--;
				}
				ball->mode = BALL_MODE_ON_CONVEYOR;
				ball->conveyor_vert = v_no;
			}
			else if(ball->conveyor_vert == v_no)
			{
				/* make sure ball keeps momentum from conveyor belt	after rolling off */
				if(ball->mode == BALL_MODE_ON_CONVEYOR)
				{
					ball->off_conveyor = 1;
					ball->conveyor_vert = -1;
				}
				ball->mode = BALL_MODE_HIT;
			}
			break;
		}
		case VERT_FLAG_LAYER_TRANSFER:
		{
			ball->cur_vert = v_no;
			if(point_in_poly(ball->s, hole->body[ball->layer][vert->ref_no[0]].poly, hole->s))
			{
				ball->layer = vert->ref_no[1];
			}
			break;
		}
		case VERT_FLAG_TRANSLUCENCY:
		{
			ball->cur_vert = v_no;
			if(point_in_poly(ball->s, hole->body[ball->layer][vert->ref_no[0]].poly, hole->s))
			{
				vgolf_hidden_layer = vert->layer + 1;
				// insert code to hide layer above current layer
			}
			break;
		}
		default:
		{
			break;
		}
	}
}

void undo_ball_vert_reaction(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole, int v_no)
{
	switch(vert->flag)
	{
		case VERT_FLAG_INDENT_HILL:   //this vert is a circular hill that moves the ball away from its center
		case VERT_FLAG_INDENT_RUT:    //this vert is a circular rut that moves the ball toward its center
		{
			break;
		}
		default:
		{
			break;
		}
	}
}

//check to see if the ball is within the radius of the vert..
int check_ball_vert_collision(BALL * ball, PLAYER * pl, VERT * vert, HOLE * hole, int v_no)
{

	//check for a collision between rectangles surrounding the ball and this vert first (might decrease # of calculations in the long run)
	if(ball->layer == vert->layer && rect_collision_f(-ball->radius, -ball->radius, ball->radius, ball->radius, -vert->radius, -vert->radius, vert->radius, vert->radius, ball->s, v_offset(hole->s, vert->s)))
	{
		//check for a collision between the ball circle and the vert circle
		if(check_circle_collision(ball->s, v_offset(hole->s, vert->s), ball->radius, vert->radius))
		{
			//react according to the vert's flag
			do_ball_vert_reaction(ball, pl, vert, hole, v_no);
			return 1;
		}
	}
	return 0;
}

//check the ball against all verts for possible collisions
void check_ball_vert_collision_all(BALL * ball, PLAYER * pl, HOLE * hole)
{
	int v;
	int reset = 1;

	ball->decel_type = 0;
	ball->hole_vert = -1;
	for(v = 0; v < hole->num_verts; v++)
	{
		if(check_ball_vert_collision(ball, pl, &hole->vert[v], hole, v))
		{
			if(hole->vert[v].flag == VERT_FLAG_INDENT_HILL || hole->vert[v].flag == VERT_FLAG_INDENT_RUT)
			{
				reset = 0;
			}
		}
	}
	if(reset)
	{
		ball->f.x = 0;
		ball->f.y = 0;
	}
}

/* check for collisions with translucency triggers */
void check_ball_vert_collision_translucency(BALL * ball, PLAYER * pl, HOLE * hole)
{
	int v;

	for(v = 0; v < hole->num_verts; v++)
	{
		if(hole->vert[v].flag == VERT_FLAG_TRANSLUCENCY)
		{
			check_ball_vert_collision(ball, pl, &hole->vert[v], hole, v);
		}
	}
}

//logic associated with the translucent ball
void ball_mirage_logic(BALL * ball)
{
	if(ball->mirage_visible)
	{                                    //mirage is visible, continue
		if(++ball->mirage_timer >= ball->mirage_timer_end)
		{  //increment the timer until the end
			ball->mirage_visible = 0;                    //make mirage invisible
			ball->mirage_timer = 0;                      //reset timer for next time
		}
	}
}

//ball logic..
void ball_logic(BALL * ball, PLAYER * pl, HOLE * hole)
{

	if(ball->mode == BALL_MODE_HIT || ball->mode == BALL_MODE_IN_MOTION_AREA || ball->mode == BALL_MODE_ON_CONVEYOR)
	{
		check_ball_hole_collision(ball, pl, hole);  //check for collisions between ball and walls
		move_player_ball(ball, pl, hole);               //move the ball around
		decell_ball(ball, pl, hole);                //slow the ball down until it stops
	}
	if(ball->mode == BALL_MODE_HIT || ball->mode == BALL_MODE_PIPE_ENTER || ball->mode == BALL_MODE_IN_MOTION_AREA || ball->mode == BALL_MODE_ON_CONVEYOR)
	{
		//check for collisions with the various verts, and react according to the vert flag
		check_ball_vert_collision_all(ball, pl, hole);
	}
	else // check for translucency triggers
	{
		check_ball_vert_collision_translucency(ball, pl, hole);
	}
	if(!ball->visible && ball->mode == BALL_MODE_WATER_SINKING)
	{
		if(do_anim_def_logic(&ball->anim_def[1], ball->s.x-12, ball->s.y-12, 0) >= (ball->anim_def[1].num_frames-1)*ball->anim_def[1].speed)
		{
			ball->anim_def[1].cur_frame = 0;   //reset the anim counter
			ball->mode = BALL_MODE_WATER_SUNK; //ball is now sunk
			current_player = pl;
			current_hole = hole;
			vgolf_ball_sunk();
		}
	}
	if(ball->mode == BALL_MODE_WATER_SUNK)
	{
	}
	else if(ball->mode == BALL_MODE_LASER_ZAPPED)
	{      //ball has been zapped by a laser
		if(!main_game_vars.cur_vmessage_slot)
		{      //wait for the "zap" message to clear before dropping ball into play again
			//bring ball back to the nearest respawn pad
			do_ball_laser_drop(ball, pl, hole);
		}
	}
	else if(ball->mode == BALL_MODE_DOOR_SMASHED)
	{      //ball has been zapped by a laser
		if(!main_game_vars.cur_vmessage_slot)
		{      //wait for the "zap" message to clear before dropping ball into play again
			//bring ball back to the nearest respawn pad
			do_ball_smash_drop(ball, pl, hole);
		}
	}
	if(ball->mode == BALL_MODE_SUNK)
	{                   //ball is in the cup
		do_player_hole_finish(pl, hole);            //finish the hole
	}
	ball_mirage_logic(ball);                            //countdown until the mirage disappears
}

void vgolf_ball_sunk(void)
{
	if(game_player[main_game_vars.cur_player].cur_strokes >= game_course->current_hole.par_score + 4 && !game_player[main_game_vars.cur_player].gimme_offered)
	{
		main_game_vars.game_state = GAMESTATE_OFFER_GIMME;
		fake_page_hidden = false;
		game_player[main_game_vars.cur_player].gimme_offered = 2;
	}
	else
	{
		main_game_vars.game_state = GAMESTATE_BALL_LOST;
		fake_page_hidden = false;
	}
}

//draw the ball "mirage" effect
void ball_mirage_display(ALLEGRO_BITMAP * bp, BALL * ball)
{
	t3f_draw_animation(game_course->image_bank.ball_pic, BALL_MIRAGE_COLOR, 0, ball->s.x - vgolf_get_bitmap_width(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.x, ball->s.y - vgolf_get_bitmap_height(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
}

void ball_shadow_display(ALLEGRO_BITMAP * bp, BALL * ball)
{
	//set to transparent drawing mode
//	color_map = &trans_table;
//	drawing_mode(DRAW_MODE_TRANS, 0, 0, 0);
	//draw the circle primitive
//	al_draw_filled_circle(ball->s.x+1, ball->s.y+3, ball->radius, BALL_SHADOW_COLOR);
	t3f_draw_animation(game_course->image_bank.ball_pic, BALL_SHADOW_COLOR, 0, ball->s.x - vgolf_get_bitmap_width(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.x + 1.0, ball->s.y - vgolf_get_bitmap_height(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.y + 3.0, -main_game_vars.camera.z, 0);
	//return to normal drawing mode
//	solid_mode();
}
