#include "t3f/t3f.h"
#include "t3f/resource.h"

#include "structs.h"
#include "core/course.h"
#include "core/legacy_course.h"
#include "init.h"
#include "globals.h"
#include "core/resource.h"
#include "replay.h"
#include "core/utility.h"
#include "menu_proc.h"
#include "core/atlas.h"
#include "core/vert.h"
#include "fx.h"
#include "network/client.h"
#include "version.h"

#include "game_camera.h"
#include "gameplay_player.h"

/* create atlas for in-game objects */
static void vgolf_make_game_atlas(VGOLF_COURSE * cr)
{
	cr->image_bank.game_atlas = t3f_create_atlas(1024, 1024);
	if(cr->image_bank.game_atlas)
	{
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.vert_pic[0][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.vert_pic[1][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.vert_pic[1][1], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[0][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[1][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[2][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[2][1], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[3][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[3][1], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[4][0], T3F_ATLAS_TILE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[4][1], T3F_ATLAS_TILE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[4][2], T3F_ATLAS_TILE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.object_pic[4][3], T3F_ATLAS_TILE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[0][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[0][1], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[0][2], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[0][3], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[1][0], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[1][1], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[1][2], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.animal_pic[1][3], T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.ball_pic, T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.ball_marker_pic, T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.target_pic, T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.club_pic, T3F_ATLAS_SPRITE);
		vgolf_atlas_sprite(cr->image_bank.game_atlas, cr->image_bank.club_outline_pic, T3F_ATLAS_SPRITE);
	}
}

//load the sound effect files
static int load_main_game_samples(SOUND_BANK * sbank)
{
	sbank->sfx_swing[0] = al_load_sample("data/sounds/sfx_h000.wav");
	if(!sbank->sfx_swing[0])
	{
		return 0;
	}
	sbank->sfx_swing[1] = al_load_sample("data/sounds/sfx_h001.wav");
	if(!sbank->sfx_swing[1])
	{
		return 0;
	}
	sbank->sfx_swing[2] = al_load_sample("data/sounds/sfx_h002.wav");
	if(!sbank->sfx_swing[2])
	{
		return 0;
	}
	sbank->sfx_ball_collision[0] = al_load_sample("data/sounds/sfx_c000.wav");
	if(!sbank->sfx_ball_collision[0])
	{
		return 0;
	}
	sbank->sfx_ball_collision[1] = al_load_sample("data/sounds/sfx_c001.wav");
	if(!sbank->sfx_ball_collision[1])
	{
		return 0;
	}
	sbank->sfx_ball_collision[2] = al_load_sample("data/sounds/sfx_c002.wav");
	if(!sbank->sfx_ball_collision[2])
	{
		return 0;
	}
	sbank->sfx_ball_holesink = al_load_sample("data/sounds/sfx_hole.wav");
	if(!sbank->sfx_ball_holesink)
	{
		return 0;
	}
	sbank->sfx_hole_finish[0][0] = al_load_sample("data/sounds/sfx_f000.wav");
	if(!sbank->sfx_hole_finish[0][0])
	{
		return 0;
	}
	sbank->sfx_hole_finish[0][1] = al_load_sample("data/sounds/sfx_f001.wav");
	if(!sbank->sfx_hole_finish[0][1])
	{
		return 0;
	}
	sbank->sfx_hole_finish[0][2] = al_load_sample("data/sounds/sfx_f002.wav");
	if(!sbank->sfx_hole_finish[0][2])
	{
		return 0;
	}
	sbank->sfx_hole_finish[0][3] = al_load_sample("data/sounds/sfx_f003.wav");
	if(!sbank->sfx_hole_finish[0][3])
	{
		return 0;
	}
	sbank->sfx_hole_finish[1][0] = al_load_sample("data/sounds/sfx_f100.wav");
	if(!sbank->sfx_hole_finish[1][0])
	{
		return 0;
	}
	sbank->sfx_hole_finish[1][1] = al_load_sample("data/sounds/sfx_f200.wav");
	if(!sbank->sfx_hole_finish[1][1])
	{
		return 0;
	}
	sbank->sfx_hole_finish[1][2] = al_load_sample("data/sounds/sfx_f201.wav");
	if(!sbank->sfx_hole_finish[1][2])
	{
		return 0;
	}
	sbank->sfx_hole_finish[1][3] = al_load_sample("data/sounds/sfx_f202.wav");
	if(!sbank->sfx_hole_finish[1][3])
	{
		return 0;
	}
	sbank->sfx_hole_finish[2][0] = al_load_sample("data/sounds/sfx_f300.wav");
	if(!sbank->sfx_hole_finish[2][0])
	{
		return 0;
	}
	sbank->sfx_hole_finish[2][1] = al_load_sample("data/sounds/sfx_f301.wav");
	if(!sbank->sfx_hole_finish[2][1])
	{
		return 0;
	}
	sbank->sfx_hole_finish[2][2] = al_load_sample("data/sounds/sfx_f302.wav");
	if(!sbank->sfx_hole_finish[2][2])
	{
		return 0;
	}
	sbank->sfx_hole_finish[2][3] = al_load_sample("data/sounds/sfx_f303.wav");
	if(!sbank->sfx_hole_finish[2][3])
	{
		return 0;
	}
	sbank->sfx_hole_ahhh[0] = al_load_sample("data/sounds/sfx_a000.wav");
	if(!sbank->sfx_hole_ahhh[0])
	{
		return 0;
	}
	sbank->sfx_hole_ahhh[1] = al_load_sample("data/sounds/sfx_a001.wav");
	if(!sbank->sfx_hole_ahhh[1])
	{
		return 0;
	}
	sbank->sfx_ball_splash = al_load_sample("data/sounds/sfx_spla.wav");
	if(!sbank->sfx_ball_splash)
	{
		return 0;
	}
	sbank->sfx_ball_rotate = al_load_sample("data/sounds/sfx_r000.wav");
	if(!sbank->sfx_ball_rotate)
	{
		return 0;
	}
	sbank->sfx_coin = al_load_sample("data/sounds/sfx_coin.wav");
	if(!sbank->sfx_coin)
	{
		return 0;
	}
	sbank->sfx_ball_zapped = al_load_sample("data/sounds/sfx_zap.wav");
	if(!sbank->sfx_ball_zapped)
	{
		return 0;
	}
	sbank->sfx_ball_teleport = al_load_sample("data/sounds/sfx_tele.wav");
	if(!sbank->sfx_ball_teleport)
	{
		return 0;
	}
	sbank->sfx_ball_enter = al_load_sample("data/sounds/sfx_pipi.wav");
	if(!sbank->sfx_ball_enter)
	{
		return 0;
	}
	sbank->sfx_ball_exit = al_load_sample("data/sounds/sfx_pipo.wav");
	if(!sbank->sfx_ball_exit)
	{
		return 0;
	}

	return 1;
}

//destroy all the sound effect samples
static void destroy_game_sample_bank(SOUND_BANK * sbank)
{
	int i,j;

	for(i = 0; i < MAX_SFX_SWING_SAMPLES; i++) al_destroy_sample(sbank->sfx_swing[i]);
	for(i = 0; i < MAX_SFX_COLLISION_SAMPLES; i++) al_destroy_sample(sbank->sfx_ball_collision[i]);
	al_destroy_sample(sbank->sfx_ball_holesink);
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < MAX_SFX_HOLE_FINISH_SAMPLES; j++)
		{
			al_destroy_sample(sbank->sfx_hole_finish[i][j]);
		}
	}
	for(i = 0; i < MAX_SFX_HOLE_AHHH_SAMPLES; i++) al_destroy_sample(sbank->sfx_hole_ahhh[i]);
	al_destroy_sample(sbank->sfx_ball_splash);
	al_destroy_sample(sbank->sfx_ball_zapped);
	al_destroy_sample(sbank->sfx_ball_teleport);
	al_destroy_sample(sbank->sfx_ball_rotate);
	al_destroy_sample(sbank->sfx_coin);
}

//set the victory screen up for display
static void set_victory_screen_vars(MAIN_GAME_VARS * gvars)
{
	int i;

	gvars->vict_pody = t3f_virtual_display_height;   //place the victory podium at the bottom of the screen

	for(i = 0; i < 4; i++)
	{        //generate some random values for the background clouds
		gvars->vict_cloud_x[i] = vgolf_rand()%640;
		gvars->vict_cloud_y[i] = vgolf_rand()%200;
	}
	for(i = 0; i < 4; i++)
	{        //place all the trophies offscreen
		gvars->vict_trophy_x[i] = gvars->vict_trophy_y[i] = -100;
		gvars->vict_winner[i] = 0;
	}
}

//this is the sequence that occurs in the transition between the title and the game
static void do_title_game_transition(void)
{
	main_game_vars.quit_game = 0;                       //quit game = false
	main_game_vars.quit_title = 1;                      //quit title = true
	main_game_vars.game_state = GAMESTATE_SCOREBOARD_SEGIN;    //we're ready for the scoreboard seg-in sequence
	main_game_vars.game_state_old = main_game_vars.game_state;
	main_game_vars.cur_vmessage_slot = 0;                      //reset the message slot to 0
	main_game_vars.cur_player = 0;                             //give player 1 the first shot
	set_victory_screen_vars(&main_game_vars);                  //set the victory screen up
	main_game_vars.cur_hole = vgolf_start_hole - 1;                              //current hole is -1 (means we're just starting up)
}

//set the ball polygon (just an arbitrary number of points)
void set_ball_poly(BALL * ball, float rad)
{
	int i;

	ball->body.mass = 100.0f;
	ball->body.vel = new_vertex(0, 0);
	ball->body.pos = new_vertex(0, 0);
	ball->body.inv_mass = (ball->body.mass > 0.0000001f) ? 1.0f / ball->body.mass : 0.0f;
	ball->body.pos.x = 400;
	ball->body.pos.y = 400;
	ball->body.poly.num_vertices = 32;
	ball->body.moveable = 1;
	ball->body.glue = 0.0;

	ball->radius = rad;
	for(i = 0; i < 32; i++)
	{
		ball->body.poly.p[i].x = ball->radius * cos(i * M_PI / 16.0f);
		ball->body.poly.p[i].y = ball->radius * sin(i * M_PI / 16.0f);
	}
}

static int vgolf_load_game_graphics(VGOLF_COURSE * cr)
{
	cr->image_bank.ball_pic = vgolf_load_animation("ball.t3a");
	cr->image_bank.ball_marker_pic = vgolf_load_animation("marker.t3a");
	cr->image_bank.target_pic = vgolf_load_animation("target.t3a");
	cr->image_bank.club_pic = vgolf_load_animation("club.t3a");
	cr->image_bank.club_outline_pic = vgolf_load_animation("club_outline.t3a");
	cr->image_bank.ball_splash_pic[0] = t3f_load_resource((void **)&cr->image_bank.ball_splash_pic[0], VGOLF_RESOURCE_TYPE_PCX, "data/graphics/splash00.pcx", 0, 0, 0);
	cr->image_bank.ball_splash_pic[1] = t3f_load_resource((void **)&cr->image_bank.ball_splash_pic[1], VGOLF_RESOURCE_TYPE_PCX, "data/graphics/splash01.pcx", 0, 0, 0);
	cr->image_bank.ball_splash_pic[2] = t3f_load_resource((void **)&cr->image_bank.ball_splash_pic[2], VGOLF_RESOURCE_TYPE_PCX, "data/graphics/splash02.pcx", 0, 0, 0);
	cr->image_bank.ball_splash_pic[3] = t3f_load_resource((void **)&cr->image_bank.ball_splash_pic[3], VGOLF_RESOURCE_TYPE_PCX, "data/graphics/splash03.pcx", 0, 0, 0);
	cr->image_bank.ball_splash_pic[4] = t3f_load_resource((void **)&cr->image_bank.ball_splash_pic[4], VGOLF_RESOURCE_TYPE_PCX, "data/graphics/splash04.pcx", 0, 0, 0);
	cr->image_bank.scorecard = t3f_load_resource((void **)&cr->image_bank.scorecard, T3F_RESOURCE_TYPE_BITMAP, "data/graphics/scorecard_strip.png", 0, 0, 0);
	return 1;
}

void vgolf_destroy_game_graphics(VGOLF_COURSE * cr)
{
	int i;

	if(cr->image_bank.ball_pic)
	{
		t3f_destroy_animation(cr->image_bank.ball_pic);
		cr->image_bank.ball_pic = NULL;
	}
	if(cr->image_bank.ball_marker_pic)
	{
		t3f_destroy_animation(cr->image_bank.ball_marker_pic);
		cr->image_bank.ball_marker_pic = NULL;
	}
	if(cr->image_bank.target_pic)
	{
		t3f_destroy_animation(cr->image_bank.target_pic);
		cr->image_bank.target_pic = NULL;
	}
	if(cr->image_bank.club_pic)
	{
		t3f_destroy_animation(cr->image_bank.club_pic);
		cr->image_bank.club_pic = NULL;
	}
	if(cr->image_bank.club_outline_pic)
	{
		t3f_destroy_animation(cr->image_bank.club_outline_pic);
		cr->image_bank.club_outline_pic = NULL;
	}

	for(i = 0; i < 5; i++)
	{
		if(cr->image_bank.ball_splash_pic[i])
		{
			t3f_destroy_resource(cr->image_bank.ball_splash_pic[i]);
			cr->image_bank.ball_splash_pic[i] = NULL;
		}
	}
	if(cr->image_bank.scorecard)
	{
		t3f_destroy_resource(cr->image_bank.scorecard);
		cr->image_bank.scorecard = NULL;
	}
}

char * vgolf_get_music(char * fn)
{
	char * type[5] = {".xm", ".it", ".s3m", ".mod", ".ogg"};
	int types = 5;
	int i;
	ALLEGRO_PATH * pp;

	for(i = 0; i < types; i++)
	{
		pp = al_create_path(fn);
		if(pp)
		{
			al_set_path_extension(pp, type[i]);
			if(al_filename_exists(al_path_cstr(pp, '/')))
			{
				sprintf(vgolf_music_fn, "%s", al_path_cstr(pp, '/'));
				return vgolf_music_fn;
			}
		}
	}
	return NULL;
}

//set the main global game variables
void vgolf_set_main_game_vars(void)
{
	int p;

	main_game_vars.quit_game = 0;
	main_game_vars.quit_all = 0;
	main_game_vars.quit_title = 0;
	//main_game_vars.game_state = GAMESTATE_START_HOLE;
	main_game_vars.game_state = GAMESTATE_SCOREBOARD_SEGIN;
	main_game_vars.game_state_old = main_game_vars.game_state;
	main_game_vars.cur_vmessage_slot = 0;
	main_game_vars.cur_player = 0;
	set_victory_screen_vars(&main_game_vars);   //reset the victory screen stuff
	main_game_vars.num_players = 2;             //set it to 1 for now..
	main_game_vars.game_mode = GAMEMODE_STROKE_PLAY;  //set it to stroke play as default
	main_game_vars.cur_hole = vgolf_start_hole - 1;                //current hole is 0
	next_hole = vgolf_start_hole;
	main_game_vars.cur_hole = vgolf_start_hole - 1;
	main_game_vars.hole_finished = 0;           //hole is not finished
	main_game_vars.cur_statsheet = -1;
	main_game_vars.ingame_timer = 0;
	main_game_vars.ingame_clicker = 0;
	main_game_vars.ingame_timer_change = 0;     //timing stuff
	main_game_vars.player_huds_state = 0;
	main_game_vars.player_huds_offset = 0;
	main_game_vars.got_hiscore = 0;

	main_game_vars.num_player_images = 8;

	//give all the players some default settings
	for(p = 0; p < VGOLF_MAX_PLAYERS; p++)
	{
		game_player[p].id = p;
		game_player[p].profile = 0;
		game_player[p].savatar = vgolf_avatars.sum[0];
		game_player[p].scourse = 0;
	}
}

//calculate poly normals..
void initialize_hole_polys(HOLE * hole)
{
	int i, j;

	for(j = 0; j < hole->body_layers; j++)
	{
		for(i = 0; i < hole->bodies[j]; i++)
		{
			calc_body_normals(&hole->body[j][i]);
		}
	}
}

//reset the message queue
void initialize_vmessage_queue(V_MESSAGE vmsg[], int num_messages, T3F_FONT * vf)
{
	int i;

	for(i = 0; i < num_messages; i++)
	{
		set_vfont_message(&vmsg[i], vf, "", 0, 0, 0, VMSG_TEXT_PLAIN, 0, 0, 0, 0, 0);
	}
}

//do all the initial vert calculations/settings..
void setup_hole_verts(HOLE * hole)
{
	int v, h;
	int hole_vert[32];
	int hole_verts = 0;
	float p, a, s;

	/* set verts to their initial states */
	hole_initialize_verts(hole);

	/* place hole */
	for(v = 0; v < hole->num_verts; v++)
	{
		if(hole->vert[v].flag == VERT_FLAG_HOLE)
		{
			hole_vert[hole_verts] = v;
			hole_verts++;
		}
	}
	if(hole_verts > 0)
	{
		h = hole_vert[vgolf_rand() % hole_verts];
		hole->vert[h].ref_no[0] = 0;   // 0 = hole empty, 1 = ball in hole

		/* optionally randomize hole position */
		if(hole->vert[h].ref_no[1] == 255)
		{
			p = vgolf_drand() * (hole->vert[h].radius - BALL_STD_RADIUS);
			a = vgolf_drand() * (ALLEGRO_PI * 2.0);
			hole->vert[h].s.x += cos(a) * p;
			hole->vert[h].s.y += sin(a) * p;
			hole->vert[h].s.x = (int)hole->vert[h].s.x;
			hole->vert[h].s.y = (int)hole->vert[h].s.y;
			hole->vert[h].radius = BALL_STD_RADIUS; // "so close" needs this
		}

		/* place hole randomly along line defined by angle stored in ref_no[2] */
		else if(hole->vert[h].ref_no[1] == 254)
		{
			s = hole->vert[h].radius - BALL_STD_RADIUS; // space where ball can be placed
			p = vgolf_drand() * ((s * 2.0) - s);
			a = ALLEGRO_PI * (float)hole->vert[h].ref_no[2] / 128.0;
			hole->vert[h].s.x += cos(a) * p;
			hole->vert[h].s.y += sin(a) * p;
			hole->vert[h].s.x = (int)hole->vert[h].s.x;
			hole->vert[h].s.y = (int)hole->vert[h].s.y;
			hole->vert[h].radius = BALL_STD_RADIUS; // "so close" needs this
		}

		/* remove unused holes */
		for(v = 0; v < hole->num_verts; v++)
		{
			if(v != h && hole->vert[v].flag == VERT_FLAG_HOLE)
			{
				hole->vert[v].flag = VERT_FLAG_NULL;
			}
		}
	}
}

//get the hole ready for play..
void vgolf_initialize_hole(HOLE * hole)
{
	hole->deccel_factor = 0.05;
	hole->s.x = t3f_virtual_display_width/2;
	hole->s.y = t3f_virtual_display_height/2;
	initialize_hole_polys(hole);
	setup_hole_verts(hole);

	initialize_vmessage_queue(game_vmessage, GAME_VMESSAGE_QUEUE_SIZE, vgolf_font[VGOLF_FONT_48]);
}

//give the course some starter values that show it's empty..
void preinitialize_course(VGOLF_COURSE * cr)
{
	strcpy(cr->name, "");
}

//reset this player's scorecard
void reset_player_scorecard(SCORECARD * sc, PLAYER * pl)
{
	int i;

//	for(i = 0; i < 18; i++) sc->score[i] = sc->par_score[i] = sc->match_won[i] = 0;
	for(i = 0; i < 18; i++) sc->score[i] = sc->par_score[i] = 0;
	strcpy(sc->score_text, "E");
//	sc->num_birdies = sc->num_eagles = sc->num_albo = 0;
//	sc->matches_won = 0;

//	strcpy(sc->name, vgolf_profiles.item[pl->profile].name);
//	sc->coin_score = 0;
//	sc->coin_best_combo = 0;
}

//reset the trajectory vertex list
void reset_trajectory_points(VERTEX t[], int num_points)
{
	int i;

	for(i = 0; i < num_points; i++)
	{
		t[i].x = t[i].y = 0;
	}
}

//do the first initialization of the player
void vgolf_reset_player_data(PLAYER * pl, int plno)
{
	int i;
	unsigned long avatar, course;
	int profile;
	int id;
	VGOLF_AVATAR * ap;

	avatar = pl->savatar;
	course = pl->scourse;
	profile = pl->profile;
	ap = pl->avatar;
	id = pl->id;

	memset(pl, 0, sizeof(PLAYER));
	pl->savatar = avatar;
	pl->scourse = course;
	pl->id = id;
	pl->avatar = ap;

	pl->ball.s.x = pl->ball.s.y = -100;
	pl->ball.v.x = pl->ball.v.y = 0;
	pl->ball.f.x = pl->ball.f.y = 0;
	pl->ball.s_old = pl->ball.s;
	pl->ball.speed = 0;
	pl->ball.visible = 0;
	pl->ball.cur_vert = 0;
	pl->ball.idle_timer = 0;
	pl->ball.idle_timeout = 75;

	pl->cur_hole = main_game_vars.cur_hole;
	pl->club_data.angle = pl->club_data.f_angle = 0;
	pl->club_data.power = 0;
	pl->club_data.rot_speed = 4;
	pl->club_data.f_rot_speed = pl->club_data.rot_speed / 128.0;
	pl->club_data.swing = 0;
	pl->local_club_data.angle = pl->local_club_data.f_angle = 0;
	pl->local_club_data.power = 0;
	pl->local_club_data.rot_speed = 4;
	pl->local_club_data.f_rot_speed = pl->local_club_data.rot_speed / 128.0;
	pl->local_club_data.swing = 0;
	pl->ball.radius = BALL_STD_RADIUS;
	pl->ball.s_med = pl->ball.s;
	pl->ball.mirage_visible = 0;
	pl->ball.mirage_timer = 0;
	pl->ball.mirage_timer_end = 5;
	pl->ball.layer = 0;

	//reset player stroke tally
	pl->cur_strokes = 0;
	//define the ball's polygon
	set_ball_poly(&pl->ball, BALL_STD_RADIUS);
	//load ball's various images
	load_ball_images(&pl->ball);

	//ball's hit rectangle should just encompass the ball's diameter
	pl->ball.hit_rect.x1 = pl->ball.hit_rect.y1 = -pl->ball.radius;
	pl->ball.hit_rect.x2 = pl->ball.hit_rect.y2 = pl->ball.radius;

	reset_player_scorecard(&pl->scorecard, pl);
	//player 1 is on the top left
	if(plno == 0)
	{
		pl->hud.x = 5;
		pl->hud.y = 10;
	}
	//player 2 is on the bottom right
	else if(plno == 1)
	{
		pl->hud.x = 435;
		pl->hud.y = 10;
	}
	//player 3 is on the top right
	else if(plno == 2)
	{
		pl->hud.x = 5;
		pl->hud.y = 420;
	}
	//player 4 is on the bottom left
	else if(plno == 3)
	{
		pl->hud.x = 5;
		pl->hud.y = 420;
	}
	//set avatar to "content" mode
	pl->pic_mode = 0;
	//reset coin counters
	//reset ball angle
	pl->ball.angle = pl->ball.f_angle = 0;
	//reset trajectory point list
//	reset_trajectory_points(pl->traj_v, MAX_TRAJECTORY_POINTS);

	pl->profile = profile;
	if(pl->avatar)
	{
		for(i = 0; i < 4; i++)
		{
			pl->pic[i] = &pl->avatar->image[i];
		}
	}
}

//initialize all players before game
void vgolf_preinitialize_players(PLAYER pl[], int num_players)
{
	int i;
	int first = -1;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			vgolf_reset_player_data(&pl[i], i);
			if(first < 0)
			{
				first = i;
			}
		}
	}
	vgolf_passed_turn_frame = -1;
	vgolf_select_player(first);
	vgolf_passing_turn = false;
}

//unload scoreboard images
void destroy_course_scoreboard(SCOREBOARD * sb)
{
	t3f_destroy_animation(sb->bgpic);
}

//unload/deinitialize all main game variable stuff
void deinitialize_main_game_vars(void)
{
	destroy_main_image_bank();
	destroy_sample_bank(&main_game_vars.sound_bank);
	destroy_vgolf_commentary(main_game_vars.commentary);
}

//unload all the ball's images from memory (if necessary)
void unload_ball_images(BALL * ball)
{

	al_destroy_bitmap(game_course->image_bank.scorecard);
}

//deinitialize the player..
void deinitialize_player(PLAYER * pl, int plno)
{
	//trash all the ball's images from memory
	unload_ball_images(&pl->ball);
}

//deinitialize all players before game exit
void deinitialize_players(PLAYER pl[], int num_players)
{
	int i;

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			deinitialize_player(&pl[i], i);
		}
	}
}

//load ball images
int load_ball_images(BALL * ball)
{
	//splash animation..
	ball->anim_def[1].speed = 4;
	ball->anim_def[1].num_frames = 4;
	ball->anim_def[1].cur_frame = 0;

	return 0;
}

void vgolf_reset_ball_data(PLAYER * pl)
{
	int i;
	for(i = 0; i < MAX_BALL_ANIM_MODES; i++)
	{
		pl->ball.anim_def[i].cur_frame = 0;
	}
	set_ball_poly(&pl->ball, BALL_STD_RADIUS);
	pl->ball.s.x = pl->ball.s.y = -100;
	pl->ball.v.x = pl->ball.v.y = 0;
	pl->ball.f.x = pl->ball.f.y = 0;
	pl->ball.cf.x = pl->ball.cf.y = 0;
	pl->ball.s_old = pl->ball.s;
	pl->ball.speed = 0;
	pl->ball.visible = 0;
	pl->ball.cur_vert = 0;
	pl->ball.idle_timer = 0;
	pl->ball.idle_timeout = 75; // this value should be set elsewhere and should suit the situation
	pl->ball.radius = BALL_STD_RADIUS;
	pl->ball.off_conveyor = 0;
	pl->ball.conveyor_vert = -1;
	pl->ball.decel_type = 0;
	pl->ball.angle = pl->ball.f_angle = 0;
	pl->ball.s_med = pl->ball.s;
	pl->ball.mirage_visible = 0;
	pl->ball.mirage_timer = 0;
	pl->ball.mirage_timer_end = 5;
	pl->ball.layer = 0;
	pl->ball.old_layer = 0;
	pl->ball.in_polymorph = 0;
	pl->ball.hole_vert = 0;
	pl->ball.water_vert = 0;
	pl->ball.lazer_poly = 0;
	pl->ball.mode = 0;
	pl->ball.mode2 = 0;
	pl->ball.hit_rect.x1 = pl->ball.hit_rect.y1 = -pl->ball.radius;
	pl->ball.hit_rect.x2 = pl->ball.hit_rect.y2 = pl->ball.radius;
}

//initialize the player before the next hole
void hole_initialize_player(PLAYER * pl, int plno, int cur_hole)
{
	vgolf_reset_ball_data(pl);

	pl->cur_hole = cur_hole;
	pl->club_data.angle = pl->club_data.f_angle = 0;
	pl->club_data.power = 0;
	pl->local_club_data.angle = pl->local_club_data.f_angle = 0;
	pl->local_club_data.power = 0;
	pl->local_club_data.swing = 0;
//	pl->club_v[0].x = pl->club_v[0].y = 0;
//	pl->club_v[1].x = pl->club_v[1].y = 0;

	pl->cur_strokes = 0;
	pl->pic_mode = 0;

	//reset trajectory point list
//	reset_trajectory_points(pl->traj_v, MAX_TRAJECTORY_POINTS);
}

//initialize all players before hole
void vgolf_hole_initialize_players(void)
{
	int i;

	main_game_vars.player_huds_state = 0;
	main_game_vars.player_huds_offset = 0;
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			hole_initialize_player(&game_player[i], i, main_game_vars.cur_hole);
		}
	}
	game_player[0].hud.x = 5;
	game_player[0].hud.y = 10;
	game_player[1].hud.x = 435;
	game_player[1].hud.y = 10;
	game_player[2].hud.x = 5;
	game_player[2].hud.y = 420;
	game_player[3].hud.x = 435;
	game_player[3].hud.y = 420;
	game_player[4].hud.x = 5;
	game_player[4].hud.y = 10 + 60;
	game_player[5].hud.x = 435;
	game_player[5].hud.y = 10 + 60;
	game_player[6].hud.x = 5;
	game_player[6].hud.y = 420 - 60;
	game_player[7].hud.x = 435;
	game_player[7].hud.y = 420 - 60;
}

bool vgolf_start_game(void)
{
	char tempfn[1024] = {0};
	char buffer[1024];
	int chosen = -1;
	int i;
	const ALLEGRO_FS_INTERFACE * old_fs;
	const ALLEGRO_FILE_INTERFACE * old_fi;

	vgolf_state = VGOLF_STATE_NONE;
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		vgolf_remove_player[i] = 0;
	}
	#ifndef T3F_ANDROID
		if(!vgolf_replay_file)
		{
			sprintf(tempfn, "%s%s.vdr", t3f_get_filename(t3f_data_path, "replays/", buffer, 1024), vgolf_get_date_string());
			vgolf_record_replay(tempfn);
		}
	#endif
	vgolf_srand(vgolf_seed);
	chosen = vgolf_find_course(vgolf_selected_course);
	al_stop_timer(t3f_timer);
	if(game_course)
	{
		vgolf_destroy_course(game_course);
	}
	if(chosen >= 0)
	{
		game_course = vgolf_load_course(vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], 0, vgolf_courses.item[chosen].fi);
		if(!game_course)
		{
			game_course = vgolf_load_legacy_course(vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], 0, vgolf_courses.item[chosen].fi);
		}
	}
	if(!game_course)
	{
		t3f_debug_message("Cannot load course: %s!", vgolf_courses.item[chosen].filename);
		return false;
	}
	if(!vgolf_load_game_graphics(game_course))
	{
		t3f_debug_message("Failed to load in-game graphics!\n");
		return false;
	}
	vgolf_make_game_atlas(game_course);
	if(!load_main_game_samples(&main_game_vars.sound_bank))
	{
		t3f_debug_message("Failed to load sound samples!\n");
		return false;
	}
//	vgolf_save_course(game_course, "test.vdc", 1);
	if(main_game_vars.commentary)
	{
		destroy_vgolf_commentary(main_game_vars.commentary);
	}
	main_game_vars.commentary = load_vgolf_commentary(vgolf_commentaries.filename[vgolf_config[CONFIG_COMMENTARY]], vgolf_commentaries.fi[vgolf_config[CONFIG_COMMENTARY]]);
	if(!main_game_vars.commentary)
	{
		t3f_debug_message("Unable to load commentary: %s!\n", vgolf_commentaries.filename[vgolf_config[CONFIG_COMMENTARY]]);
//		return false;
	}

	show_records = 0;

	/* count the players */
	vgolf_player_count = 0;
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
//			vgolf_player_list[vgolf_player_count] = i;
			vgolf_player_count++;
		}
		vgolf_upload_list[i] = false;
	}

	old_fs = al_get_fs_interface();
	al_set_fs_interface(vgolf_courses.item[chosen].fs);
	old_fi = al_get_new_file_interface();
	al_set_new_file_interface(vgolf_courses.item[chosen].fi);
	if(vgolf_get_music(vgolf_courses.item[chosen].filename))
	{
		t3f_play_music(vgolf_get_music(vgolf_courses.item[chosen].filename));
	}
	else
	{
		t3f_stop_music();
	}
	al_set_fs_interface(old_fs);
	al_set_new_file_interface(old_fi);

	do_title_game_transition();

	vgolf_preinitialize_players(game_player, vgolf_player_count); // clear and set up player data

	for(i = 0; i < GAME_VMESSAGE_QUEUE_SIZE; i++)
	{
		deactivate_vfont_message(&game_vmessage[i]);
	}
	main_game_vars.cur_vmessage_slot = 0;
	next_hole = vgolf_start_hole;

	memcpy(&game_course->current_hole, &game_course->hole[vgolf_start_hole], sizeof(HOLE));
	vgolf_initialize_hole(&game_course->current_hole);       //initialize the next hole before we start it
	vgolf_hole_initialize_players();   //initialize the players before starting the next hole
	vgolf_state = VGOLF_STATE_PLAY;
	vgolf_skip_leaderboard = true;
	vgolf_old_mouse_x = vgolf_old_mouse_y = vgolf_old_mouse_z = vgolf_old_mouse_b = 0;

	/* initialize camera */
	vgolf_reset_camera();

	main_game_vars.tick = 0;
	fx_fade_start(al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 2, NULL);
	al_start_timer(t3f_timer);
	return true;
}

void end_game_helper(void)
{
	int c;

	save_profiles(&vgolf_profiles, "vgolf.pdt"); // save profiles so we don't lose progress
	c = vgolf_find_course(vgolf_selected_course);
	if(c < 0 || vgolf_replay_mode == VGOLF_REPLAY_MODE_PLAY)
	{
		vgolf_skip_leaderboard = true;
	}
	vgolf_leaderboard_num_holes = game_course->num_holes;
	if(game_course->image_bank.game_atlas)
	{
		t3f_destroy_atlas(game_course->image_bank.game_atlas);
		game_course->image_bank.game_atlas = NULL;
	}
	vgolf_destroy_game_graphics(game_course);
	vgolf_destroy_course(game_course);
	game_course = NULL;
	destroy_game_sample_bank(&main_game_vars.sound_bank);
	load_main_image_bank();
	if(vgolf_config[CONFIG_UPLOAD] && !vgolf_skip_leaderboard)
	{
		t3f_stop_music();
		vgolf_state = VGOLF_STATE_LEADERBOARDS;
		profile_course = c;
		leaderboard_mode = 1;
		create_leaderboard_page();
		strcpy(leaderboard_menu_text, "< Menu");
	}
	else
	{
		if(vgolf_hosting)
		{
			current_newmenu = nmenu[VGOLF_MENU_MAIN_HOST];
		}
		else if(vgolf_client)
		{
			current_newmenu = nmenu[VGOLF_MENU_MAIN_JOIN];
		}
		else
		{
			current_newmenu = nmenu[VGOLF_MENU_MAIN];
		}
		vgolf_state = VGOLF_STATE_MENU;
		t3f_play_music("data/music/menu.xm");
	}
	if(vgolf_replay_file)
	{
		vgolf_stop_replay();
	}
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 2, NULL);
}

//main deinitialization routine
void deinitialize(void){
	//unload everything we allocated from memory
//	deinitialize_main_game_vars();
//        deinitialize_game_menus();
        deinitialize_players(game_player, vgolf_player_count);

        //save the current configuration settings to file
        save_config("vgolf.cfg");
}

void vgolf_handle_score_upload(void)
{
	int i;
	bool done_uploading = true;
	char text[64] = {0};

	vgolf_spawn_client_keep_alive_thread();
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local && game_player[i].profile > 0 && strlen(vgolf_client_game->player[i]->name) > 0 && !vgolf_upload_list[i])
		{
			sprintf(text, "%lu", vgolf_selected_course);
			al_stop_timer(t3f_timer);
			if(t3net_upload_score("http://www.t3-i.com/t3net2/leaderboards/insert.php", "vgolf_deluxe", "1.0", "normal", text, vgolf_client_game->player[i]->name, vgolf_player_course_stroke_total(&game_player[i], game_course->num_holes) * 2 + 'v' + 'g' + 'o' + 'l' + 'f', NULL))
			{
				vgolf_upload_list[i] = true;
			}
			al_start_timer(t3f_timer);
		}
	}

	/* see if all relevant scores have been uploaded */
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local && game_player[i].profile > 0 && strlen(vgolf_client_game->player[i]->name) > 0 && !vgolf_upload_list[i])
		{
			done_uploading = false;
		}
	}

	if(done_uploading)
	{
		start_victory_screen_segin(&main_game_vars, game_course, game_player, vgolf_player_count);
	}

	/* ask for retry or cancel */
	else
	{
		current_newmenu = nmenu[VGOLF_MENU_UPLOAD_FAIL];
		vgolf_state = VGOLF_STATE_PAUSE;
	}
	vgolf_kill_client_keep_alive_thread();
}
