#include "t3f/t3f.h"
#include "t3f/view.h"
#include "structs.h"
#include "globals.h"
#include "menu_proc.h"
#include "core/utility.h"
#include "fx.h"
#include "replay.h"

#include "gameplay_sound.h"
#include "gameplay_message.h"
#include "gameplay_ball.h"
#include "gameplay_animation.h"

void vgolf_get_player_club_angle(PLAYER * pp, VGOLF_PLAYER_CLUB_DATA * cp, int ox, int oy)
{
	cp->f_angle = atan2(t3f_project_y(pp->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z) - oy, t3f_project_x(pp->ball.s.x - main_game_vars.camera.x, -main_game_vars.camera.z) - ox);
}

//total strokes by player in course so far..
int vgolf_player_course_stroke_total(PLAYER * pl, int cur_hole)
{
	int c, sum = 0;

	//make sure we don't go over the # of holes
	//cur_hole = (cur_hole < num_holes ? cur_hole : num_holes-1);
	for(c = 0; c < cur_hole; c++)
	{
		sum += pl->scorecard.score[c];
	}
	return sum;
}

//get par up to the specified hole..
int course_stroke_total(PLAYER pl, int cur_hole, int num_holes)
{
	int c, sum = 0;

	for(c = 0; c < cur_hole; c++)
	{
		sum += pl.scorecard.par_score[c];
	}
	return sum;
}

void vgolf_select_player(int player)
{
	main_game_vars.cur_player = player;
	vgolf_passing_turn = true;
	joynet_select_player(vgolf_client_game, main_game_vars.cur_player);
}

//hit the ball
void do_player_hit_ball(PLAYER * pl)
{
	pl->ball.s_old = pl->ball.s;
	pl->ball.angle = pl->ball.f_angle = pl->club_data.f_angle;
	pl->ball.mode = BALL_MODE_HIT;
	pl->ball.speed = pl->club_data.power/8.0f;
	pl->ball.v.x = pl->ball.speed*cos(pl->club_data.f_angle);
	pl->ball.v.y = pl->ball.speed*sin(pl->club_data.f_angle);
	pl->ball.body.vel = pl->ball.v;
	pl->shot_distance = 0.0;
	pl->cur_strokes++;
	if(vgolf_client_game->player[pl->id]->local)
	{
		vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_STROKES]++;
	}
	if(main_game_vars.player_huds_state != 0)
	{
		main_game_vars.player_huds_state = 3;
	}
}

//swing the club.. (two keypresses required for ball hit)
void do_player_club_swing(PLAYER * pl)
{
	if(pl->club_data.swing == 1)
	{
		if(pl->club_data.power < MAX_CLUB_POWER) pl->club_data.power++;
		else pl->club_data.swing = 2;
	}
	if(pl->club_data.swing == 2)
	{
		if(pl->club_data.power > 0) pl->club_data.power--;
		else pl->club_data.swing = 0;
	}

//	pl->club_v[0].x = -pl->club_power/2*cos(pl->f_club_angle+pl->f_club_width);
//	pl->club_v[0].y = -pl->club_power/2*sin(pl->f_club_angle+pl->f_club_width);
//	pl->club_v[1].x = -pl->club_power/2*cos(pl->f_club_angle-pl->f_club_width);
//	pl->club_v[1].y = -pl->club_power/2*sin(pl->f_club_angle-pl->f_club_width);

	if(!(vgolf_game_mouse_b & 1) || pl->club_data.swing == 3)
	{
		al_set_mouse_z(0);
		t3f_mouse_z = 0;

		//hit the ball
		do_player_hit_ball(pl);
		//reset the hit power meter
		pl->club_data.power = 0;
		//we're not swinging anymore..
		pl->club_data.swing = 0;
		//reset the club vertices
//		pl->club_v[0] = pl->club_v[1] = zero_v();
		//play the "ball hitting club" sample
		play_club_ball_hit_sample();
	}
}

//finish the current player's turn, go to the next player (if there are others)
void do_player_turn_finish(PLAYER * pl, HOLE * hole)
{
	/* if we're close to the hole play the sounds */
	if(pl->so_close && pl->ball.hole_vert >= 0)
	{
		add_so_close_message();
		play_ahhh_sample();
		pl->so_close = 0;
  	}
  	if(pl->cur_strokes == game_course->current_hole.par_score + 4)
  	{
		main_game_vars.game_state = GAMESTATE_OFFER_GIMME;
		fake_page_hidden = false;
		pl->gimme_offered = 1;
		return;
	}
	if(vgolf_player_count > 1)
	{
		int p_no = vgolf_get_next_player();

		if(p_no != -1)
		{
			vgolf_select_player(p_no);
			build_turn_text(p_no);
			add_vfont_queue_message(game_vmessage, GAME_VMESSAGE_QUEUE_SIZE, vgolf_font[VGOLF_FONT_48], &main_game_vars.cur_vmessage_slot, temp_text, t3f_virtual_display_width / 2 - t3f_get_text_width(vgolf_font[VGOLF_FONT_48], temp_text) / 2, t3f_virtual_display_height / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2, 0, VMSG_TEXT_ENTER_FROMRIGHT, -10, 4, 0, 0, -1);
		}
		else
		{
			pl->local_club_data.swing = 0;
			vgolf_local_club_blocked = true;
		}
	}
	else
	{
		pl->local_club_data.swing = 0;
		vgolf_local_club_blocked = true;
	}
}

//either seg to next hole, or let the next player in line take their turn
void do_player_hole_finish(PLAYER * pl, HOLE * hole)
{
	int c_total, p_total;

	/* record longest shot */
	if(vgolf_client_game->player[pl->id]->local)
	{
		if((int)pl->shot_distance > vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_LONGEST_SHOT])
		{
			vgolf_profiles.item[pl->profile].total.data[VGOLF_PROFILE_DATA_LONGEST_SHOT] = (int)pl->shot_distance;
		}
	}

	/* cache overall score text */
	p_total = vgolf_player_course_stroke_total(pl, main_game_vars.cur_hole + 1);
	c_total = course_stroke_total(*pl, main_game_vars.cur_hole + 1, game_course->num_holes);
	if(p_total - c_total == 0)
	{
		sprintf(pl->scorecard.score_text, "E");
	}
	else if(p_total - c_total > 0)
	{
		sprintf(pl->scorecard.score_text, "+%d", p_total - c_total);
	}
	else
	{
		sprintf(pl->scorecard.score_text, "%d", p_total - c_total);
	}

	if(vgolf_player_count == 1)
	{
		if(!main_game_vars.cur_vmessage_slot)
		{
			//begin seg to scoreboard
			main_game_vars.game_state = GAMESTATE_HOLE_FINISHED;
			main_game_vars.hole_finished = 1;
			pl->pic_mode = 0;

		}
	}
	else
	{
	  	main_game_vars.game_state = GAMESTATE_PLAYER_FINISHED_HOLE;
	}
	pl->gimme_offered = 0;
}

//move the ball around
void move_player_ball(BALL * ball, PLAYER * pl, HOLE * hole)
{
	if(ball->off_conveyor)
	{
		/* recalculate speed based on velocity vector */
		ball->speed = distance(0, 0, ball->v.x + ball->cf.x, ball->v.y + ball->cf.y);
		ball->f_angle = atan2(ball->v.y + ball->cf.y, ball->v.x + ball->cf.x);
		ball->cf.x = 0;
		ball->cf.y = 0;
	}
	else
	{
		/* recalculate speed based on velocity vector */
		ball->speed = distance(0, 0, ball->v.x, ball->v.y);
	}

	/* decelerate ball */
	//keep slowing the ball down until its speed is 0
	if(ball->speed > 0)
	{
		if(ball->decel_type == 0)
		{
			ball->speed -= hole->deccel_factor;
		}
		else if(ball->decel_type == 1)
		{
			ball->speed -= hole->deccel_factor * 2;
		}
		else if(ball->decel_type == 2)
		{
			ball->speed -= hole->deccel_factor * 3;
		}
		if(ball->speed < 0)
		{
			ball->speed = 0;
			check_ball_hole_collision(ball, pl, hole);  //check for collisions between ball and walls
			if(ball->mode != BALL_MODE_SUNK && ball->mode != BALL_MODE_WATER_SINKING && ball->mode != BALL_MODE_IN_MOTION_AREA && ball->mode != BALL_MODE_LASER_ZAPPED && ball->mode != BALL_MODE_ON_CONVEYOR)
			{
					ball->mode = BALL_MODE_IDLE;
					ball->idle_timer = 0;
					do_player_turn_finish(pl, hole);
			}
		}
	}
	else
	{
		//set speed to 0
		ball->speed = 0;

		check_ball_hole_collision(ball, pl, hole);  //check for collisions between ball and walls

		//get ball ready for another hit, finish this player's turn
		if(ball->mode != BALL_MODE_SUNK && ball->mode != BALL_MODE_WATER_SINKING && ball->mode != BALL_MODE_IN_MOTION_AREA && ball->mode != BALL_MODE_LASER_ZAPPED && ball->mode != BALL_MODE_ON_CONVEYOR)
		{
				ball->mode = BALL_MODE_IDLE;
				ball->idle_timer = 0;
				do_player_turn_finish(pl, hole);
		}
		//ball is currently within a motion area, do the idle timer
		else if(ball->mode == BALL_MODE_IN_MOTION_AREA)
		{
			if(ball->mode2 == 0)
			{
				//if(ball->idle_timer < ball->idle_timeout) ball->idle_timer += main_game_vars.ingame_timer_change;
				//make sure ball is within the motion area before handling the idle timer
				if(check_circle_collision(ball->s, v_offset(hole->s, hole->vert[ball->cur_vert].s), ball->radius, hole->vert[ball->cur_vert].radius))
				{
					//handle the idle timer
					if(++ball->idle_timer >= ball->idle_timeout)
					{
						ball->idle_timer = 0;
						ball->mode = BALL_MODE_IDLE;
						do_player_turn_finish(pl, hole);
					}
				}
				else
				{
					ball->idle_timer = 0;
					ball->mode = BALL_MODE_IDLE;
					do_player_turn_finish(pl, hole);
				}
			}
			else if(ball->mode2 == 1)
			{
				if(++ball->idle_timer >= ball->idle_timeout)
				{
					ball->idle_timer = 0;
					ball->mode = BALL_MODE_IDLE;
					do_player_turn_finish(pl, hole);
				}
			}
		}
		//ball is on a conveyor belt..
		else if(ball->mode == BALL_MODE_ON_CONVEYOR)
		{
			if(!check_circle_collision(ball->s, v_offset(hole->s, hole->vert[ball->cur_vert].s), ball->radius, hole->vert[ball->cur_vert].radius))
			{
				ball->mode = BALL_MODE_IDLE;
				do_player_turn_finish(pl, hole);
			}
			else
			{
				ball->mode = BALL_MODE_ON_CONVEYOR;
			}
		}
		else
		{
			ball->idle_timer = 0;
		}
	}

	pl->shot_distance += distance(ball->s.x, ball->s.y, ball->s.x + ball->v.x + ball->cf.x, ball->s.y + ball->v.y + ball->cf.y);
	ball->s.x += ball->v.x + ball->cf.x;
	ball->s.y += ball->v.y + ball->cf.y;
	ball->v.x = ball->speed*cos(ball->f_angle) + ball->f.x;
	ball->v.y = ball->speed*sin(ball->f_angle) + ball->f.y;
	if(!ball->off_conveyor)
	{
		ball->f_angle = atan2(ball->v.y, ball->v.x);
	}
	else
	{
		ball->off_conveyor = 0;
	}
	ball->body.pos = ball->s;
	ball->body.vel = ball->v;
    //reset ball's force vector
	ball->f.x = ball->f.y = 0;
}

//find the p player
void get_player_keys(PLAYER * pl, HOLE * hole)
{
	/* line up shot */
	vgolf_get_player_club_angle(pl, &pl->club_data, vgolf_game_mouse_x, vgolf_game_mouse_y);

	//only get club swinging keys when ball is idle
	if(!pl->club_data.swing)
	{
		if(pl->ball.mode == BALL_MODE_IDLE && main_game_vars.game_state != GAMESTATE_BALL_LOST)
		{
			pl->club_data.power = vgolf_game_mouse_z;
			if(pl->club_data.power < 0)
			{
				pl->club_data.power = 0;
			}
			else if(pl->club_data.power > MAX_CLUB_POWER)
			{
				pl->club_data.power = MAX_CLUB_POWER;
			}
//			pl->club_v[0].x = -pl->club_power/2*cos(pl->f_club_angle+pl->f_club_width);
//			pl->club_v[0].y = -pl->club_power/2*sin(pl->f_club_angle+pl->f_club_width);
//			pl->club_v[1].x = -pl->club_power/2*cos(pl->f_club_angle-pl->f_club_width);
//			pl->club_v[1].y = -pl->club_power/2*sin(pl->f_club_angle-pl->f_club_width);

			if((vgolf_game_mouse_b & 1))
			{
				if(pl->club_data.power > 0)
				{
					pl->club_data.swing = 3;     //swing mode active..
					locally_block_click = 1;
				}
				else
				{
					pl->club_data.swing = 1;     //swing mode active..
				}
			}
			if(vgolf_game_mouse_b & 2)
			{
				//if trajectory display active, deactivate it before swing begins
				if(main_game_vars.player_huds_state == 0)
				{         //if the huds aren't already hiding..
					main_game_vars.player_huds_state = 1;  //..hide the huds
				}
				else if(main_game_vars.player_huds_state == 2)
				{         //if the huds aren't already hiding..
					main_game_vars.player_huds_state = 3;  //..hide the huds
				}
			}
		}

		//if ball is sunk in the water, wait for "d" or "r" to be pressed to drop or rehit
		else if(pl->ball.mode == BALL_MODE_WATER_SUNK)
		{
			if(main_game_vars.cur_vmessage_slot == 1)    //make sure the current message is the "[d]rop or [r]ehit?" one
			{
				//drop the ball nearby
			}
		}
	}
	//swing the player's club
	else
	{
		do_player_club_swing(pl);
	}
}

//does the player logic for one player
void player_logic(PLAYER * pl, HOLE * hole)
{
	//if the player's club isn't being swung, get key input
	get_player_keys(pl, hole);
	//handle ball logic (movement, collision, etc)
	ball_logic(&pl->ball, pl, hole);
}

//does the player logic for all players in the game
void player_logic_all(PLAYER pl[], int num_players, HOLE * hole)
{
	//handle logic for the current player
	player_logic(&pl[main_game_vars.cur_player], hole);
}

//drop the player's ball on the tee vert spot
void player_start_hole(PLAYER * pl, HOLE * hole)
{
	int i;

	//look through all the verts for the tee vert
	for(i = 0; i < hole->num_verts; i++)
	{
		if(hole->vert[i].flag == VERT_FLAG_TEE)
		{                 //tee flag found..
			pl->ball.s = v_offset(hole->s, hole->vert[i].s); //place ball on tee vert position
			pl->ball.s_old = pl->ball.s;
			pl->ball.layer = hole->vert[i].layer;
			break;
		}
	}
	pl->ball.body.pos = pl->ball.s;
	pl->ball.mode = BALL_MODE_IDLE;          //ball needs to be idle to start off
	pl->ball.idle_timer = 0;                 //reset the idle timer
	pl->ball.visible = 1;                    //ball should be visible
	main_game_vars.hole_finished = 0;        //hole isn't finished
}

//draw this player's ball
void player_ball_display(ALLEGRO_BITMAP * bp, BALL * ball, char active, int layer)
{
	if(active)
	{
		if(ball->visible && ball->layer == layer)
		{
			ball_shadow_display(bp, ball);
			//draw the ball circle
//			al_draw_filled_circle(ball->s.x, ball->s.y, ball->radius, al_map_rgb(255, 0, 0));
			t3f_draw_animation(game_course->image_bank.ball_pic, t3f_color_white, 0, ball->s.x - vgolf_get_bitmap_width(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.x, ball->s.y - vgolf_get_bitmap_height(game_course->image_bank.ball_pic) / 2.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);

			//draw the idle timer if it's active
			if(ball->idle_timer > 0)
			{
				if(ball->mode == 0)
				{
					//change this to transparency later..
					t3f_draw_textf(vgolf_font[VGOLF_FONT_48], al_map_rgb(0, 192, 0), ball->s.x - main_game_vars.camera.x, ball->s.y - ball->radius * 2 - ball->idle_timer / 25 - main_game_vars.camera.y, -main_game_vars.camera.z, 0.0, 0.0, 0, "%d", (ball->idle_timeout+1-ball->idle_timer)/19+1);
				}
				else
				{
					t3f_draw_textf(vgolf_font[VGOLF_FONT_48], al_map_rgb(0, 192, 0), ball->s.x - main_game_vars.camera.x, ball->s.y - ball->radius * 2 - ball->idle_timer / 6 - main_game_vars.camera.y, -main_game_vars.camera.z, 0.0, 0.0, 0, "%d", 9 - ball->idle_timer / 60);
				}
			}
		}
		else
		{
			//draw the splash animation if the ball is sinking
			if(ball->mode == BALL_MODE_WATER_SINKING)
			{
				do_anim_def(bp, &ball->anim_def[1], ball->s.x-12, ball->s.y-12, 0);
			}
		}
		//draw the translucent ball if necessary
		if(ball->mirage_visible)
		{
			ball_mirage_display(bp, ball);
		}
	}
	else
	{
		//draw the ball as a smaller circle if it's inactive
		if(ball->visible && ball->layer == layer)
		{
			t3f_draw_animation(game_course->image_bank.ball_marker_pic, t3f_color_white, 0, ball->s.x - 2.5 - main_game_vars.camera.x, ball->s.y - 2.5 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
		}
	}
}

//do the ball display code for all players' balls
void player_ball_display_all(ALLEGRO_BITMAP * bp, PLAYER pl[], int num_players, int layer)
{
	int p;

	for(p = 0; p < VGOLF_MAX_PLAYERS; p++)
	{
		if(vgolf_player_list[p] && p != main_game_vars.cur_player)
		{
			player_ball_display(bp, &pl[p].ball, 0, layer);
		}
	}
	if(vgolf_player_list[main_game_vars.cur_player])
	{
		player_ball_display(bp, &pl[main_game_vars.cur_player].ball, 1, layer);
	}
}

//draw one player's club (local version)
void player_club_display_local(PLAYER * pl)
{
	ALLEGRO_COLOR club_color;
	int power;

	if(pl->ball.mode == BALL_MODE_IDLE && !pl->local_club_data.hidden)
	{

		if(pl->local_club_data.swing == 0)
		{
			power = pl->local_club_data.power;
		}
		else
		{
			/* show remote club power only when using click-release swing */
			if(vgolf_client_game->player_mouse[pl->id]->z == 0)
			{
				power = pl->club_data.power;
			}
			else
			{
				power = pl->local_club_data.power;
			}
		}

		//set to transparent drawing mode
		if(power < MAX_CLUB_POWER / 2)
		{
			club_color = fx_fade_color(al_map_rgba_f(0.0, 1.0, 0.0, 1.0), al_map_rgba_f(1.0, 1.0, 0.0, 1.0), (float)power / (float)(MAX_CLUB_POWER / 2));
		}
		else
		{
			club_color = fx_fade_color(al_map_rgba_f(1.0, 1.0, 0.0, 1.0), al_map_rgba_f(1.0, 0.0, 0.0, 1.0), (float)(power - MAX_CLUB_POWER / 2) / (float)(MAX_CLUB_POWER / 2));
		}
		club_color = al_map_rgba_f(0.0, 1.0, 0.0, 1.0);
		t3f_draw_rotated_animation(game_course->image_bank.club_pic, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, pl->local_club_data.f_angle + ALLEGRO_PI / 2.0, 0);
//		al_draw_tinted_scaled_rotated_bitmap_region(game_course->image_bank.club_pic, 0, 0, al_get_bitmap_width(game_course->image_bank.club_pic), (float)al_get_bitmap_height(game_course->image_bank.club_pic) * (float)pl->club_power / (float)MAX_CLUB_POWER, club_color, 12, 1, pl->ball.s.x, pl->ball.s.y, 1.0, 1.0, pl->f_club_angle + ALLEGRO_PI / 2.0, 0);
		t3f_draw_scaled_rotated_animation_region(game_course->image_bank.club_pic, 0, 0, vgolf_get_bitmap_width(game_course->image_bank.club_pic), (float)vgolf_get_bitmap_height(game_course->image_bank.club_pic) * (float)power / (float)MAX_CLUB_POWER, club_color, 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, 1.0, pl->local_club_data.f_angle + ALLEGRO_PI / 2.0, 0);
		t3f_draw_rotated_animation(game_course->image_bank.club_outline_pic, t3f_color_white, 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, pl->local_club_data.f_angle + ALLEGRO_PI / 2.0, 0);

		/* draw crosshair */
		t3f_draw_animation(game_course->image_bank.target_pic, al_map_rgb(0, 0, 0), 0, pl->ball.s.x + 48.0 * cos(pl->local_club_data.f_angle) - vgolf_get_bitmap_width(game_course->image_bank.target_pic) / 2.0 - 1.0 - main_game_vars.camera.x, pl->ball.s.y + 48.0 * sin(pl->local_club_data.f_angle) - vgolf_get_bitmap_height(game_course->image_bank.target_pic) / 2.0 + 1.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
		t3f_draw_animation(game_course->image_bank.target_pic, t3f_color_white, 0, pl->ball.s.x + 48.0 * cos(pl->local_club_data.f_angle) - vgolf_get_bitmap_width(game_course->image_bank.target_pic) / 2.0 - main_game_vars.camera.x, pl->ball.s.y + 48.0 * sin(pl->local_club_data.f_angle) - vgolf_get_bitmap_height(game_course->image_bank.target_pic) / 2.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
	}
}

//draw one player's club
void player_club_display(PLAYER * pl)
{
	ALLEGRO_COLOR club_color;
	if(pl->ball.mode == BALL_MODE_IDLE)
	{
		//set to transparent drawing mode
		if(pl->club_data.power < MAX_CLUB_POWER / 2)
		{
			club_color = fx_fade_color(al_map_rgba_f(0.0, 1.0, 0.0, 1.0), al_map_rgba_f(1.0, 1.0, 0.0, 1.0), (float)pl->club_data.power / (float)(MAX_CLUB_POWER / 2));
		}
		else
		{
			club_color = fx_fade_color(al_map_rgba_f(1.0, 1.0, 0.0, 1.0), al_map_rgba_f(1.0, 0.0, 0.0, 1.0), (float)(pl->club_data.power - MAX_CLUB_POWER / 2) / (float)(MAX_CLUB_POWER / 2));
		}
		club_color = al_map_rgba_f(0.0, 1.0, 0.0, 1.0);
		t3f_draw_rotated_animation(game_course->image_bank.club_pic, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, pl->club_data.f_angle + ALLEGRO_PI / 2.0, 0);
//		al_draw_tinted_scaled_rotated_bitmap_region(game_course->image_bank.club_pic, 0, 0, al_get_bitmap_width(game_course->image_bank.club_pic), (float)al_get_bitmap_height(game_course->image_bank.club_pic) * (float)pl->club_power / (float)MAX_CLUB_POWER, club_color, 12, 1, pl->ball.s.x, pl->ball.s.y, 1.0, 1.0, pl->f_club_angle + ALLEGRO_PI / 2.0, 0);
		t3f_draw_scaled_rotated_animation_region(game_course->image_bank.club_pic, 0, 0, vgolf_get_bitmap_width(game_course->image_bank.club_pic), (float)vgolf_get_bitmap_height(game_course->image_bank.club_pic) * (float)pl->club_data.power / (float)MAX_CLUB_POWER, club_color, 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, 1.0, pl->club_data.f_angle + ALLEGRO_PI / 2.0, 0);
		t3f_draw_rotated_animation(game_course->image_bank.club_outline_pic, t3f_color_white, 0, 12, 1, pl->ball.s.x - main_game_vars.camera.x, pl->ball.s.y - main_game_vars.camera.y, -main_game_vars.camera.z, pl->club_data.f_angle + ALLEGRO_PI / 2.0, 0);

		/* draw crosshair */
		t3f_draw_animation(game_course->image_bank.target_pic, al_map_rgb(0, 0, 0), 0, pl->ball.s.x + 48.0 * cos(pl->club_data.f_angle) - vgolf_get_bitmap_width(game_course->image_bank.target_pic) / 2.0 - 1.0 - main_game_vars.camera.x, pl->ball.s.y + 48.0 * sin(pl->club_data.f_angle) - vgolf_get_bitmap_height(game_course->image_bank.target_pic) / 2.0 + 1.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
		t3f_draw_animation(game_course->image_bank.target_pic, t3f_color_white, 0, pl->ball.s.x + 48.0 * cos(pl->club_data.f_angle) - vgolf_get_bitmap_width(game_course->image_bank.target_pic) / 2.0 - main_game_vars.camera.x, pl->ball.s.y + 48.0 * sin(pl->club_data.f_angle) - vgolf_get_bitmap_height(game_course->image_bank.target_pic) / 2.0 - main_game_vars.camera.y, -main_game_vars.camera.z, 0);
	}
}

//draw all players' clubs
void player_club_display_all(PLAYER pl[], int num_players, int cur_player)
{

	if(!vgolf_passing_turn)
	{

		/* display real club in replay mode */
		if(vgolf_replay_mode == VGOLF_REPLAY_MODE_PLAY)
		{
			if(!vgolf_low_bandwidth_mode || pl[cur_player].club_data.swing == 1)
			{
				player_club_display(&pl[cur_player]);
			}
		}
		/* always display local club for local games */
		else if(!vgolf_client)
		{
			player_club_display_local(&pl[cur_player]);
		}
		/* display local club if the local club hasn't been "swung" and player is local in networked games */
		else if(vgolf_client_game->player[cur_player]->local)
		{
			if(pl[cur_player].local_club_data.swing < 2)
			{
				player_club_display_local(&pl[cur_player]);
			}
		}
		/* only display club for remote players when not in low-bandwidth mode */
		else
		{
			if(!vgolf_low_bandwidth_mode)
			{
				player_club_display(&pl[cur_player]);
			}
		}
	}
}
