#include "structs.h"
#include "core/course.h"
#include "main.h"
#include "globals.h"
#include "core/legacy_course.h"
#include "menu_proc.h"
#include "core/utility.h"

#include "gameplay_init.h"
#include "gameplay_scoreboard.h"

//start the victory screen seg-in sequence
void start_victory_screen_segin(MAIN_GAME_VARS * gvars, VGOLF_COURSE * cr, PLAYER pl[], int num_players)
{
	int i, j, c;
	int total[VGOLF_MAX_PLAYERS];
	int low = 100000;
	int li = -1;
	int chosen;

	chosen = vgolf_find_course(vgolf_selected_course);
	if(cr->image_bank.game_atlas)
	{
		t3f_destroy_atlas(cr->image_bank.game_atlas);
		cr->image_bank.game_atlas = NULL;
	}
	vgolf_destroy_course_graphics(cr);
	vgolf_destroy_game_graphics(cr);
	if(cr->legacy)
	{
		vgolf_load_legacy_course_victory_graphics(cr, vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], 0, vgolf_courses.item[chosen].fi);
	}
	else
	{
		vgolf_load_course_victory_graphics(cr, vgolf_courses.item[chosen].filename, vgolf_config[CONFIG_HD_TEXTURES], 0, vgolf_courses.item[chosen].fi);
	}
	if(vgolf_player_count == 1)
	{
		gvars->game_state = gvars->game_state_old = GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN;
	}
	else
	{
		gvars->game_state = gvars->game_state_old = GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN;
	}
	fake_page_hidden = false;

	/* update course profile for all players */
	c = vgolf_find_course(vgolf_selected_course);
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
		{
			total[i] = 0;
			for(j = 0; j < cr->num_holes; j++)
			{
				total[i] += pl[i].scorecard.score[j];
			}
			if(c >= 0)
			{
				if(vgolf_profiles.item[pl[i].profile].course_info[c].total_score == 0 || vgolf_profiles.item[pl[i].profile].course_info[c].total_score > total[i])
				{
					vgolf_profiles.item[pl[i].profile].course_info[c].total_score = total[i];
				}
				vgolf_profiles.item[pl[i].profile].course_info[c].completed++;
			}
		}
	}

	/* update wins/losses in multiplayer mode */
	if(vgolf_player_count > 1)
	{
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
			{
				if(total[i] < low)
				{
					low = total[i];
					li = i;
				}
			}
		}
		if(c >= 0)
		{
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(vgolf_player_list[i] && vgolf_client_game->player[i]->local)
				{
					if(total[i] == low)
					{
						vgolf_profiles.item[pl[i].profile].course_info[c].won++;
					}
					else
					{
						vgolf_profiles.item[pl[i].profile].course_info[c].lost++;
					}
				}
			}
		}
	}
	gvars->cur_hole = vgolf_start_hole - 1;	                       //restart the cur_hole counter
}

//do the single player logic upon course completion
void single_player_course_completed_logic(MAIN_GAME_VARS * gvars, PLAYER * pl, VGOLF_COURSE * cr)
{
	//start the victory screen seg-in sequence
	gvars->game_state = GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGIN;
}

//do the multi-player logic upon course completion
void multi_player_course_completed_logic(MAIN_GAME_VARS * gvars, PLAYER pl[], int num_players, VGOLF_COURSE * cr)
{
	//if there are only two players, sorting will be a little easier..
	V_INT p_total, p_no;
	int i;

	switch(gvars->game_mode)
	{
		case GAMEMODE_STROKE_PLAY:
		{
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				p_no.c[i] = i;
				if(vgolf_player_list[i])
				{
					p_total.c[i] = vgolf_player_course_stroke_total(&pl[i], cr->num_holes);
				}
				else
				{
					p_total.c[i] = 100000;
				}
			}

			//sort from smallest to largest score
			v_int_heap_sort_ref(&p_total, &p_no, VGOLF_MAX_PLAYERS);

			//set winner array
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				gvars->vict_winner[i] = p_no.c[i];
			}

			break;
		}
		default:
		{
			break;
		}
	}

	//start the victory screen seg-in sequence
	gvars->game_state = GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGIN;
}

//move the clouds on the victory screen
void victory_screen_cloud_logic(MAIN_GAME_VARS * gvars)
{
	int i;

	for(i = 0; i < 4; i++)
	{
		gvars->vict_cloud_x[i] -= 0.5*(1+i);  //make the further clouds move slower
		if(gvars->vict_cloud_x[i] <= -250)
		{
			gvars->vict_cloud_x[i] = t3f_virtual_display_width+vgolf_rand()%150;  //respawn clouds at the edge of the screen when they go off the other edge
			gvars->vict_cloud_y[i] = vgolf_rand()%200;
		}
	}
}

//do all the victory screen stuff
void victory_screen_segin_logic(MAIN_GAME_VARS * gvars, VGOLF_COURSE * cr)
{
	//move podium
	if(gvars->vict_pody > 480 - vgolf_get_bitmap_height(cr->image_bank.victory_podiumpic) + 1)
	{
		gvars->vict_pody -= 2;
	}
	else
	{
		//seg-in is done, go to next state
		if(vgolf_player_count == 1) gvars->game_state = gvars->game_state_old = GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID;
		else gvars->game_state = gvars->game_state_old = GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID;
	}
	//move clouds
	victory_screen_cloud_logic(gvars);

	//do the scoreboard disappearance
	course_scoreboard_segout_logic(cr);
}

//get keys for the victory screen
void get_victory_screen_keys(MAIN_GAME_VARS * gvars, VGOLF_COURSE * cr)
{
	hyperlink_click_logic(game_complete_page);
	hyperlink_page_logic(game_complete_page);
}

//victory screen logic for single player game
void victory_screen_single_player_logic_main(MAIN_GAME_VARS * gvars, PLAYER * pl, VGOLF_COURSE * cr)
{
	//move clouds
	victory_screen_cloud_logic(gvars);
	//get keys
	get_victory_screen_keys(gvars, cr);
}

//victory screen logic for multiplayer game
void victory_screen_multi_player_logic_main(MAIN_GAME_VARS * gvars, PLAYER pl[], int num_players, VGOLF_COURSE * cr)
{
	//move clouds
	victory_screen_cloud_logic(gvars);
	//get keys
	get_victory_screen_keys(gvars, cr);
}

//draw the victory screen background image
void course_victory_screen_bg_display(ALLEGRO_BITMAP * bp, VGOLF_COURSE * cr)
{
	t3f_draw_animation(cr->image_bank.victory_bgpic, t3f_color_white, 0, 0, 0, 0, 0);
}

//draw the victory screen clouds
void course_victory_screen_cloud_display(ALLEGRO_BITMAP * bp, VGOLF_COURSE * cr, MAIN_GAME_VARS * gvars)
{
	int i;

	for(i = 3; i >= 0; i--)
	{
		t3f_draw_animation(cr->image_bank.victory_cloudpic[i/2], t3f_color_white, 0, gvars->vict_cloud_x[i], gvars->vict_cloud_y[i], 0, 0);
	}
}

//display the victory message
void victory_screen_text_display(ALLEGRO_BITMAP * bp, int x, int y, int pl, VGOLF_COURSE * cr, MAIN_GAME_VARS * gvars, char tied)
{
	char buf[64];

	al_hold_bitmap_drawing(true);
	switch(gvars->game_mode)
	{
		case GAMEMODE_STROKE_PLAY:
			sprintf(buf, "Congratulations, %s.", tied ? "players" : vgolf_client_game->player[pl]->name);
			outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x, y, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
			sprintf(buf, "Here %s courtesy of", tied ? "are some trophies" : "is a trophy");
			outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x, y + 30, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
			sprintf(buf, "the %s club membership!", cr->name);
			outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x, y + 60, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);

			if(gvars->got_hiscore)
			{
				sprintf(buf, "You got a high score!");
				outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x, y + 90, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
			}
			if(vgolf_player_count == 1)
			{
				sprintf(buf, "Do come again!");
				outline_text_shadow_center(vgolf_font[VGOLF_FONT_32], x, t3f_virtual_display_height - 100, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), buf);
			}
			break;
		default:
			break;
	}
	al_hold_bitmap_drawing(false);
}

//draw the player podium and trophy stuff
void victory_screen_player_podium_display(ALLEGRO_BITMAP * bp, int x, int y, VGOLF_COURSE * cr, MAIN_GAME_VARS * gvars, PLAYER pl, int plno, int place, int spot)
{
	int dy = 0, dx = 0;
//	char * ptext[3] = {"1st", "2nd", "3rd"};

	//the lower the place, the higher the podium sits
	if(vgolf_player_count == 1)
	{
		dy = (gvars->vict_pody / (vgolf_player_count + 1)) * spot;
		//podium sits in the center of the screen in single player mode
		dx = 0;
	}
	else if(vgolf_player_count == 2)
	{
		dy = (gvars->vict_pody / (vgolf_player_count + 1)) * spot;
		//podiums sit side by side centered in the middle of the screen
		if(!spot) dx = -vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2;
		else dx = vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2;
	}
	else if(vgolf_player_count >= 3)
	{
		dy = (gvars->vict_pody / 6) * spot;
		//podiums sit in cross-pattern in the middle of the screen
		if(!spot) dx = -vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2 - 48;
		else if(spot == 1) dx = vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2 + 48;
		else dx = 0;
	}

	//draw place string (1st, 2nd, or 3rd)
//    sprintf(text_buffer, "Strokes: %i", pl->cur_strokes);
	if(place >= 0 && place < 3)
	{
		//draw podium, trophy, and player character image
		t3f_draw_animation(cr->image_bank.victory_podiumpic, t3f_color_white, main_game_vars.tick, x + dx - vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2, gvars->vict_pody + dy, 0, 0);
		t3f_draw_animation(cr->image_bank.victory_trophypic[place], t3f_color_white, main_game_vars.tick, x + dx - vgolf_get_bitmap_width(cr->image_bank.victory_trophypic[0]) / 2, gvars->vict_pody + dy - vgolf_get_bitmap_height(cr->image_bank.victory_trophypic[0]), 0, 0);
		t3f_draw_animation(*pl.pic[1], t3f_color_white, main_game_vars.tick, x + dx - vgolf_get_bitmap_width(*pl.pic[1]) / 2, gvars->vict_pody + dy + 55, 0, 0);

//		outline_text_shadow_center(vgolf_font[VGOLF_FONT_48], x + dx - 3, gvars->vict_pody + dy + 20, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), ptext[place]);
//		outline_text_shadow_center(vgolf_font[VGOLF_FONT_48], x + dx - 3, gvars->vict_pody + dy + 116, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), vgolf_client_game->player[spot]->name);
	}
}

//draw the player podium and trophy stuff
void victory_screen_player_podium_text_display(ALLEGRO_BITMAP * bp, int x, int y, VGOLF_COURSE * cr, MAIN_GAME_VARS * gvars, PLAYER pl, int plno, int place, int spot)
{
	int dy = 0, dx = 0;
	char * ptext[3] = {"1st", "2nd", "3rd"};

	//the lower the place, the higher the podium sits
	if(vgolf_player_count == 1)
	{
		dy = (gvars->vict_pody / (vgolf_player_count + 1)) * spot;
		//podium sits in the center of the screen in single player mode
		dx = 0;
	}
	else if(vgolf_player_count == 2)
	{
		dy = (gvars->vict_pody / (vgolf_player_count + 1)) * spot;
		//podiums sit side by side centered in the middle of the screen
		if(!spot) dx = -vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2;
		else dx = vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2;
	}
	else if(vgolf_player_count >= 3)
	{
		dy = (gvars->vict_pody / 6) * spot;
		//podiums sit in cross-pattern in the middle of the screen
		if(!spot) dx = -vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2 - 48;
		else if(spot == 1) dx = vgolf_get_bitmap_width(cr->image_bank.victory_podiumpic) / 2 + 48;
		else dx = 0;
	}

	//draw place string (1st, 2nd, or 3rd)
//    sprintf(text_buffer, "Strokes: %i", pl->cur_strokes);
	if(place >= 0 && place < 3)
	{
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_48], x + dx - 3, gvars->vict_pody + dy + 20, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), ptext[place]);
		outline_text_shadow_center(vgolf_font[VGOLF_FONT_48], x + dx - 3, gvars->vict_pody + dy + 116, -2, 2, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), vgolf_client_game->player[plno]->name);
	}
}

//draws the victory screen
// fix me: shows players in their player slot instead of their place in the vict_winner[] array
void victory_screen_display(ALLEGRO_BITMAP * bp, VGOLF_COURSE * cr, MAIN_GAME_VARS * gvars, PLAYER pl[], int num_players)
{
	int i;
	int place[VGOLF_MAX_PLAYERS] = {0, 1, 2, 3, 4, 5, 6, 7};
	int spot = 0;

	course_victory_screen_bg_display(bp, cr);            //draw background layer
	course_victory_screen_cloud_display(bp, cr, gvars);  //draw clouds

	switch(vgolf_player_count)
	{
		case 1:    //single player mode..
			victory_screen_player_podium_display(bp, t3f_virtual_display_width/2, t3f_virtual_display_height/2, cr, gvars, pl[gvars->cur_player], 0, 0, 0);
			//draw the text last..
			victory_screen_text_display(bp, t3f_virtual_display_width/2, 50, 0, cr, gvars, 0);
			break;
		case 2:    //multi-player mode..
		case 3:    //multi-player mode..
		case 4:    //multi-player mode..
		case 5:    //multi-player mode..
		case 6:    //multi-player mode..
		case 7:    //multi-player mode..
		case 8:    //multi-player mode..
			//draw victory screen for all players
			if(vgolf_player_course_stroke_total(&pl[gvars->vict_winner[0]], cr->num_holes) == vgolf_player_course_stroke_total(&pl[gvars->vict_winner[1]], cr->num_holes) && vgolf_player_course_stroke_total(&pl[gvars->vict_winner[2]], cr->num_holes))
			{
				place[0] = 0;
				place[1] = 0;
				place[2] = 0;
			}
			else if(vgolf_player_course_stroke_total(&pl[gvars->vict_winner[0]], cr->num_holes) == vgolf_player_course_stroke_total(&pl[gvars->vict_winner[1]], cr->num_holes))
			{
				place[0] = 0;
				place[1] = 0;
				place[2] = 1;
			}
			else if(vgolf_player_course_stroke_total(&pl[gvars->vict_winner[1]], cr->num_holes) == vgolf_player_course_stroke_total(&pl[gvars->vict_winner[2]], cr->num_holes))
			{
				place[0] = 0;
				place[1] = 1;
				place[2] = 1;
			}
			spot = 0;
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(gvars->vict_winner[i] >= 0 && gvars->vict_winner[i] < VGOLF_MAX_PLAYERS && vgolf_player_list[gvars->vict_winner[i]])
				{
					victory_screen_player_podium_display(bp, t3f_virtual_display_width/2, t3f_virtual_display_height/2, cr, gvars, pl[gvars->vict_winner[i]], gvars->vict_winner[i], place[i], spot);
					spot++;
				}
			}
			spot = 0;
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(gvars->vict_winner[i] >= 0 && gvars->vict_winner[i] < VGOLF_MAX_PLAYERS && vgolf_player_list[gvars->vict_winner[i]])
				{
					victory_screen_player_podium_text_display(bp, t3f_virtual_display_width/2, t3f_virtual_display_height/2, cr, gvars, pl[gvars->vict_winner[i]], gvars->vict_winner[i], place[i], spot);
					spot++;
				}
			}
			//draw the text last..
			victory_screen_text_display(bp, t3f_virtual_display_width/2, 100, gvars->vict_winner[0], cr, gvars, (gvars->vict_winner[0] != -1));
			break;
		default:
			break;
	}
	switch(main_game_vars.game_state)
	{
		case GAMESTATE_COURSE_COMPLETED_1P_VICTORY_SEGMID: //victory screen is set in place
		case GAMESTATE_COURSE_COMPLETED_XP_VICTORY_SEGMID:
		{
			if(vgolf_client_game->player[main_game_vars.cur_player]->local)
			{
				if(!fake_page_hidden)
				{
					hyperlink_page_render(fake_game_complete_page, bp);
				}
			}
			else if(!vgolf_client)
			{
				hyperlink_page_render(game_complete_page, bp);
			}
			break;
		}
	}
}
