#include "t3f/t3f.h"
#include "t3f/music.h"
#include "t3f/resource.h"
#include "t3f/debug.h"
#include "t3net/dlc.h"
#include "network/server.h"
#include "globals.h"
#include "core/course.h"
#include "core/legacy_course.h"
#include "network/callbacks.h"
#include "fx.h"
#include "gameplay.h"
#include "config.h"
#include "menu_proc.h"
#include "version.h"
#include "init.h"
#include "dlc.h"
#include "voip.h"
#include "core/resource.h"
#include "core/utility.h"
#include "events.h"
#include "t3_logo.h"

void set_gfx_mode(void)
{
	char buffer[32] = {0};
	ALLEGRO_MONITOR_INFO info;
	int nw, nh;
	int r;

	al_stop_timer(t3f_timer);
	switch(vgolf_config[CONFIG_GFX_MODE])
	{
		case 0:
		{
			t3f_set_gfx_mode(640, 480, 0);
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			if(al_get_display_flags(t3f_display) & (ALLEGRO_FULLSCREEN))
			{
				if(vgolf_state != VGOLF_STATE_LOAD)
				{
					ncdmsg_add(&vgolf_messages, "Must restart for change to take effect!", al_map_rgb(255, 255, 255), 0.0, 300);
				}
			}
			break;
		}
		case 1:
		{
			t3f_set_gfx_mode(1280, 960, 0);
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			if(al_get_display_flags(t3f_display) & (ALLEGRO_FULLSCREEN))
			{
				if(vgolf_state != VGOLF_STATE_LOAD)
				{
					ncdmsg_add(&vgolf_messages, "Must restart for change to take effect!", al_map_rgb(255, 255, 255), 0.0, 300);
				}
			}
			break;
		}
		case 2:
		{
			r = t3f_set_gfx_mode(640, 480, T3F_USE_FULLSCREEN);
			if(r == 2) // fullscreen window not supported
			{
				al_get_monitor_info(0, &info);
				nw = info.x2 - info.x1;
				nh = info.y2 - info.y1;
				al_set_config_value(t3f_config, "T3F", "force_fullscreen", "true");
				sprintf(buffer, "%d", nw);
				al_set_config_value(t3f_config, "T3F", "display_width", buffer);
				sprintf(buffer, "%d", nh);
				al_set_config_value(t3f_config, "T3F", "display_height", buffer);
				if(!(al_get_display_flags(t3f_display) & (ALLEGRO_FULLSCREEN)))
				{
					if(vgolf_state != VGOLF_STATE_LOAD)
					{
						ncdmsg_add(&vgolf_messages, "Must restart for change to take effect!", al_map_rgb(255, 255, 255), 0.0, 300);
					}
				}
			}
			else
			{
				al_set_config_value(t3f_config, "T3F", "force_fullscreen", "true");
			}
			break;
		}
	}
	al_start_timer(t3f_timer);
}

T3F_ANIMATION * vgolf_load_animation(const char * fn)
{
	ALLEGRO_PATH * pp;
	ALLEGRO_PATH * pp_hd;
	T3F_ANIMATION * ap = NULL;

	pp = al_create_path("data/graphics_sd/");
	if(pp)
	{
		pp_hd = al_create_path("data/graphics/");
		if(pp_hd)
		{
			al_set_path_filename(pp, fn);
			al_set_path_filename(pp_hd, fn);

			/* attempt to load HD texture if option is set */
			if(vgolf_config[CONFIG_HD_TEXTURES])
			{
				ap = t3f_load_animation(al_path_cstr(pp_hd, '/'));
			}

			/* if no animation loaded yet (HD load failed or option not set) */
			if(!ap)
			{
				ap = t3f_load_animation(al_path_cstr(pp, '/'));
			}
			al_destroy_path(pp_hd);
		}
		al_destroy_path(pp);
	}
	return ap;
}

bool vgolf_load_bitmap(ALLEGRO_BITMAP ** bp, const char * fn)
{
	ALLEGRO_PATH * pp;
	ALLEGRO_PATH * pp_hd;
	bool ret = false;

	pp = al_create_path("data/graphics_sd/");
	if(pp)
	{
		pp_hd = al_create_path("data/graphics/");
		if(pp_hd)
		{
			al_set_path_filename(pp, fn);
			al_set_path_filename(pp_hd, fn);

			/* attempt to load HD texture if option is set */
			if(vgolf_config[CONFIG_HD_TEXTURES])
			{
				*bp = t3f_load_resource((void **)bp, T3F_RESOURCE_TYPE_BITMAP, al_path_cstr(pp_hd, '/'), 0, 0, 0);
			}

			/* if no animation loaded yet (HD load failed or option not set) */
			if(!(*bp))
			{
				*bp = t3f_load_resource((void **)bp, T3F_RESOURCE_TYPE_BITMAP, al_path_cstr(pp, '/'), 0, 0, 0);
			}
			if(*bp)
			{
				ret = true;
			}
			al_destroy_path(pp_hd);
		}
		al_destroy_path(pp);
	}
	return ret;
}

//loads the main images
int load_main_image_bank(void)
{
	if(vgolf_config[CONFIG_HD_TEXTURES] && al_get_display_option(t3f_display, ALLEGRO_MAX_BITMAP_SIZE) >= 2560)
	{
		vgolf_title_bg = t3f_load_animation("data/graphics/title_bg.t3a");
	}
	else
	{
		vgolf_title_bg = t3f_load_animation("data/graphics_sd/title_bg.t3a");
	}
	if(!vgolf_title_bg)
	{
		return 0;
	}
	vgolf_title_deluxe = vgolf_load_animation("title_deluxe.t3a");
	if(!vgolf_title_deluxe)
	{
		return 0;
	}
	letter[0].bp = vgolf_load_animation("title_v.t3a");
	if(!letter[0].bp)
	{
		return 0;
	}
	letter[1].bp = vgolf_load_animation("title_g.t3a");
	if(!letter[1].bp)
	{
		return 0;
	}
	letter[2].bp = vgolf_load_animation("title_o.t3a");
	if(!letter[2].bp)
	{
		return 0;
	}
	letter[3].bp = vgolf_load_animation("title_l.t3a");
	if(!letter[3].bp)
	{
		return 0;
	}
	letter[4].bp = vgolf_load_animation("title_f.t3a");
	if(!letter[4].bp)
	{
		return 0;
	}
	menu_bg_color = al_get_pixel(vgolf_title_bg->bitmaps->bitmap[0], 0, 0);

	return 1;
}

int load_main_samples(SOUND_BANK * sbank)
{
	sbank->sfx_menu_select = al_load_sample("data/sounds/sfx_m000.wav");
	if(!sbank->sfx_menu_select)
	{
		return 0;
	}
	sbank->sfx_menu_escape = al_load_sample("data/sounds/sfx_m001.wav");
	if(!sbank->sfx_menu_escape)
	{
		return 0;
	}
	sbank->sfx_menu_slider = al_load_sample("data/sounds/sfx_m002.wav");
	if(!sbank->sfx_menu_slider)
	{
		return 0;
	}
	sbank->sfx_menu_itemsel = al_load_sample("data/sounds/sfx_m003.wav");
	if(!sbank->sfx_menu_itemsel)
	{
		return 0;
	}
	sbank->sfx_title_swing = al_load_sample("data/sounds/golfswing.wav");
	if(!sbank->sfx_title_swing)
	{
		return 0;
	}
	sbank->message_sound = al_load_sample("data/sounds/message.wav");
	if(!sbank->message_sound)
	{
		return 0;
	}
	return 1;
}

//destroy images in the main image bank
void destroy_main_image_bank(void)
{
	int i;

	if(vgolf_title_bg)
	{
		t3f_destroy_animation(vgolf_title_bg);
		vgolf_title_bg = NULL;
	}
	if(vgolf_title_deluxe)
	{
		t3f_destroy_animation(vgolf_title_deluxe);
		vgolf_title_deluxe = NULL;
	}
	for(i = 0; i < 5; i++)
	{
		if(letter[i].bp)
		{
			t3f_destroy_animation(letter[i].bp);
			letter[i].bp = NULL;
		}
	}
}

//destroy all the sound effect samples
void destroy_sample_bank(SOUND_BANK * sbank)
{
	al_destroy_sample(sbank->sfx_menu_select);
	al_destroy_sample(sbank->sfx_menu_escape);
	al_destroy_sample(sbank->sfx_menu_slider);
	al_destroy_sample(sbank->sfx_menu_itemsel);
	al_destroy_sample(sbank->sfx_title_swing);
	al_destroy_sample(sbank->message_sound);
}

unsigned long check_sum(const char * fn)
{
	ALLEGRO_FILE * fp;
	unsigned long sum = 0;
	int c;

	if(al_filename_exists(fn))
	{
		fp = al_fopen(fn, "rb");
		if(fp)
		{
			while(!al_feof(fp))
			{
				c = al_fgetc(fp);
				if(c != EOF)
				{
					sum += c;
				}
			}
			al_fclose(fp);
		}
	}
	return sum;
}

unsigned long vgolf_have_checksum(const char * fn)
{
	const char * val;

	val = al_get_config_value(t3f_config, fn, "checksum");
	if(val)
	{
		return atoi(val);
	}
	return 0;
}

time_t vgolf_get_file_time(const char * fn)
{
	ALLEGRO_FS_ENTRY * fe;
	time_t file_time = 0;

	fe = al_create_fs_entry(fn);
	if(fe)
	{
		file_time = al_get_fs_entry_mtime(fe);
		al_destroy_fs_entry(fe);
	}
	return file_time;
}

time_t vgolf_get_stored_file_time(const char * fn)
{
	const char * val;
	time_t file_time = 0;

	val = al_get_config_value(t3f_config, fn, "time");
	if(val)
	{
		file_time = atoi(val);
	}
	return file_time;
}

/* write checksum to config file so we don't have to recalculate it */
void vgolf_set_checksum(const char * fn, unsigned long sum)
{
	char buf[256] = {0};

	snprintf(buf, 256, "%lu", sum);
	al_set_config_value(t3f_config, fn, "checksum", buf);
}

/* write date to config file so we can check for modifications */
void vgolf_set_file_time(const char * fn, long file_time)
{
	char buf[256] = {0};

	snprintf(buf, 256, "%ld", file_time);
	al_set_config_value(t3f_config, fn, "time", buf);
}

T3F_ANIMATION * vgolf_load_course_preview(const char * fn, const ALLEGRO_FILE_INTERFACE * fi)
{
	VGOLF_COURSE * cp;
	T3F_ANIMATION * ap = NULL;

	cp = vgolf_load_course_info(fn, vgolf_config[CONFIG_HD_TEXTURES], fi);
	if(!cp)
	{
		cp = vgolf_load_legacy_course_info(fn, fi);
	}
	if(cp)
	{
		ap = t3f_clone_animation(cp->image_bank.iconpic);
		vgolf_destroy_course(cp);
	}
	return ap;
}

int vgolf_add_course(const char * fn, int attrib, void * param)
{
	VGOLF_COURSE * cp;
	const ALLEGRO_FS_INTERFACE * fs = al_get_fs_interface();
	const ALLEGRO_FILE_INTERFACE * fi = al_get_new_file_interface();
	time_t file_time;

	cp = vgolf_load_course_info(fn, vgolf_config[CONFIG_HD_TEXTURES], fi);
	if(!cp)
	{
		cp = vgolf_load_legacy_course_info(fn, fi);
	}
	if(cp)
	{
		strcpy(vgolf_courses.item[vgolf_courses.items].filename, fn);
		strcpy(vgolf_courses.item[vgolf_courses.items].name, cp->name);
		vgolf_courses.item[vgolf_courses.items].par = cp->course_par;
		vgolf_courses.item[vgolf_courses.items].holes = cp->num_holes;
		vgolf_courses.item[vgolf_courses.items].sum = vgolf_have_checksum(fn);
		file_time = vgolf_get_file_time(fn);
		if(!vgolf_courses.item[vgolf_courses.items].sum || (file_time != vgolf_get_stored_file_time(fn)))
		{
			vgolf_courses.item[vgolf_courses.items].sum = check_sum(fn);
			vgolf_set_checksum(fn, vgolf_courses.item[vgolf_courses.items].sum);
			vgolf_set_file_time(fn, file_time);
		}
		vgolf_courses.item[vgolf_courses.items].fs = fs;
		vgolf_courses.item[vgolf_courses.items].fi = fi;
		vgolf_courses.items++;
		vgolf_destroy_course(cp);
	}
	return 0;
}

int vgolf_add_avatar(const char * fn, int attrib, void * param)
{
	VGOLF_AVATAR * ap;
	time_t file_time;

	ap = vgolf_load_avatar(fn, NULL, al_get_new_file_interface());
	if(ap)
	{
		strncpy(vgolf_avatars.fn[vgolf_avatars.avatars], fn, 1024);
		vgolf_avatars.sum[vgolf_avatars.avatars] = vgolf_have_checksum(fn);
		file_time = vgolf_get_file_time(fn);
		if(!vgolf_avatars.sum[vgolf_avatars.avatars] || (file_time != vgolf_get_stored_file_time(fn)))
		{
			vgolf_avatars.sum[vgolf_avatars.avatars] = check_sum(fn);
			vgolf_set_checksum(fn, vgolf_avatars.sum[vgolf_avatars.avatars]);
			vgolf_set_file_time(fn, file_time);
		}
		vgolf_avatars.fi[vgolf_avatars.avatars] = al_get_new_file_interface();
		vgolf_avatars.avatars++;
		vgolf_destroy_avatar(ap);
	}
	return 0;
}

int vgolf_add_commentary(const char * fn, int attrib, void * param)
{
	VGOLF_COMMENTARY * commentary_info;
	time_t file_time;

	commentary_info = load_vgolf_commentary_info(fn, al_get_new_file_interface());
	if(commentary_info)
	{
		strcpy(vgolf_commentaries.filename[vgolf_commentaries.count], fn);
		strcpy(vgolf_commentaries.name[vgolf_commentaries.count], commentary_info->name);
		strcpy(vgolf_commentaries.author[vgolf_commentaries.count], commentary_info->author);
		strcpy(vgolf_commentaries.comment[vgolf_commentaries.count], commentary_info->comment);
		vgolf_commentaries.sum[vgolf_commentaries.count] = vgolf_have_checksum(fn);
		file_time = vgolf_get_file_time(fn);
		if(!vgolf_commentaries.sum[vgolf_commentaries.count] || (file_time != vgolf_get_stored_file_time(fn)))
		{
			vgolf_commentaries.sum[vgolf_commentaries.count] = check_sum(fn);
			vgolf_set_checksum(fn, vgolf_commentaries.sum[vgolf_commentaries.count]);
			vgolf_set_file_time(fn, file_time);
		}
		vgolf_commentaries.fi[vgolf_commentaries.count] = al_get_new_file_interface();
		vgolf_commentaries.count++;
		destroy_vgolf_commentary(commentary_info);
	}
	return 0;
}

bool vgolf_setup_joynet(void)
{
	int i;

	if(!joynet_init())
	{
		return false;
	}
	vgolf_client_game = joynet_create_game(VGOLF_SERVER_VERSION_STRING, JOYNET_GAME_TYPE_MOUSE, VGOLF_MAX_PLAYERS, 4, vgolf_game_channel_callback);
	if(!vgolf_client_game)
	{
		return false;
	}
	joynet_setup_game_controllers(vgolf_client_game, 1, 3, 256);
	joynet_add_game_option(vgolf_client_game, (int *)(&vgolf_seed));
	joynet_add_game_option(vgolf_client_game, (int *)(&vgolf_selected_course));
	joynet_add_game_option(vgolf_client_game, &vgolf_low_bandwidth_mode);
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		joynet_add_player_option(vgolf_client_game, i, &game_player[i].setup_step);
	}

	return true;
}

void set_up_credits(void)
{
	vgolf_credits.items = 0;
	credits_add_item(&vgolf_credits, "Game Design", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 0, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 1, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Programming", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 8, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 9, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 10, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Game Engine", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 17, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 18, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Course Design", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 25, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 26, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Interface", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 33, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 34, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Graphics & Art", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 41, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 42, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Additional Graphics", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 49, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 50, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Music", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 57, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 58, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 59, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Sound", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 66, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 67, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 68, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Commentary", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 75, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 76, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Web Support", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 83, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Grady O'Connell", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 84, al_map_rgb(255, 255, 255));

	credits_add_item(&vgolf_credits, "Testing & Feedback", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 91, al_map_rgb(0, 255, 0));
	credits_add_item(&vgolf_credits, "Todd Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 92, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Travis Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 93, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Brandy Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 94, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Grady O'Connell", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 95, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Mark Oates", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 96, al_map_rgb(255, 255, 255));
	credits_add_item(&vgolf_credits, "Michael Cope", &vgolf_font[VGOLF_FONT_48], t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) * 97, al_map_rgb(255, 255, 255));

	vgolf_credits.images = 0;
	credits_add_image(&vgolf_credits, "data/graphics/credits1.pcx", t3f_virtual_display_width / 4, 240);
	credits_add_image(&vgolf_credits, "data/graphics/credits2.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 1);
	credits_add_image(&vgolf_credits, "data/graphics/credits3.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 2);
	credits_add_image(&vgolf_credits, "data/graphics/credits4.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 3);
	credits_add_image(&vgolf_credits, "data/graphics/credits5.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 4);
	credits_add_image(&vgolf_credits, "data/graphics/credits6.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 5);
	credits_add_image(&vgolf_credits, "data/graphics/credits7.pcx", t3f_virtual_display_width / 4, 240 - CREDITS_SPACE * 6);
	credits_add_image(&vgolf_credits, "data/graphics/credits8.pcx", 0, 240 - CREDITS_SPACE * 7 - 160);
}

/* prepare directories to store DLC */
void vgolf_setup_directories(void)
{
	ALLEGRO_PATH * temp_path = NULL;

	temp_path = al_clone_path(t3f_data_path);
	al_append_path_component(temp_path, "avatars");
	t3f_setup_directories(temp_path);
	al_destroy_path(temp_path);

	temp_path = al_clone_path(t3f_data_path);
	al_append_path_component(temp_path, "courses");
	t3f_setup_directories(temp_path);
	al_destroy_path(temp_path);

	temp_path = al_clone_path(t3f_data_path);
	al_append_path_component(temp_path, "commentaries");
	t3f_setup_directories(temp_path);
	al_destroy_path(temp_path);

	temp_path = al_clone_path(t3f_data_path);
	al_append_path_component(temp_path, "replays");
	t3f_setup_directories(temp_path);
	al_destroy_path(temp_path);

	temp_path = al_clone_path(t3f_data_path);
	al_append_path_component(temp_path, "chatlogs");
	t3f_setup_directories(temp_path);
	al_destroy_path(temp_path);
}

void vgolf_build_course_database(void)
{
	const ALLEGRO_FS_INTERFACE * old_fs;
	const ALLEGRO_FILE_INTERFACE * old_fi;
	char buffer[1024];

	vgolf_courses.items = 0;
	for_each_file_ex("data/courses/*.vdc", 0, 0, vgolf_add_course, 0);
	for_each_file_ex("data/courses/*.crs", 0, 0, vgolf_add_course, 0);
	old_fs = al_get_fs_interface();
	old_fi = al_get_new_file_interface();
	al_set_standard_fs_interface();
	al_set_standard_file_interface();
	for_each_file_ex(t3f_get_filename(t3f_data_path, "courses/*.vdc", buffer, 1024), 0, 0, vgolf_add_course, 0);
	for_each_file_ex(t3f_get_filename(t3f_data_path, "courses/*.crs", buffer, 1024), 0, 0, vgolf_add_course, 0);
	al_set_fs_interface(old_fs);
	al_set_new_file_interface(old_fi);
}

void vgolf_build_avatar_database(void)
{
	const ALLEGRO_FS_INTERFACE * old_fs;
	const ALLEGRO_FILE_INTERFACE * old_fi;
	char buffer[1024];

	vgolf_avatars.avatars = 0;
	vgolf_add_avatar("data/graphics/phantom.pcx", 0, NULL);
	for_each_file_ex("data/avatars/*.png", 0, 0, vgolf_add_avatar, 0);
	for_each_file_ex("data/avatars/*.pcx", 0, 0, vgolf_add_avatar, 0);
	old_fs = al_get_fs_interface();
	old_fi = al_get_new_file_interface();
	al_set_standard_fs_interface();
	al_set_standard_file_interface();
	for_each_file_ex(t3f_get_filename(t3f_data_path, "avatars/*.png", buffer, 1024), 0, 0, vgolf_add_avatar, 0);
	for_each_file_ex(t3f_get_filename(t3f_data_path, "avatars/*.pcx", buffer, 1024), 0, 0, vgolf_add_avatar, 0);
	al_set_fs_interface(old_fs);
	al_set_new_file_interface(old_fi);
}

void vgolf_build_commentary_database(void)
{
	const ALLEGRO_FS_INTERFACE * old_fs;
	const ALLEGRO_FILE_INTERFACE * old_fi;
	char buffer[1024];

	vgolf_commentaries.count = 0;
	for_each_file_ex("data/commentaries/*.vc", 0, 0, vgolf_add_commentary, 0);
	old_fs = al_get_fs_interface();
	old_fi = al_get_new_file_interface();
	al_set_standard_fs_interface();
	al_set_standard_file_interface();
	for_each_file_ex(t3f_get_filename(t3f_data_path, "commentaries/*.vc", buffer, 1024), 0, 0, vgolf_add_commentary, 0);
	al_set_fs_interface(old_fs);
	al_set_new_file_interface(old_fi);
}

void vgolf_reset_content_lists(void)
{
	int i;

	joynet_reset_game_content(vgolf_client_game, VGOLF_CONTENT_COURSES);
	for(i = 0; i < vgolf_courses.items; i++)
	{
		joynet_add_game_content(vgolf_client_game, VGOLF_CONTENT_COURSES, vgolf_courses.item[i].sum);
	}
	joynet_reset_game_content(vgolf_client_game, VGOLF_CONTENT_AVATARS);
	for(i = 0; i < vgolf_avatars.avatars; i++)
	{
		joynet_add_game_content(vgolf_client_game, VGOLF_CONTENT_AVATARS, vgolf_avatars.sum[i]);
	}
}

void vgolf_free_course_database(void)
{
/*	int i;

	for(i = 0; i < vgolf_courses.items; i++)
	{
		if(vgolf_courses.item[i].preview)
		{
			t3f_destroy_animation(vgolf_courses.item[i].preview);
		}
	} */
}

void vgolf_free_avatar_database(void)
{
}

int vgolf_init(int argc, char * argv[])
{
	int i, r;
	char dlc_path[1024] = {0};
	char buffer[1024];

	if(!t3f_initialize(T3F_APP_TITLE, 640, 480, 60.0, vgolf_logic, vgolf_render, T3F_DEFAULT | T3F_USE_MOUSE | T3F_USE_TOUCH | T3F_FORCE_ASPECT, NULL))
	{
		printf("Initialization failed!\n");
		return 0;
	}
	vgolf_state = VGOLF_STATE_NONE;
	if(al_filename_exists("data/courses/seabreeze_straits.vdc"))
	{
		vgolf_demo = false;
	}
	vgolf_messages.messages = 0;
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		memset(&game_player[i], 0, sizeof(PLAYER));
	}
	vgolf_setup_directories();
	if(!vgolf_setup_joynet())
	{
		printf("Unable to setup JoyNet!\n");
		return 0;
	}
	//try loading the configuration file..
	if(!load_config("vgolf.cfg"))
	{
		//if none found, use the default configuration settings
		set_default_config();
		if(vgolf_config[CONFIG_GFX_MODE] == 1)
		{
			set_gfx_mode();
		}
	}
	if(vgolf_demo)
	{
		vgolf_config[CONFIG_DLC] = 0; // don't allow auto DLC in demo version
	}
	vgolf_register_resource_loaders();

	/* make sure we don't have uninitialize variables */
	for(i = 0; i < 5; i++)
	{
		letter[i].bp = NULL;
	}

	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "", "", "");
//	vgolf_font[VGOLF_FONT_96] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_96], T3F_RESOURCE_TYPE_T3F_FONT_LOAD, "data/fonts/sign_painters_48.png", 0, 0, 0);
	vgolf_font[VGOLF_FONT_96] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_96], T3F_RESOURCE_TYPE_T3F_FONT_GEN, "data/fonts/Signpgsc.ttf", 96, T3F_FONT_OUTLINE, 0);
	if(!vgolf_font[VGOLF_FONT_96])
	{
		printf("Error loading font!\n");
		return 0;
	}
	vgolf_font[VGOLF_FONT_96]->scale = 0.5;
//	vgolf_font[VGOLF_FONT_64] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_64], T3F_RESOURCE_TYPE_T3F_FONT_LOAD, "data/fonts/sign_painters_32.png", 0, 0, 0);
	vgolf_font[VGOLF_FONT_64] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_64], T3F_RESOURCE_TYPE_T3F_FONT_GEN, "data/fonts/Signpgsc.ttf", 64, T3F_FONT_OUTLINE, 0);
	if(!vgolf_font[VGOLF_FONT_64])
	{
		printf("Error loading font!\n");
		return 0;
	}
	vgolf_font[VGOLF_FONT_64]->scale = 0.5;
//	vgolf_font[VGOLF_FONT_48] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_48], T3F_RESOURCE_TYPE_T3F_FONT_LOAD, "data/fonts/sign_painters_24.png", 0, 0, 0);
	vgolf_font[VGOLF_FONT_48] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_48], T3F_RESOURCE_TYPE_T3F_FONT_GEN, "data/fonts/Signpgsc.ttf", 48, T3F_FONT_OUTLINE, 0);
	if(!vgolf_font[VGOLF_FONT_48])
	{
		printf("Error loading font!\n");
		return 0;
	}
	vgolf_font[VGOLF_FONT_48]->scale = 0.5;
	vgolf_font[VGOLF_FONT_32] = t3f_load_resource((void **)&vgolf_font[VGOLF_FONT_32], T3F_RESOURCE_TYPE_T3F_FONT_GEN, "data/fonts/Signpgsc.ttf", 32, T3F_FONT_OUTLINE, 0);
	if(!vgolf_font[VGOLF_FONT_32])
	{
		printf("Error loading font!\n");
		return 0;
	}
	vgolf_font[VGOLF_FONT_32]->scale = 0.5;

	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Graphics", "", "");
	//load the main image bank up
	/* load images */
//	menu_bg_color = al_map_rgba_f(0.0, 0.0, 0.5, 1.0);

	/* copy this to where the credits image is loaded during credits logic
	al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP | ALLEGRO_STATE_TRANSFORM);
	al_set_target_bitmap(thanks_pic);
	al_identity_transform(&identity);
	al_use_transform(&identity);
	al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
	al_draw_bitmap(credit_image[7], al_get_bitmap_width(thanks_pic) / 2 - al_get_bitmap_width(credit_image[7]) / 2, al_get_bitmap_height(thanks_pic) / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2 - al_get_bitmap_height(credit_image[7]) / 2, 0);
	int epx = 320 - t3f_get_text_width(vgolf_font[VGOLF_FONT_96], "Thank You For Playing") / 2;
	int epy = al_get_bitmap_height(thanks_pic) / 2 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_96]) / 2 - al_get_bitmap_height(credit_image[7]) / 2 + al_get_bitmap_height(credit_image[7]);
	outline_text_shadow(vgolf_font[VGOLF_FONT_96], epx + 2, epy - 2, -4, 4, al_map_rgb(255, 255, 255), al_map_rgb(0, 0, 0), al_map_rgb(0, 0, 0), 0.0, "Thank You For Playing");
	al_restore_state(&old_state); */
	scorecard_pic = t3f_load_resource((void **)&scorecard_pic, T3F_RESOURCE_TYPE_BITMAP, "data/graphics/scorecard_strip.png", 0, 0, 0);

	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Sounds", "", "");
	//load the main sample bank up
	if(!load_main_samples(&main_game_vars.sound_bank))
	{
		printf("Unable to load samples %s!\n", fname_buffer);
		return 0;
	}

	vgolf_chat_width = 640.0 - t3f_get_text_width(vgolf_font[VGOLF_FONT_CHAT_OVERLAY], " <");

	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Course Info", "", "");
	vgolf_build_course_database();
	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Avatars", "", "");
	vgolf_build_avatar_database();
	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Commentary Info", "", "");
	vgolf_build_commentary_database();
	if(vgolf_config[CONFIG_COMMENTARY] >= vgolf_commentaries.count)
	{
		vgolf_config[CONFIG_COMMENTARY] = 0;
	}

	/* must load profiles after course database has been built since it has to
	 * match checksums to keep course-related info */
	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Loading Profiles", "", "");
	if(!load_profiles(&vgolf_profiles, "vgolf.pdt"))
	{
		memset(&vgolf_profiles, 0, sizeof(PROFILE_LIST));
		strcpy(vgolf_profiles.item[0].name, "Guest");
		for(i = 0; i < vgolf_courses.items; i++)
		{
			vgolf_profiles.item[0].course_info[i].id = vgolf_courses.item[i].sum;
		}
		vgolf_profiles.items = 1;
	}

	/* download new content if Download DLC option set to Auto */
	if(vgolf_config[CONFIG_DLC])
	{
		vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Checking for DLC Courses", "", "");
		vgolf_dlc_courses = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 1);
		if(vgolf_dlc_courses)
		{
			/* remove items we already have */
			for(i = 0; i < vgolf_courses.items; i++)
			{
				t3net_remove_dlc_item(vgolf_dlc_courses, vgolf_courses.item[i].sum);
			}

			if(vgolf_dlc_courses->items > 0)
			{
				strcpy(dlc_path, t3f_get_filename(t3f_data_path, "courses/", buffer, 1024));
				for(i = 0; i < vgolf_dlc_courses->items; i++)
				{
					for(r = 0; r < 3; r++)
					{
						if(vgolf_download_dlc(vgolf_dlc_courses, i, dlc_path))
						{
							break;
						}
					}
				}
				vgolf_free_course_database();
				vgolf_build_course_database();
			}
		}

		vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Checking for DLC Avatars", "", "");
		vgolf_dlc_avatars = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 0);
		/* remove items we already have */
		if(vgolf_dlc_avatars)
		{
			for(i = 0; i < vgolf_avatars.avatars; i++)
			{
				t3net_remove_dlc_item(vgolf_dlc_avatars, vgolf_avatars.sum[i]);
			}

			if(vgolf_dlc_avatars->items > 0)
			{
				strcpy(dlc_path, t3f_get_filename(t3f_data_path, "avatars/", buffer, 1024));
				for(i = 0; i < vgolf_dlc_avatars->items; i++)
				{
					for(r = 0; r < 3; r++)
					{
						if(vgolf_download_dlc(vgolf_dlc_avatars, i, dlc_path))
						{
							break;
						}
					}
				}
				vgolf_free_avatar_database();
				vgolf_build_avatar_database();
			}
		}

		vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Checking for DLC Commentaries", "", "");
		vgolf_dlc_commentaries = t3net_get_dlc_list("http://www.t3-i.com/t3net2/dlc/query.php", "vgolf_deluxe", 2);
		/* remove items we already have */
		if(vgolf_dlc_commentaries)
		{
			for(i = 0; i < vgolf_commentaries.count; i++)
			{
				t3net_remove_dlc_item(vgolf_dlc_commentaries, vgolf_commentaries.sum[i]);
			}

			if(vgolf_dlc_commentaries->items > 0)
			{
				strcpy(dlc_path, t3f_get_filename(t3f_data_path, "commentaries/", buffer, 1024));
				for(i = 0; i < vgolf_dlc_commentaries->items; i++)
				{
					for(r = 0; r < 3; r++)
					{
						if(vgolf_download_dlc(vgolf_dlc_commentaries, i, dlc_path))
						{
							break;
						}
					}
				}
				vgolf_build_commentary_database();
			}
		}
	}
	vgolf_reset_content_lists();

	vgolf_show_load_screen(al_map_rgba_f(0.0, 0.0, 0.0, 0.75), "Finalizing", "", "");
	srand(time(0));
	fx_particle_system_init();
	//set the main game variables up
	vgolf_set_main_game_vars();

	vgolf_set_up_menus();
	set_up_credits();
	vgolf_color_black = al_map_rgb(0, 0, 0);
	vgolf_color_white = al_map_rgb(255, 255, 255);

	/* curl on OpenPandora doesn't fail when the network is not available so it hangs,
	 * disable uploading by default, new users will probably go straight to the game,
	 * if they later enable uploading and experience hanging they are likely to know
	 * where the problem is */
	#ifdef PANDORA
		vgolf_config[CONFIG_UPLOAD] = 0;
	#endif
	hyperlink_link_mouse(&vgolf_mouse_x, &vgolf_mouse_y, &vgolf_mouse_z, &vgolf_mouse_b);

/*	dlc = t3net_get_dlc_list("http://www.t3-i.com/dlc/query.php", "vgolf_deluxe", 0, vgolf_dlc_preview_callback, t3f_get_filename(t3f_temp_path, "vgd_temp.png"));
	if(dlc)
	{
		t3net_destroy_dlc_list(dlc, vgolf_dlc_preview_destroy_callback);
	} */
	vgolf_server_set_message_callback(vgolf_message_callback);
	vgolf_set_course_load_callback(vgolf_show_load_screen);
	t3f_set_event_handler(vgolf_event_handler);
	al_stop_timer(t3f_timer);
	if(!load_main_image_bank())
	{
		printf("Unable to load images %s!\n", fname_buffer);
		exit(0);
	}
	al_start_timer(t3f_timer);
	reset_letters();
	vgolf_state = VGOLF_STATE_MENU;
	t3f_play_music("data/music/menu.xm");
	fx_fade_start(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 2, NULL);
	return 1;
}

void vgolf_exit(void)
{
	int i;

	t3f_stop_music();

	/* save data */
	save_profiles(&vgolf_profiles, "vgolf.pdt");
	save_config("vgolf.cfg");

	vgolf_close_chat_log();

	if(vgolf_client)
	{
		joynet_disconnect_client_from_server(vgolf_client);
		joynet_destroy_client(vgolf_client);
	}
	joynet_destroy_game(vgolf_client_game);
	if(vgolf_hosting)
	{
		vgolf_close_server();
		vgolf_hosting = false;
	}

	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(game_player[i].avatar)
		{
			vgolf_destroy_avatar(game_player[i].avatar);
		}
	}
	/* free resources */
	if(vgolf_state == VGOLF_STATE_INTRO)
	{
		t3_logo_finish();
	}
	vgolf_free_course_database();
	vgolf_free_avatar_database();
	t3f_destroy_resource(vgolf_font[VGOLF_FONT_96]);
	t3f_destroy_resource(vgolf_font[VGOLF_FONT_64]);
	t3f_destroy_resource(vgolf_font[VGOLF_FONT_48]);
	t3f_destroy_resource(vgolf_font[VGOLF_FONT_32]);
	t3f_destroy_resource(scorecard_pic);
	vgolf_destroy_menus();

	if(game_course)
	{
		vgolf_destroy_game_graphics(game_course);
		vgolf_destroy_course(game_course);
	}

	destroy_main_image_bank();
	destroy_sample_bank(&main_game_vars.sound_bank);
	if(main_game_vars.commentary)
	{
		destroy_vgolf_commentary(main_game_vars.commentary);
	}

	if(leaderboard)
	{
		t3net_destroy_leaderboard(leaderboard);
	}
	vgolf_destroy_dlc_lists();
}

int vgolf_load_screen_tick = 0;

void vgolf_show_load_screen(ALLEGRO_COLOR color, const char * text, const char * progress, const char * space)
{
	int s;
	int w;
	char final_text[256] = {0};

	t3f_process_events(true);
	t3f_render(false);
	al_draw_filled_rectangle(0.0, 0.0, 640.0, 480.0, color);
	if(vgolf_font[VGOLF_FONT_48])
	{
		s = 3;
		sprintf(final_text, "%s", text);
		strcat(final_text, progress);
		w = t3f_get_text_width(vgolf_font[VGOLF_FONT_48], text) + t3f_get_text_width(vgolf_font[VGOLF_FONT_48], space);
		outline_text_shadow(vgolf_font[VGOLF_FONT_48], 320.0 - w / 2 + s, 240.0 - t3f_get_font_line_height(vgolf_font[VGOLF_FONT_48]) / 2 - s, -s, s, VGOLF_COLOR_MENU_MAIN_DARK, t3f_color_white, t3f_color_black, 0, final_text);
	}
	al_flip_display();
	vgolf_load_screen_tick++;
}
