#include <allegro5/allegro.h>
#include "../t3f/t3f.h"
#include "../joynet/joynet.h"
#include "../joynet/game.h"
#include "../joynet/serialize.h"
#include "../joynet/chat.h"
#include "../t3net/t3net.h"
#include "../t3net/server_list.h"
#include "../version.h"
#include "../defines.h"

static ALLEGRO_THREAD * vgolf_server_thread = NULL;
static ALLEGRO_EVENT_SOURCE vgolf_server_poll_event_source;
static char * vgolf_server_key = NULL;
static char vgolf_server_id[256] = {0};
static int vgolf_server_port = 0;
static bool vgolf_server_dedicated = false;
static char vgolf_server_capacity[64] = {0};
static ALLEGRO_THREAD * vgolf_server_poll_thread = NULL;
static unsigned long vgolf_server_ticker = 0;
static JOYNET_GAME * vgolf_server_game = NULL;
static JOYNET_SERVER * vgolf_server = NULL;
static int vgolf_server_games_played = 0;

/* polling */
static unsigned long vgolf_server_total_ticks = 0;
static unsigned long vgolf_server_poll_tick = 0;
static unsigned long vgolf_server_failed_poll_tick = 0;
static ALLEGRO_EVENT vgolf_server_poll_event;

static void (*vgolf_server_message_callback)(char * message, int option) = NULL;

static void vgolf_update_server(void)
{
	int i;
	bool waiting = false;

	if(vgolf_server_game->state == JOYNET_GAME_STATE_OFF)
	{
		for(i = 0; i < 32; i++)
		{
			if(vgolf_server->client[i]->spectating)
			{
				waiting = true;
				break;
			}
		}
	}
	sprintf(vgolf_server_capacity, "%d/%d%s", vgolf_server_game->player_count, vgolf_server_game->players, waiting ? " - Waiting" : "");
	vgolf_server_poll_event.any.type = 1024;
	al_emit_user_event(&vgolf_server_poll_event_source, &vgolf_server_poll_event, NULL);
}

void vgolf_server_set_message_callback(void (*callback)(char * message, int option))
{
	vgolf_server_message_callback = callback;
}

int vgolf_server_callback(ENetEvent * ep)
{
	char message[1024] = {0};

	switch(ep->type)
	{
		case ENET_EVENT_TYPE_DISCONNECT:
		{
			int client;
			if(joynet_current_server_game)
			{
				client = joynet_get_client_from_peer(joynet_current_server_game->server, ep->peer);
				sprintf(message, "Client %d disconnected.", client);
				if(vgolf_server_message_callback)
				{
					vgolf_server_message_callback(message, 300);
				}
				vgolf_update_server();
			}
			break;
		}
		case ENET_EVENT_TYPE_CONNECT:
		{
			int client;
			if(joynet_current_server_game)
			{
				client = joynet_get_client_from_peer(joynet_current_server_game->server, ep->peer);
				sprintf(message, "Client %d connected.", client);
				if(vgolf_server_message_callback)
				{
					vgolf_server_message_callback(message, 300);
				}
			}
			break;
		}
		default:
		{
			break;
		}
	}
	return 1;
}

int vgolf_server_game_channel_callback(JOYNET_MESSAGE * mp)
{
	char message[1024] = {0};
	int i;

	switch(mp->type)
	{
		case JOYNET_GAME_MESSAGE_CHECK_ID:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			short length;
			char id[128];

			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &length);
			joynet_read(vgolf_server->serial_data, id, length);

			sprintf(message, "Client %d ID: %s", client, id);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_CONNECT:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			short player;
			short controller;

			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &controller);
			joynet_getw(vgolf_server->serial_data, &player);

			sprintf(message, "Client %d connecting controller %d to slot %d.", client, controller, player);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_DISCONNECT:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			short player;
			short controller;

			/* disconnect the controller */
			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &controller);
			joynet_getw(vgolf_server->serial_data, &player);

			sprintf(message, "Client %d disconnecting controller %d.", client, controller);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			vgolf_update_server();
			break;
		}

		/* alert players that someone new has connected */
		case JOYNET_GAME_MESSAGE_SPECTATE:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			ENetPacket * packet;
			char string[1024];
			int i, c;
			int first = 1;

			/* send generic welcome message to all clients except for originating client */
			sprintf(string, "Welcome, %s.", vgolf_server->client[client]->screen_name);
			packet = joynet_create_chat_packet(vgolf_server->serial_data, "System", string);
			for(i = 0; i < vgolf_server->max_clients; i++)
			{
				if(i != client && vgolf_server->client[i]->peer)
				{
					enet_peer_send(vgolf_server->client[i]->peer, JOYNET_CHANNEL_CHAT, packet);
				}
			}

			/* send originating client special message */
			c = 0;
			for(i = 0; i < vgolf_server->max_clients; i++)
			{
				if(i != client && vgolf_server->client[i]->peer)
				{
					c++;
				}
			}
			if(c > 0)
			{
				sprintf(string, "%s joins up with ", vgolf_server->client[client]->screen_name);
				for(i = 0; i < vgolf_server->max_clients; i++)
				{
					if(i != client && vgolf_server->client[i]->peer)
					{
						if(!first)
						{
							strcat(string, ", ");
						}
						strcat(string, vgolf_server->client[i]->screen_name);
						first = 0;
					}
				}
				strcat(string, ".");
				packet = joynet_create_chat_packet(vgolf_server->serial_data, "System", string);
				enet_peer_send(vgolf_server->client[client]->peer, JOYNET_CHANNEL_CHAT, packet);
			}
			sprintf(message, "Client %d spectating.", client);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_ADD_PLAYER:
		{
			vgolf_update_server();
			break;
		}
		case JOYNET_GAME_MESSAGE_REMOVE_PLAYER:
		{
			vgolf_update_server();
			break;
		}
		case JOYNET_GAME_MESSAGE_UPDATE_PLAYER_OPTIONS:
		{
			short player;

			/* store options */
			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &player);
			sprintf(message, "Player %d updating options.", player);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_UPDATE_OPTIONS:
		{
			sprintf(message, "Updating game options.");
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_LEAVE:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			sprintf(message, "Client %d leaving game.", client);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_UPLOAD_PLAYER_CONTENT:
		{
			short player;
			short list;
			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &player);
			joynet_getw(vgolf_server->serial_data, &list);
			sprintf(message, "Player %d uploading content list %d.", player, list);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_SELECT_PLAYER_CONTENT:
		{
			short player;
			short list;
			unsigned long hash;

			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &player);
			joynet_getw(vgolf_server->serial_data, &list);
			joynet_getl(vgolf_server->serial_data, (int32_t *)(&hash));

			sprintf(message, "Player %d selecting list %d content.", player, list);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_START:
		{
			vgolf_server_games_played++;
			sprintf(message, "Starting game.");
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			vgolf_update_server();
			break;
		}
		case JOYNET_GAME_MESSAGE_PAUSE:
		{
			sprintf(message, "Pausing game.");
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_RESUME:
		{
			sprintf(message, "Resuming game.");
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			break;
		}
		case JOYNET_GAME_MESSAGE_END:
		{
			sprintf(message, "Ending game.");
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 200);
			}
			vgolf_update_server();
			break;
		}
		case VGOLF_GAME_MESSAGE_WANT_START:
		case VGOLF_GAME_MESSAGE_CANCEL_START:
		case VGOLF_GAME_MESSAGE_STARTING:
		{
			ENetPacket * pp;

			for(i = 0; i < vgolf_server->max_clients; i++)
			{
				if(vgolf_server->client[i]->peer && (vgolf_server->client[i]->playing || vgolf_server->client[i]->spectating))
				{
					pp = joynet_create_packet(mp->type, NULL);
					enet_peer_send(vgolf_server->client[i]->peer, JOYNET_CHANNEL_GAME, pp);
				}
			}
			break;
		}
	}
	return 1;
}

int vgolf_server_chat_channel_callback(JOYNET_MESSAGE * mp)
{
	char message[1024] = {0};

	switch(mp->type)
	{
		case JOYNET_CHAT_MESSAGE_SET_NAME:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);

			sprintf(message, "Client %d set screen name to %s.", client, mp->data);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 300);
			}
			break;
		}
		case JOYNET_CHAT_MESSAGE_SET_GROUP:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);
			short group;

			joynet_serialize(vgolf_server->serial_data, mp->data);
			joynet_getw(vgolf_server->serial_data, &group);

			sprintf(message, "Client %d joined group %d.", client, group);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 300);
			}
			break;
		}
		case JOYNET_CHAT_MESSAGE_TALK:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);

			sprintf(message, "Client %d (%s) announced, \"%s.\"", client, vgolf_server->client[client]->screen_name, mp->data);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 300);
			}
			break;
		}
		case JOYNET_CHAT_MESSAGE_GROUP_TALK:
		{
			int client = joynet_get_client_from_peer(vgolf_server, mp->event->peer);

			sprintf(message, "Client %d (%s) said, \"%s.\"", client, vgolf_server->client[client]->screen_name, mp->data);
			if(vgolf_server_message_callback)
			{
				vgolf_server_message_callback(message, 300);
			}
			break;
		}
	}
	return 0;
}

void * server_poll_thread_proc(ALLEGRO_THREAD * thread, void * arg)
{
	ALLEGRO_EVENT_QUEUE * queue = NULL;
	int ret;
	char message[1024] = {0};

	queue = al_create_event_queue();
	if(!queue)
	{
		return NULL;
	}
	al_register_event_source(queue, &vgolf_server_poll_event_source);
	while(1)
	{
		ALLEGRO_EVENT event;
		if(al_get_thread_should_stop(thread))
		{
			break;
		}
		else
		{
			al_wait_for_event(queue, &event);
			if(event.type == 1024)
			{
				ret = -5000;
				while(ret != 1)
				{
					ret = t3net_update_server("www.t3-i.com/t3net2/master/poll.php", vgolf_server_port, vgolf_server_key, vgolf_server_capacity);
					if(ret != 1)
					{
						if(vgolf_server_failed_poll_tick == 0)
						{
							vgolf_server_failed_poll_tick = vgolf_server_total_ticks;
						}
						sprintf(message, "Server update failed! (%d: %lu - %lu, %lu)", ret, vgolf_server_total_ticks, vgolf_server_poll_tick, vgolf_server_failed_poll_tick);
						if(vgolf_server_message_callback)
						{
							vgolf_server_message_callback(message, 200);
							vgolf_server_message_callback(t3net_server_message, 200);
						}
						al_rest(1.0);
						vgolf_update_server();
					}
					else
					{
						vgolf_server_poll_tick = vgolf_server_total_ticks;
						vgolf_server_failed_poll_tick = 0;
					}
				}
			}
		}
	}
	return NULL;
}

/* run server in its own thread so we don't have to worry about it */
void * vgolf_server_thread_proc(ALLEGRO_THREAD * thread, void * arg)
{
	ALLEGRO_TIMER * timer = NULL;
	ALLEGRO_EVENT_QUEUE * queue = NULL;
//	char message[1024] = {0};
	ALLEGRO_EVENT poll_event;

	/* create server data */
	vgolf_server = joynet_create_server();
	if(!vgolf_server)
	{
		return NULL;
	}
	joynet_set_server_global_callback(vgolf_server, vgolf_server_callback);
	joynet_set_server_channel_callback(vgolf_server, JOYNET_CHANNEL_CHAT, vgolf_server_chat_channel_callback);
	joynet_set_server_channel_callback(vgolf_server, JOYNET_CHANNEL_GAME, vgolf_server_game_channel_callback);
	vgolf_server_game = joynet_create_game(VGOLF_SERVER_VERSION_STRING, JOYNET_GAME_TYPE_MOUSE, VGOLF_MAX_PLAYERS, 4, NULL);
	if(!vgolf_server_game)
	{
		joynet_destroy_server(vgolf_server);
		vgolf_server = NULL;
		return NULL;
	}
	joynet_setup_game_controllers(vgolf_server_game, 1, 3, 128);
	if(!joynet_open_game_server(vgolf_server_game, vgolf_server, vgolf_server_port, 32))
	{
		joynet_destroy_server(vgolf_server);
		vgolf_server = NULL;
		joynet_destroy_game(vgolf_server_game);
		vgolf_server_game = NULL;
		return NULL;
	}
	vgolf_server_key = t3net_register_server("www.t3-i.com/t3net2/master/poll.php", vgolf_server_port, VGOLF_VERSION_NAME, VGOLF_SERVER_VERSION, vgolf_server_id, NULL, vgolf_server_dedicated ? 1 : 0);
	if(!vgolf_server_key)
	{
		if(vgolf_server_message_callback)
		{
			vgolf_server_message_callback("Failed to list server!", 300);
			return NULL;
		}
	}
	vgolf_server_poll_thread = al_create_thread(server_poll_thread_proc, NULL);
	if(!vgolf_server_poll_thread)
	{
		return NULL;
	}
	al_init_user_event_source(&vgolf_server_poll_event_source);
	al_start_thread(vgolf_server_poll_thread);

	/* set up timer and event queue */
	timer = al_create_timer(1.0 / 60.0);
	if(!timer)
	{
		return NULL;
	}
	queue = al_create_event_queue();
	if(!queue)
	{
		return NULL;
	}
//	sprintf(message, "Server is now open.");
//	vgolf_add_message(vgolf_messages, message, vgolf_font[PP2_FONT_SMALL], al_map_rgba_f(0.0, 1.0, 0.0, 1.0), 300, 640, 0.0);
	if(vgolf_server_message_callback)
	{
		vgolf_server_message_callback("Server is now open.", 300);
	}
	al_register_event_source(queue, al_get_timer_event_source(timer));
	vgolf_server_ticker = 0; // set to 0 so we update the capacity column immediately
	al_start_timer(timer);
	while(1)
	{
		ALLEGRO_EVENT event;
		al_wait_for_event(queue, &event);
		if(al_get_thread_should_stop(thread))
		{
			break;
		}
		else // only do this if we are not stopping the thread so we don't update closed servers
		{
			if(event.type == ALLEGRO_EVENT_TIMER)
			{
				if(vgolf_server_ticker == 300)
				{
					vgolf_update_server();
				}
				if(vgolf_server)
				{
					joynet_poll_server(vgolf_server);
				}
				if(vgolf_server_game)
				{
					joynet_update_game_server(vgolf_server, vgolf_server_game); // call once per tick
				}
				/* keep server listed on t3-i.com */
				if(!vgolf_server_dedicated)
				{
					if(vgolf_server_ticker % 3600 == 0)
					{
						vgolf_update_server();
					}
				}
				vgolf_server_ticker++;
			}
		}
	}
	poll_event.any.type = 1025;
	al_emit_user_event(&vgolf_server_poll_event_source, &poll_event, NULL);
	t3net_unregister_server("www.t3-i.com/t3net2/master/poll.php", vgolf_server_port, vgolf_server_key);
	al_destroy_event_queue(queue);
	al_destroy_timer(timer);
	joynet_close_server(vgolf_server);
	joynet_destroy_server(vgolf_server);
	vgolf_server = NULL;
	joynet_destroy_game(vgolf_server_game);
	vgolf_server_game = NULL;
//	sprintf(message, "Server closed.");
//	vgolf_add_message(vgolf_messages, message, vgolf_font[PP2_FONT_SMALL], al_map_rgba_f(0.0, 1.0, 0.0, 1.0), 300, 640, 0.0);
	if(vgolf_server_message_callback)
	{
		vgolf_server_message_callback("Server closed.", 300);
	}
	return NULL;
}

bool vgolf_open_server(char * name, int port, bool dedicated)
{
	bool ret = false;

	vgolf_server_thread = al_create_thread(vgolf_server_thread_proc, NULL);
	if(vgolf_server_thread)
	{
		vgolf_server_games_played = 0;
		strcpy(vgolf_server_id, name);
		vgolf_server_port = port;
		vgolf_server_dedicated = dedicated;
		al_start_thread(vgolf_server_thread);
		ret = true;
	}
	return ret;
}

void vgolf_close_server(void)
{
	al_join_thread(vgolf_server_thread, NULL);
	al_destroy_thread(vgolf_server_thread);
	vgolf_server_thread = NULL;
}

JOYNET_SERVER * vgolf_get_server(void)
{
	return vgolf_server;
}

JOYNET_GAME * vgolf_server_get_game(void)
{
	return vgolf_server_game;
}
