#include "t3f/t3f.h"
#include "globals.h"
#include "gameplay.h"
#include "replay.h"

ALLEGRO_FILE * vgolf_replay_file = NULL;
int vgolf_replay_mode = 0;

bool vgolf_record_replay(const char * fn)
{
	char header[16] = {'V', 'G', 'D', 'R'};
	int i, l;

	vgolf_replay_file = al_fopen(fn, "wb");
	if(!vgolf_replay_file)
	{
		return false;
	}
	header[15] = 2;
	al_fwrite(vgolf_replay_file, header, 16);
	for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
	{
		if(vgolf_player_list[i])
		{
			al_fputc(vgolf_replay_file, 1);
			l = strlen(vgolf_client_game->player[i]->name) + 1;
			al_fputc(vgolf_replay_file, l);
			al_fwrite(vgolf_replay_file, vgolf_client_game->player[i]->name, l);
			al_fwrite32le(vgolf_replay_file, game_player[i].savatar);
		}
		else
		{
			al_fputc(vgolf_replay_file, 0);
		}
	}
	al_fwrite32le(vgolf_replay_file, vgolf_selected_course);
	al_fwrite32le(vgolf_replay_file, vgolf_seed);
	al_fputc(vgolf_replay_file, vgolf_low_bandwidth_mode ? 1 : 0);
	vgolf_replay_mode = VGOLF_REPLAY_MODE_RECORD;
	return true;
}

bool vgolf_play_replay(const char * fn)
{
	char header[16];
	int i, l, a;

	vgolf_replay_file = al_fopen(fn, "rb");
	if(!vgolf_replay_file)
	{
		return false;
	}
	al_fread(vgolf_replay_file, header, 16);
	if(header[0] != 'V' || header[1] != 'G' || header[2] != 'D' || header[3] != 'R')
	{
		al_fclose(vgolf_replay_file);
		vgolf_replay_file = NULL;
		return false;
	}
	switch(header[15])
	{
		case 0:
		{
			for(i = 0; i < 4; i++)
			{
				if(al_fgetc(vgolf_replay_file))
				{
					vgolf_player_list[i] = 1;
					l = al_fgetc(vgolf_replay_file);
					al_fread(vgolf_replay_file, vgolf_client_game->player[i]->name, l);
					game_player[i].savatar = al_fread32le(vgolf_replay_file);
				}
				else
				{
					vgolf_player_list[i] = 0;
				}
			}
			vgolf_selected_course = al_fread32le(vgolf_replay_file);
			vgolf_seed = al_fread32le(vgolf_replay_file);
			vgolf_low_bandwidth_mode = false;
			break;
		}
		case 1:
		{
			for(i = 0; i < 4; i++)
			{
				if(al_fgetc(vgolf_replay_file))
				{
					vgolf_player_list[i] = 1;
					l = al_fgetc(vgolf_replay_file);
					al_fread(vgolf_replay_file, vgolf_client_game->player[i]->name, l);
					game_player[i].savatar = al_fread32le(vgolf_replay_file);
				}
				else
				{
					vgolf_player_list[i] = 0;
				}
			}
			vgolf_selected_course = al_fread32le(vgolf_replay_file);
			vgolf_seed = al_fread32le(vgolf_replay_file);
			vgolf_low_bandwidth_mode = al_fgetc(vgolf_replay_file);
			break;
		}
		case 2:
		{
			for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
			{
				if(al_fgetc(vgolf_replay_file))
				{
					vgolf_player_list[i] = 1;
					l = al_fgetc(vgolf_replay_file);
					al_fread(vgolf_replay_file, vgolf_client_game->player[i]->name, l);
					game_player[i].savatar = al_fread32le(vgolf_replay_file);
					a = vgolf_find_avatar(game_player[i].savatar);
					if(a >= 0)
					{
						if(game_player[i].avatar)
						{
							vgolf_destroy_avatar(game_player[i].avatar);
						}
						game_player[i].avatar = vgolf_load_avatar(vgolf_avatars.fn[a], NULL, vgolf_avatars.fi[a]);
					}
					vgolf_client_game->player[i]->local = true;
				}
				else
				{
					vgolf_player_list[i] = 0;
				}
			}
			vgolf_selected_course = al_fread32le(vgolf_replay_file);
			vgolf_seed = al_fread32le(vgolf_replay_file);
			vgolf_low_bandwidth_mode = al_fgetc(vgolf_replay_file);
			break;
		}
	}
	destroy_main_image_bank();
	vgolf_replay_mode = VGOLF_REPLAY_MODE_PLAY;
	vgolf_start_game();
	return true;
}

void vgolf_stop_replay(void)
{
	int i;

	if(vgolf_replay_file)
	{
		al_fclose(vgolf_replay_file);
		vgolf_replay_file = NULL;
		vgolf_replay_mode = VGOLF_REPLAY_MODE_OFF;
		for(i = 0; i < VGOLF_MAX_PLAYERS; i++)
		{
			vgolf_player_list[i] = 0;
		}
	}
}

void vgolf_write_replay_frame(void)
{
	al_fputc(vgolf_replay_file, vgolf_client_game->player_mouse[0]->b);
	al_fwrite16le(vgolf_replay_file, vgolf_client_game->player_mouse[0]->x);
	al_fwrite16le(vgolf_replay_file, vgolf_client_game->player_mouse[0]->y);
	al_fwrite16le(vgolf_replay_file, vgolf_client_game->player_mouse[0]->z);
}

bool vgolf_read_replay_frame(void)
{
	int c;

	c = al_fgetc(vgolf_replay_file);
	if(al_feof(vgolf_replay_file))
	{
		return false;
	}
	vgolf_client_game->mouse[0]->b = c;
	c = al_fread16le(vgolf_replay_file);
	if(al_feof(vgolf_replay_file))
	{
		return false;
	}
	vgolf_client_game->mouse[0]->x = c;
	c = al_fread16le(vgolf_replay_file);
	if(al_feof(vgolf_replay_file))
	{
		return false;
	}
	vgolf_client_game->mouse[0]->y = c;
	c = al_fread16le(vgolf_replay_file);
	if(al_feof(vgolf_replay_file))
	{
		return false;
	}
	vgolf_client_game->mouse[0]->z = c;
	return true;
}

bool vgolf_replay_done(void)
{
	if(vgolf_client_game->mouse[0]->b == EOF || vgolf_client_game->mouse[0]->x == EOF || vgolf_client_game->mouse[0]->y == EOF || vgolf_client_game->mouse[0]->z == EOF)
	{
		return true;
	}
	return false;
}
