#ifndef VGOLF_STRUCTS_H
#define VGOLF_STRUCTS_H

#include "t3f/t3f.h"
#include "t3f/animation.h"
#include "defines.h"
#include "core/avatar.h"
#include "core/poly.h"
#include "core/body.h"
#include "core/commentary.h"

typedef struct
{

	T3F_ANIMATION * bp;
	float x, y, z;
	float vx, vy, vz;
	int active;
	int state;

} TITLE_LETTER;

typedef struct
{

	char filename[1024];
	char name[64];
	int par;
	int holes;
	unsigned long sum;
	const ALLEGRO_FS_INTERFACE * fs;
	const ALLEGRO_FILE_INTERFACE * fi;

} VGOLF_COURSE_DATABASE_ITEM;

typedef struct
{

	VGOLF_COURSE_DATABASE_ITEM item[VGOLF_MAX_COURSES];
	int items;

} VGOLF_COURSE_DATABASE;

typedef struct
{

//	VGOLF_AVATAR * avatar[VGOLF_MAX_AVATARS];
	char fn[VGOLF_MAX_AVATARS][1024];
	unsigned long sum[VGOLF_MAX_AVATARS];
	const ALLEGRO_FILE_INTERFACE * fi[VGOLF_MAX_AVATARS];
	int avatars;

} VGOLF_AVATAR_DATABASE;

typedef struct
{

	char filename[256][1024];
	char name[256][256];
	char author[256][256];
	char comment[1024][256];
	unsigned long sum[256];
	const ALLEGRO_FILE_INTERFACE * fi[256];
	int count;

} VGOLF_COMMENTARY_DATABASE;

typedef struct
{
//	ALLEGRO_BITMAP * pic[MAX_ANIMDEF_FRAMES];    //images in this animation def
	short speed;              //animation speed
	short num_frames;         //# of frames
	short cur_frame;          //current frame
} ANIM_DEF;

typedef struct
{
	VERTEX s,v,s_old,s_med;   //position, velocity, old position, hit position vectors
	VERTEX f, cf;                 //vector for additional forces
	BODY body;                //ball body (defines poly and collision stuff)
	ANIM_DEF anim_def[MAX_BALL_ANIM_MODES];  //ball animations where applicable
	VGOLF_RECT hit_rect;            //hit rectangle that surrounds the ball circle
//	ALLEGRO_BITMAP * shad_pic;        //shadow image

	float speed;              //current ball speed
	char mode;                //current ball mode
	char visible;             //is ball visible?
	char mirage_visible;      //is the translucent ball visible?
	char in_polymorph;        //is ball inside a polymorph poly?
	int angle;                //ball angle (int)
	float f_angle;            //ball angle (float)
	float radius;             //ball radius
	int cur_vert;             //current vert being interacted with
	int idle_timeout;         //max time to wait before resetting back to idle
	int idle_timer;           //idle timer..
	short mirage_timer;       //timer for displaying translucent ball
	short mirage_timer_end;   //timer end for displaying translucent ball
	int off_conveyor; // tell us when the ball has just rolled of a conveyor belt (ugh, whatever works)
	int conveyor_vert;
	int decel_type;
    int hole_vert;
    int water_vert;
    int lazer_poly;
    char mode2;
    int layer, old_layer;
} BALL;

typedef struct
{
//	char name[32];
	int score[18];
	int par_score[18];
	char score_text[16]; // cache for storing -1, 'E', etc. for overall score
//	char match_won[18];
//	int num_birdies;
//	int num_eagles;
//	int num_albo;
//	int matches_won;
//	int coin_best_combo;
//	long coin_score;
//	short pic_ref;
} SCORECARD;

typedef struct
{
	int x, y;         //hud display coords
//	int w, h;         //hud width and height
} HUD;

typedef struct
{

	char swing;          //is the club being swung?
	int angle;           //the angle the club is facing
	float f_angle;
	int power;           //the hit meter power of the club
	int rot_speed;       //speed of club rotation
	float f_rot_speed;
	bool hidden;

} VGOLF_PLAYER_CLUB_DATA;

typedef struct
{

	SCORECARD scorecard;      //current scorecard (tells total score,# birdies, eagles, etc.)

	BALL ball;                //player ball
	HUD hud;                  //player heads up display

	VGOLF_PLAYER_CLUB_DATA club_data;
	VGOLF_PLAYER_CLUB_DATA local_club_data; // use this to hide lag (updates locally but doesn't affect gameplay)

	int cur_strokes;          //current # of strokes on this hole
	int cur_hole;             //current hole being played (0-17)
	float shot_distance;

	int profile;
	unsigned long savatar;
	unsigned long scourse;
	int setup_step;
//	char name[32];            //player name
	VGOLF_AVATAR * avatar;
	T3F_ANIMATION ** pic[VGOLF_MAX_PLAYERS];    // shortcut to avatar pics
	short pic_mode;           //pic emotion state
	int id;
	int so_close;
	int gimme_offered;
	T3F_ANIMATION * course_preview;

} PLAYER;

typedef struct{
        ALLEGRO_SAMPLE * sfx_swing[MAX_SFX_SWING_SAMPLES];    //club swing sounds
        ALLEGRO_SAMPLE * sfx_ball_collision[MAX_SFX_COLLISION_SAMPLES];    //ball-wall collision sounds
        ALLEGRO_SAMPLE * sfx_ball_holesink;                   //sound of ball hitting cup
        ALLEGRO_SAMPLE * sfx_hole_finish[4][MAX_SFX_HOLE_FINISH_SAMPLES];  //sounds played when hole is completed (0:bad, 1:good, 2:very good)
        ALLEGRO_SAMPLE * sfx_hole_ahhh[MAX_SFX_HOLE_AHHH_SAMPLES];         //crowd "ahhh" sounds for really close moments
        ALLEGRO_SAMPLE * sfx_ball_splash;                     //sound of ball hitting water
        ALLEGRO_SAMPLE * sfx_ball_zapped;                     //sound of ball being zapped by laser
        ALLEGRO_SAMPLE * sfx_ball_teleport;                   //sound of ball teleporting
        ALLEGRO_SAMPLE * sfx_ball_rotate;                     //sound of club rotating
        ALLEGRO_SAMPLE * sfx_menu_select;                     //sound of pressing enter on a menu item
        ALLEGRO_SAMPLE * sfx_menu_escape;                     //sound of esc'ing out of a menu
        ALLEGRO_SAMPLE * sfx_menu_slider;                     //sound of changing a slider's value
        ALLEGRO_SAMPLE * sfx_menu_itemsel;                    //sound of selecting a new menu item
        ALLEGRO_SAMPLE * sfx_coin;                            //sound of gathering a coin
        ALLEGRO_SAMPLE * sfx_ball_enter;                      //sound of gathering a coin
        ALLEGRO_SAMPLE * sfx_ball_exit;                       //sound of gathering a coin
        ALLEGRO_SAMPLE * sfx_title_swing;
        ALLEGRO_SAMPLE * message_sound;
        }SOUND_BANK;

typedef struct
{

	float x, y, z;
	float target_x, target_y, target_z;
	float vx, vy, vz;

} VGOLF_CAMERA;

typedef struct
{

	VGOLF_CAMERA camera;

	float vict_cloud_x[4];  //coordinates for floating clouds
	float vict_cloud_y[4];
	float vict_trophy_x[VGOLF_MAX_PLAYERS]; //coordinates for trophies
	float vict_trophy_y[VGOLF_MAX_PLAYERS];
	char game_mode;         //stroke or match play?
	char hole_finished;     //is hole finished?
	char quit_game;         //exit game?
	char quit_title;        //exit title/frontend?
	char quit_all;          //exit everything?
	char debug_on;          //toggles debug mode
	int game_state;         //current game state
	int game_state_old;     //old game state for restoring
	int cur_vmessage_slot;  //current message queue slot
	int num_players;        //# players currently playing
	int cur_player;         //current player in use
	int cur_hole;           //current hole being played (0-17)
	int vict_pody;          //vertical placement for podiums on victory screen
	int vict_winner[VGOLF_MAX_PLAYERS];     //winner refs
	int cur_statsheet;      //current statsheet to display (used on player setup screen)
	int ingame_timer;       //ingame timer
	int ingame_clicker;
	char ingame_timer_change;
	int player_huds_state;
	int player_huds_offset;
	char got_hiscore;        //tells when a player has gotten a high score (on the victory screen)
	short save_stats;        //save stats to file?
	short num_player_images; //# player character images
	int tick;

	SOUND_BANK sound_bank;  //samples
	VGOLF_COMMENTARY * commentary;    //game quotes/messages

} MAIN_GAME_VARS;

#endif
