#include <curses.h>
#include "t3f/t3f.h"
#include "joynet/joynet.h"
#include "joynet/game.h"
#include "joynet/serialize.h"
#include "t3net/t3net.h"
#include "network/server.h"
#include "defines.h"
#include "version.h"

#define MAX_SERVER_LOGS 13

char * game_state_name[3] =
{
	"Waiting",
	"In Progress",
	"Paused"
};

char server_log[MAX_SERVER_LOGS][256] = {{0}};
int server_logs = 0;
char message[1024] = {0};

/* polling */
unsigned long total_ticks = 0;
unsigned long poll_tick = 0;
unsigned long failed_poll_tick = 0;
ALLEGRO_EVENT poll_event;

int clients;
int players;
int played = 0;
int finished = 0;

/* display stuff */
WINDOW * screen;

void server_add_log(char * text)
{
	int i;

	if(server_logs < MAX_SERVER_LOGS)
	{
		strcpy(server_log[server_logs], text);
		server_logs++;
	}
	else
	{
		for(i = 0; i < MAX_SERVER_LOGS - 1; i++)
		{
			strcpy(server_log[i], server_log[i + 1]);
		}
		strcpy(server_log[MAX_SERVER_LOGS - 1], text);
	}
}

int game_state_okay(int state)
{
	if(state >= 0 && state <= 2)
	{
		return 1;
	}
	return 0;
}

void update_console(void)
{
	int i;
	JOYNET_GAME * gp = vgolf_server_get_game();
	JOYNET_SERVER * sp = vgolf_get_server();

	clear();
	curs_set(0);
	mvwprintw(screen, 0, 0, "Server Status: Online");
	mvwprintw(screen, 1, 0, "Game Status:   %s", gp ? game_state_name[gp->state] : "N/A");
//	mvwprintw(screen, 2, 0, "Clients:       %02d/%02d", sp->clients, sp->max_clients);
	mvwprintw(screen, 3, 0, "Players:       %02d/%02d", gp ? gp->player_count : -1, gp ? gp->players : -1);
//	mvwprintw(screen, 4, 0, "Games Played:  %d", played);

	mvwprintw(screen, 7, 0, "Server Log");
	mvwprintw(screen, 8, 0, "----------");
	for(i = 0; i < server_logs; i++)
	{
		mvwprintw(screen, 9 + i, 0, "%s", server_log[i]);
	}

	mvwprintw(screen, 23, 0, "Press 'Q' to quit.");
	wrefresh(screen);
	refresh();
}

void server_message_callback(char * message, int option)
{
	server_add_log(message);
	update_console();
}

void logic(void * data)
{
	int key = getch();

	if(key == 'q' || key == 'Q')
	{
		vgolf_server_set_message_callback(NULL);
		t3f_exit();
	}
}

int main(int argc, char * argv[])
{
	if(argc < 3)
	{
		printf("Usage: vgolf_server <port> <name of server>\n");
		return 1;
	}
	if(!t3f_initialize("vGolf Deluxe Server", 640, 480, 60.0, logic, NULL, T3F_USE_KEYBOARD | T3F_NO_DISPLAY, NULL))
	{
		return 0;
	}
	if(!joynet_init())
	{
		return 1;
	}
	if(!vgolf_open_server(argv[2], atoi(argv[1]), true))
	{
		return 1;
	}

	/* init console display */
	screen = initscr();
	noecho();
	cbreak();
	nodelay(screen, TRUE);
	refresh();
	wrefresh(screen);

	/* run application */
	update_console();
	vgolf_server_set_message_callback(server_message_callback);
	t3f_run();

	endwin();

	vgolf_close_server();
	joynet_exit();
	return 0;
}
